@@ -424,10 +424,10 @@
 !
 !     here to check the integral of the charge density
 !
-      IF( ( iverbosity > 2 ) .OR. ( nfi == 0 ) .OR. &
+      IF( ( iverbosity > 1 ) .OR. ( nfi == 0 ) .OR. &
           ( MOD(nfi, iprint_stdout) == 0 ) .AND. ( .NOT. tcg ) ) THEN
 
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             CALL checkrho( dfftp%nnr, nspin, rhor, rmin, rmax, rsum, rnegsum )
             rnegsum = rnegsum * omega / DBLE(dfftp%nr1*dfftp%nr2*dfftp%nr3)
             rsum    = rsum    * omega / DBLE(dfftp%nr1*dfftp%nr2*dfftp%nr3)
@@ -446,12 +446,12 @@
       ENDIF
 
 10    FORMAT( /, 3X, 'from rhoofr: total integrated electronic density', &
-            & /, 3X, 'in g-space = ', f11.6, 3x, 'in r-space =', f11.6 )
+            & /, 3X, 'in g-space = ', f13.6, 3x, 'in r-space =', f13.6 )
 20    FORMAT( /, 3X, 'from rhoofr: total integrated electronic density', &
             & /, 3X, 'spin up', &
-            & /, 3X, 'in g-space = ', f11.6, 3x, 'in r-space =', f11.6 , &
+            & /, 3X, 'in g-space = ', f13.6, 3x, 'in r-space =', f13.6 , &
             & /, 3X, 'spin down', &
-            & /, 3X, 'in g-space = ', f11.6, 3x, 'in r-space =', f11.6 )
+            & /, 3X, 'in g-space = ', f13.6, 3x, 'in r-space =', f13.6 )
 !
       CALL stop_clock( 'rhoofr' )
 
@@ -1243,7 +1243,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
                !  qv = US augmentation charge in real space on box grid
                !       for atomic species is, real(qv)=atom ia, imag(qv)=atom ia+1
  
-               IF( iverbosity > 2 ) THEN
+               IF( iverbosity > 1 ) THEN
                   ca = SUM(qv)
                   WRITE( stdout,'(a,f12.8)') ' rhov: 1-atom g-sp = ',         &
      &                 omegab*DBLE(qgbt(1,1))
@@ -1279,7 +1279,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          END DO
 
 !
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             ca = SUM(v)
 
             CALL mp_sum( ca, intra_bgrp_comm )
@@ -1290,7 +1290,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !
          CALL fwfft('Dense',v, dfftp )
 !
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             WRITE( stdout,*) ' rhov: smooth ',omega*rhog(1,iss)
             WRITE( stdout,*) ' rhov: vander ',omega*v(1)
             WRITE( stdout,*) ' rhov: all    ',omega*(rhog(1,iss)+v(1))
@@ -1303,7 +1303,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          END DO
 
 !
-         IF( iverbosity > 2 ) WRITE( stdout,'(a,2f12.8)')                          &
+         IF( iverbosity > 1 ) WRITE( stdout,'(a,2f12.8)')                          &
      &        ' rhov: n_v(g=0) = ',omega*DBLE(rhog(1,iss))
 !
       ELSE
@@ -1353,7 +1353,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !  qv is the now the US augmentation charge for atomic species is
 !  and atom ia: real(qv)=spin up, imag(qv)=spin down
 !
-               IF( iverbosity > 2 ) THEN
+               IF( iverbosity > 1 ) THEN
                   ca = SUM(qv)
                   WRITE( stdout,'(a,f12.8)') ' rhov: up   g-space = ',        &
      &                 omegab*DBLE(qgbt(1,1))
@@ -1378,7 +1378,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
             rhor(ir,isdw)=rhor(ir,isdw)+AIMAG(v(ir)) 
          END DO
 !
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             ca = SUM(v)
             CALL mp_sum( ca, intra_bgrp_comm )
             WRITE( stdout,'(a,2f12.8)') 'rhov:in n_v  ',omega*ca/(dfftp%nr1*dfftp%nr2*dfftp%nr3)
@@ -1386,7 +1386,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
 !
          CALL fwfft('Dense',v, dfftp )
 !
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             WRITE( stdout,*) 'rhov: smooth up',omega*rhog(1,isup)
             WRITE( stdout,*) 'rhov: smooth dw',omega*rhog(1,isdw)
             WRITE( stdout,*) 'rhov: vander up',omega*DBLE(v(1))
@@ -1405,7 +1405,7 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
          END DO
 
 !
-         IF( iverbosity > 2 ) THEN
+         IF( iverbosity > 1 ) THEN
             WRITE( stdout,'(a,2f12.8,/,a,2f12.8)')                 &
      &        ' rhov: n_v(g=0) up   = ',omega*DBLE (rhog(1,isup)), &
      &        ' rhov: n_v(g=0) down = ',omega*DBLE(rhog(1,isdw))
@@ -1450,17 +1450,17 @@ END SUBROUTINE rhov
       !
       filename = TRIM(dirname) // '/' // 'charge-density'
       !
-      CALL read_rho_xml( filename, rhor(:,1), dfftp%nr1, dfftp%nr2, dfftp%nr3, &
+      CALL read_rho_xml( filename, dfftp%nr1, dfftp%nr2, dfftp%nr3, &
                          dfftp%nr1x, dfftp%nr2x, dfftp%ipp, dfftp%npp, &
-                         ionode, intra_bgrp_comm, inter_bgrp_comm )
+                         rhor(:,1) )
       !
       IF( nspin == 2 ) THEN
          !
          filename = TRIM(dirname) // '/' // 'spin-polarization'
          !
-         CALL read_rho_xml( filename, rhor(:,2), dfftp%nr1, dfftp%nr2, dfftp%nr3, &
+         CALL read_rho_xml( filename, dfftp%nr1, dfftp%nr2, dfftp%nr3, &
                             dfftp%nr1x, dfftp%nr2x, dfftp%ipp, dfftp%npp, &
-                            ionode, intra_bgrp_comm, inter_bgrp_comm )
+                            rhor(:,2) )
          !
          !  Convert rho_tot, spin_pol back to rho_up, rho_down
          !
