@@ -49,7 +49,7 @@ MODULE cp_restart
       !------------------------------------------------------------------------
       !
       USE control_flags,            ONLY : gamma_only, force_pairing, trhow, &
-                                           tksw, twfcollect, do_makov_payne
+                                           tksw, twfcollect, do_makov_payne, smallmem
       USE control_flags,            ONLY : tksw, lwfpbe0nscf, lwfnscf ! Lingzhu Kong
       USE io_files,                 ONLY : psfile, pseudo_dir, iunwfc, &
                                            nwordwfc, tmp_dir, diropn
@@ -59,6 +59,8 @@ MODULE cp_restart
                                            my_bgrp_id, intra_bgrp_comm, &
                                            intra_image_comm, inter_bgrp_comm, &
                                            root_bgrp, intra_pool_comm
+      USE mp_global,                ONLY : nproc_pot, nproc_bgrp, nproc_ortho, &
+                                           get_ntask_groups
       USE run_info,                 ONLY : title
       USE gvect,                    ONLY : ngm, ngm_g
       USE gvecs,                    ONLY : ngms_g, ecuts, dual
@@ -72,13 +74,13 @@ MODULE cp_restart
       USE ldaU_cp,                  ONLY : lda_plus_U, ns, ldmx
       USE energies,                 ONLY : enthal, ekin, eht, esr, eself, &
                                            epseu, enl, exc, vave
-      USE mp,                       ONLY : mp_sum
+      USE mp,                       ONLY : mp_sum, mp_barrier
       USE fft_base,                 ONLY : dfftp, dffts, dfftb
-      USE constants,                ONLY : pi
       USE uspp_param,               ONLY : n_atom_wfc
       USE global_version,           ONLY : version_number
       USE cp_main_variables,        ONLY : descla
       USE cp_interfaces,            ONLY : collect_lambda, collect_zmat
+      USE kernel_table,             ONLY : vdw_table_name
       !
       IMPLICIT NONE
       !
@@ -137,7 +139,6 @@ MODULE cp_restart
       INTEGER               :: nk1, nk2, nk3
       INTEGER               :: j, i, iss, ig, nspin_wfc, iss_wfc
       INTEGER               :: is, ia, isa, ik, ierr
-      INTEGER,  ALLOCATABLE :: mill_g(:,:)
       INTEGER,  ALLOCATABLE :: ftmp(:,:)
       INTEGER,  ALLOCATABLE :: ityp(:)
       REAL(DP), ALLOCATABLE :: tau(:,:)
@@ -156,6 +157,7 @@ MODULE cp_restart
       CHARACTER(LEN=256)    :: tmp_dir_save
       LOGICAL               :: exst
       INTEGER               :: inlc
+      INTEGER               :: ntask_groups
       !
       ! ... subroutine body
       !
@@ -257,16 +259,6 @@ MODULE cp_restart
       !
       CALL s_to_r( stau0, tau, na, nsp, h )
       !   
-      ! ... Collect G vectors
-      !   
-      ALLOCATE( mill_g( 3, ngm_g ) )
-      !
-      mill_g = 0
-      !
-      mill_g(:,ig_l2g(1:ngm)) = mill(:,1:ngm)
-      !
-      CALL mp_sum( mill_g, intra_bgrp_comm )
-      !
       lsda = ( nspin == 2 )
       !
       ALLOCATE( ftmp( nbnd_tot , nspin ) )
@@ -364,7 +356,7 @@ MODULE cp_restart
          !
          CALL write_planewaves( ecutwfc, dual, ngw_g, gamma_only, dfftp%nr1, dfftp%nr2, &
                                 dfftp%nr3, ngm_g, dffts%nr1, dffts%nr2, dffts%nr3, ngms_g, dfftb%nr1, &
-                                dfftb%nr2, dfftb%nr3, mill_g, .FALSE. )
+                                dfftb%nr2, dfftb%nr3, mill, .FALSE. )
          !
 !-------------------------------------------------------------------------------
 ! ... SPIN
@@ -378,8 +370,16 @@ MODULE cp_restart
          !
          dft_name = get_dft_name()
          inlc = get_inlc()
-         CALL write_xc( DFT = dft_name, NSP = nsp, LDA_PLUS_U = .FALSE., INLC = inlc )
-
+         !
+         CALL write_xc( DFT = dft_name, NSP = nsp, LDA_PLUS_U = .FALSE., &
+                        INLC = inlc, VDW_TABLE_NAME = vdw_table_name, &
+                        PSEUDO_DIR = pseudo_dir, DIRNAME = dirname)
+         !!! should be replaced by the following: !!!
+         !CALL write_xc( DFT = dft_name, NSP = nsp, LDA_PLUS_U = lda_plus_u,                  &
+         !               LDA_PLUS_U_KIND = lda_plus_u_kind, HUBBARD_LMAX = Hubbard_lmax,      &
+         !               HUBBARD_L = Hubbard_l, HUBBARD_U = Hubbard_U, HUBBARD_J = Hubbard_J, &
+         !               HUBBARD_ALPHA = Hubbard_alpha, INLC = inlc, VDW_TABLE_NAME = vdw_table_name, &
+         !               PSEUDO_DIR = pseudo_dir, DIRNAME = dirname)
          !
 !-------------------------------------------------------------------------------
 ! ... OCCUPATIONS
@@ -399,7 +399,9 @@ MODULE cp_restart
 ! ... PARALLELISM
 !-------------------------------------------------------------------------------
          !
-         CALL write_para( kunit, nproc, nproc_pool, nproc_image )
+         ntask_groups=get_ntask_groups()
+         CALL write_para( kunit, nproc, nproc_pool, nproc_image, ntask_groups, &
+                          nproc_pot, nproc_bgrp, nproc_ortho )
          !
       END IF
       !
@@ -702,7 +704,9 @@ MODULE cp_restart
             !
          END IF
          !
-         CALL write_gk( iunout, ik, mill_g, filename )
+         IF( .NOT. smallmem ) THEN
+            CALL write_gk( iunout, ik, filename )
+         END IF
          !
          DO iss = 1, nspin
             ! 
@@ -907,6 +911,8 @@ MODULE cp_restart
       !
       IF ( ionode ) CALL iotk_close_write( iunpun )
       !
+      call mp_barrier()
+      !
       IF( .NOT. twfcollect ) THEN
          !
          tmp_dir_save = tmp_dir
@@ -933,7 +939,6 @@ MODULE cp_restart
       DEALLOCATE( ftmp )
       DEALLOCATE( tau  )
       DEALLOCATE( ityp )
-      DEALLOCATE( mill_g )
       !
       CALL save_history( dirname, nfi )
       !
@@ -1033,7 +1038,6 @@ MODULE cp_restart
       INTEGER              :: i, j, iss, ig, nspin_wfc, ierr, ik
       REAL(DP)             :: omega, htm1(3,3), hinv(3,3), scalef
       LOGICAL              :: found
-      INTEGER, ALLOCATABLE :: mill_g(:,:)
       !
       ! ... variables read for testing pourposes
       !
@@ -1847,7 +1851,7 @@ MODULE cp_restart
       !
       if ( nprint_nfi == -2 ) then
          CALL read_print_counter( nprint_nfi, tmp_dir, ndr )
-         IF( iverbosity > 2 ) write( stdout,*) 'nprint_nfi= ',nprint_nfi
+         IF( iverbosity > 1 ) write( stdout,*) 'nprint_nfi= ',nprint_nfi
       endif
       !
       RETURN
@@ -2309,11 +2313,12 @@ MODULE cp_restart
     !
     !
     !
-    SUBROUTINE write_gk( iun, ik, mill_g, filename )
+    SUBROUTINE write_gk( iun, ik, filename )
        !
        USE gvecw,                    ONLY : ngw, ngw_g
+       USE gvect,                    ONLY : ngm, ngm_g
        USE control_flags,            ONLY : gamma_only
-       USE gvect,       ONLY : ig_l2g, mill
+       USE gvect,                    ONLY : ig_l2g, mill
        USE mp,                       ONLY : mp_sum
        USE mp_global,                ONLY : intra_bgrp_comm
        USE io_global,                ONLY : ionode
@@ -2321,14 +2326,25 @@ MODULE cp_restart
        IMPLICIT NONE
        !
        INTEGER,            INTENT(IN) :: iun, ik
-       INTEGER,            INTENT(IN) :: mill_g(:,:)
        CHARACTER(LEN=256), INTENT(IN) :: filename
        !
        INTEGER, ALLOCATABLE :: igwk(:)
        INTEGER, ALLOCATABLE :: itmp1(:)
+       INTEGER, ALLOCATABLE :: mill_g(:,:)
        INTEGER  :: npwx_g, npw_g, ig, ngg
        REAL(DP) :: xk(3)
 
+       ! ... Collect G vectors
+       !   
+       ALLOCATE( mill_g( 3, ngm_g ) )
+       !
+       mill_g = 0
+       !
+       mill_g(:,ig_l2g(1:ngm)) = mill(:,1:ngm)
+       !
+       CALL mp_sum( mill_g, intra_bgrp_comm )
+       !
+
        xk     = 0.0d0
        npwx_g = ngw_g
        npw_g  = ngw_g
@@ -2386,6 +2402,7 @@ MODULE cp_restart
        END IF
        !
        DEALLOCATE( igwk )
+       DEALLOCATE( mill_g )
 
        RETURN
 
