@@ -13,13 +13,12 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   USE kinds,                    ONLY : DP
   USE constants,                ONLY : bohr_radius_angs, amu_au
   USE control_flags,            ONLY : iprint, isave, thdyn, tpre, iverbosity, &
-                                       tfor, remove_rigid_rot, taurdr,         &
-                                       tprnfor, tsdc, lconstrain, lwf, lneb,   &
-                                       ndr, ndw, nomore, tsde, &
+                                       tfor, remove_rigid_rot, taurdr, llondon,&
+                                       tprnfor, tsdc, lconstrain, lwf,         &
+                                       ndr, ndw, nomore, tsde, textfor,        &
                                        tortho, tnosee, tnosep, trane, tranp,   &
                                        tsdp, tcp, tcap, ampre, amprp, tnoseh,  &
-                                       tolp, ortho_eps, ortho_max, printwfc,   &
-                                       textfor
+                                       tolp, ortho_eps, ortho_max, printwfc
   USE core,                     ONLY : rhoc
   USE uspp_param,               ONLY : nhm, nh, nvb, ish
   USE uspp,                     ONLY : nkb, vkb, becsum, deeq, okvan, nlcc_any
@@ -45,7 +44,8 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   USE gvect,       ONLY : gstart, mill, eigts1, eigts2, eigts3
   USE ions_base,                ONLY : na, nat, amass, nax, nsp, rcmax
   USE ions_base,                ONLY : ind_srt, ions_cofmass, ions_kinene, &
-                                       ions_temp, ions_thermal_stress, if_pos, extfor
+                                       ions_temp, ions_thermal_stress, &
+                                       if_pos, extfor
   USE ions_base,                ONLY : ions_vrescal, fricp, greasp, &
                                        iforce, ndfrz, ions_shiftvar, ityp, &
                                        atm, ind_bck, cdm, cdms, ions_cofmsub
@@ -112,10 +112,12 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   USE orthogonalize_base,       ONLY : updatc
   USE control_flags,            ONLY : force_pairing
   USE mp,                       ONLY : mp_bcast, mp_sum
-  USE mp_global,                ONLY : root_bgrp, intra_bgrp_comm, np_ortho, me_ortho, ortho_comm, &
+  USE mp_global,                ONLY : root_bgrp, intra_bgrp_comm, np_ortho, &
+                                       me_ortho, ortho_comm, &
                                        me_bgrp, inter_bgrp_comm, nbgrp
   USE ldaU_cp,                  ONLY : lda_plus_u, vupsi
   USE fft_base,                 ONLY : dfftp
+  USE london_module,            ONLY : energy_london, force_london, stres_london
   !
   IMPLICIT NONE
   !
@@ -128,7 +130,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
   ! ... control variables
   !
   LOGICAL :: tfirst, tlast, tstop, tconv
-  LOGICAL :: ttprint, tfile, tstdout
+  LOGICAL :: tprint, tfile, tstdout
     !  logical variable used to control printout
   !
   ! ... forces on ions
@@ -200,7 +202,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      dt2bye   = dt2 / emass
      nfi     = nfi + 1
      tlast   = ( nfi == nomore ) .OR. tlast
-     ttprint = ( MOD( nfi, iprint ) == 0 ) .OR. tlast 
+     tprint  = ( MOD( nfi, iprint ) == 0 ) .OR. tlast 
      tfile   = ( MOD( nfi, iprint ) == 0 )
      tstdout = ( MOD( nfi, iprint_stdout ) == 0 ) .OR. tlast
      !
@@ -325,6 +327,19 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      !
      IF (lda_plus_u) fion = fion + forceh
      !
+     ! DFT+D (Grimme) dispersion forces (factor 0.5 converts to Ha/a.u.)
+     !
+     IF ( llondon ) THEN
+        ALLOCATE( usrt_tau0( 3, nat ), usrt_fion( 3, nat ) )
+        usrt_tau0(:,:) = tau0(:,ind_bck(:))/alat
+        usrt_fion =  0.5_dp*force_london ( alat, nat,ityp, at,bg, usrt_tau0 )
+        fion(:,:) = fion(:,:) + usrt_fion(:,ind_srt(:))
+        IF ( tpre ) stress = stress + 0.5_dp * stres_london ( alat , nat , &
+                              ityp , at , bg , usrt_tau0 , omega )
+        DEALLOCATE (usrt_fion, usrt_tau0)
+
+     END IF
+     !
      IF ( tpre ) THEN
         !
         CALL nlfh( stress, bec_bgrp, dbec, lambda, descla )
@@ -463,6 +478,10 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
            hold = h
            h    = hnew
            !
+           IF( nbgrp > 1 ) THEN
+              CALL mp_bcast( h, 0, inter_bgrp_comm )
+           END IF
+           !
            CALL newinit( h, iverbosity )
            !
            CALL newnlinit()
@@ -496,13 +515,13 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
            !
            CALL gram_bgrp( vkb, bec_bgrp, nkb, cm_bgrp, ngw )
            !
-           IF ( iverbosity > 3 ) CALL dotcsc( eigr, cm_bgrp, ngw, nbsp_bgrp )
+           IF ( iverbosity > 2 ) CALL dotcsc( eigr, cm_bgrp, ngw, nbsp_bgrp )
            !
         END IF
         !
         !  correction to displacement of ions
         !
-        IF ( iverbosity > 2 ) CALL print_lambda( lambda, descla, nbsp, 9, 1.D0 )
+        IF ( iverbosity > 1 ) CALL print_lambda( lambda, descla, nbsp, 9, 1.D0 )
         !
         IF ( tortho ) THEN
            CALL updatc( ccc, lambda, phi_bgrp, bephi, becp_bgrp, bec_bgrp, cm_bgrp, descla )
@@ -521,7 +540,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
            CALL caldbec_bgrp( eigr, cm_bgrp, dbec, descla )
         END IF
         !
-        IF ( iverbosity > 2 ) CALL dotcsc( eigr, cm_bgrp, ngw, nbsp_bgrp )
+        IF ( iverbosity > 1 ) CALL dotcsc( eigr, cm_bgrp, ngw, nbsp_bgrp )
         !
      END IF
      !
@@ -609,7 +628,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
         !
      END IF
      !
-     IF ( MOD( nfi, iprint ) == 0 .OR. tlast ) THEN
+     IF ( tprint ) THEN
         !
         IF( tortho ) THEN
            !
@@ -632,17 +651,22 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      !
      IF ( lwf ) CALL ef_enthalpy( enthal, tau0 )
      !
-     IF ( tens ) THEN
+     IF ( tens .AND. tprint ) THEN
         !
-        IF ( MOD( nfi, iprint ) == 0 .OR. tlast ) THEN
-           !
-           WRITE( stdout, '("Occupations  :")' )
-           WRITE( stdout, '(10F9.6)' ) ( f(i), i = 1, nbsp )
-           !
-        END IF
+        WRITE( stdout, '("Occupations  :")' )
+        WRITE( stdout, '(10F9.6)' ) ( f(i), i = 1, nbsp )
         !
      END IF
      !
+     ! DFT+D (Grimme) dispersion contribution to energy (0.5 converts to Ha)
+     !
+     IF ( llondon ) THEN
+        ALLOCATE( usrt_tau0( 3, nat ) )
+        usrt_tau0(:,:) = tau0(:,ind_bck(:))/alat
+        etot = etot + 0.5_dp*energy_london (alat, nat,ityp,at,bg, usrt_tau0)
+        DEALLOCATE (usrt_tau0)
+     END IF
+     !
      epot = eht + epseu + exc
      !
      IF ( .NOT. tcg ) THEN
@@ -683,7 +707,9 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      if (abivol) etot = etot - P_ext*volclu
      if (abisur) etot = etot - Surf_t*surfclu
      !
-     CALL printout_new( nfi, tfirst, tfile, ttprint, tps, hold, stress, &
+     IF ( tstdout) CALL spinsq ( c0_bgrp, bec_bgrp, rhor )
+     !
+     CALL printout_new( nfi, tfirst, tfile, tprint, tps, hold, stress, &
                         tau0, vels, fion, ekinc, temphc, tempp, temps, etot, &
                         enthal, econs, econt, vnhh, xnhh0, vnhp, xnhp0, atot, &
                         ekin, epot, tprnfor, tpre, tstdout )
@@ -703,6 +729,15 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
         IF ( tnosee ) CALL electrons_nose_shiftvar( xnhep, xnhe0, xnhem )
         IF ( tnoseh ) CALL cell_nose_shiftvar( xnhhp, xnhh0, xnhhm )
         !
+        IF( nbgrp > 1 ) THEN
+           CALL mp_bcast( tau0, 0, inter_bgrp_comm )
+           CALL mp_bcast( taus, 0, inter_bgrp_comm )
+           CALL mp_bcast( vels, 0, inter_bgrp_comm )
+           CALL mp_bcast( xnhp0, 0, inter_bgrp_comm )
+           CALL mp_bcast( xnhe0, 0, inter_bgrp_comm )
+           CALL mp_bcast( xnhh0, 0, inter_bgrp_comm )
+        END IF
+        !
      END IF
      !
      IF ( thdyn .AND. tfirst ) CALL emass_precond( ema0bg, ggp, ngw, tpiba2, emass_cutoff )
@@ -719,8 +754,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
         !
         CALL cg_update( tfirst, nfi, c0_bgrp )
         !
-        IF ( tfor .AND. .NOT. tens .AND. &
-             ( ( MOD( nfi, isave ) == 0 ) .OR. tlast ) ) THEN
+        IF ( tfor .AND. .NOT. tens .AND. tprint ) THEN
            !
            ! ... in this case optimize c0 and lambda for smooth
            ! ... restart with CP
@@ -859,19 +893,14 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
      DO ia = 1, na(is)
         !
         isa = isa + 1
-        !
         ipos = ind_srt( isa )
-        !
         tau_out(:,ipos) = tau0(:,isa)
-        !
         fion_out(:,ipos) = fion(:,isa)
         !
      END DO
      !
   END DO
   !
-  IF ( lneb ) fion_out(:,1:nat) = fion(:,1:nat) * DBLE( if_pos(:,1:nat) )
-  !
   conv_elec = .TRUE.
   !
   IF ( tcg ) cm_bgrp = c0old
@@ -881,7 +910,7 @@ SUBROUTINE cprmain( tau_out, fion_out, etot_out )
                   xnhp0, xnhpm, vnhp, nhpcl,nhpdim,ekincm, xnhh0, xnhhm,    &
                   vnhh, velh, fion, tps, z0t, f, rhor )
   !
-  IF( iverbosity > 2 ) CALL print_lambda( lambda, descla, nbsp, nbsp, 1.D0 )
+  IF( iverbosity > 1 ) CALL print_lambda( lambda, descla, nbsp, nbsp, 1.D0 )
   !
   IF (lda_plus_u) DEALLOCATE( forceh )
 
@@ -961,6 +990,12 @@ SUBROUTINE terminate_run()
   CALL print_clock( 'tmp_clk1' )
   CALL print_clock( 'tmp_clk2' )
   CALL print_clock( 'tmp_clk3' )
+  CALL print_clock( 'gram' )
+  CALL print_clock( 'nlinit' )
+  CALL print_clock( 'init_dim' )
+  CALL print_clock( 'newnlinit' )
+  CALL print_clock( 'from_scratch' )
+  CALL print_clock( 'from_restart' )
 !==============================================================
 ! Lingzhu Kong
   CALL print_clock( 'wf' )
