@@ -79,14 +79,14 @@ MODULE input
      !-------------------------------------------------------------------------
      !
      USE control_flags,      ONLY : fix_dependencies, lconstrain
-     USE io_global,          ONLY : meta_ionode, stdout
+     USE io_global,          ONLY : ionode, stdout
      USE ions_base,          ONLY : nat, tau, ityp
      USE constraints_module, ONLY : init_constraint
      !
      IMPLICIT NONE
      !
      !
-     IF ( meta_ionode ) THEN
+     IF ( ionode ) THEN
         !
         WRITE( UNIT = stdout, &
                FMT = "(//,3X,'Main Simulation Parameters (from input)',/ &
@@ -188,7 +188,8 @@ MODULE input
      USE control_flags, ONLY : textfor
      USE control_flags, ONLY : do_makov_payne, twfcollect
      USE control_flags, ONLY : lwf, lwfnscf, lwfpbe0, lwfpbe0nscf ! Lingzhu Kong
-     USE control_flags, ONLY : lneb, lpath
+     USE control_flags, ONLY : lneb, lsmd, lpath
+     USE control_flags, ONLY : llondon, smallmem
      !
      ! ...  Other modules
      !
@@ -213,7 +214,7 @@ MODULE input
         electron_dynamics, electron_damping, electron_temperature,   &
         ion_dynamics, ekin_conv_thr, etot_conv_thr, forc_conv_thr, ion_maxstep,&
         electron_maxstep, ion_damping, ion_temperature, ion_velocities, tranp, &
-        amprp, ion_nstepe, cell_nstepe, cell_dynamics, cell_damping,           &
+        amprp, ion_nstepe, cell_nstepe, cell_dynamics, cell_damping, london,   &
         cell_parameters, cell_velocities, cell_temperature, force_pairing,     &
         tapos, tavel, ecutwfc, emass, emass_cutoff, taspc, trd_ht, ibrav,      &
         ortho_eps, ortho_max, ntyp, tolp, calculation, disk_io, dt,            &
@@ -221,7 +222,8 @@ MODULE input
         ampre, nstep, restart_mode, ion_positions, startingwfc, printwfc,      &
         orthogonalization, electron_velocities, nat, if_pos, phase_space,      &
         tefield, epol, efield, tefield2, epol2, efield2, remove_rigid_rot,     &
-        iesr_inp, saverho, tdipole_card, rd_for, assume_isolated, wf_collect
+        iesr_inp, saverho, tdipole_card, rd_for, assume_isolated, wf_collect,  &
+        memory
      !
      IMPLICIT NONE
      !
@@ -243,10 +245,17 @@ MODULE input
      forc_conv_thr_ = forc_conv_thr
      ekin_maxiter_  = electron_maxstep
      iesr           = iesr_inp
+     llondon        = london
      !
      remove_rigid_rot_ = remove_rigid_rot
      twfcollect = wf_collect
      !
+     ! ... define memory related internal switches
+     !
+     IF( TRIM( memory ) == 'small' ) THEN
+        smallmem = .TRUE.
+     END IF
+     !
      ! Options for isolated system
      SELECT CASE( TRIM( assume_isolated ) )
          !
@@ -283,7 +292,11 @@ MODULE input
      emaec_ = emass_cutoff
      ! no longer implemented!
      lneb = ( TRIM( calculation ) == 'neb' )
-     lpath = lneb
+     IF ( lneb ) CALL errore ( 'iosys', &
+                 'NEB no longer implemented, use "neb.x" instead', 1)
+     lsmd = ( TRIM( calculation ) == 'smd' )
+     IF ( lsmd ) CALL errore ( 'iosys', 'SMD no longer implemented', 1)
+     lpath = lneb .OR. lsmd
 !====================================================================
 !Lingzhu Kong
      lwf = ( TRIM( calculation ) == 'cp-wf'      .OR. &
@@ -304,7 +317,7 @@ MODULE input
      trhow_ = saverho
      tksw_  = ( TRIM( disk_io ) == 'high' )
      !
-     iverbosity_ = 1
+     iverbosity_ = 0
      timing_ = .FALSE.
           ! The code write to files fort.8 fort.41 fort.42 fort.43
           ! a detailed report of subroutines timing
@@ -319,29 +332,29 @@ MODULE input
      SELECT CASE( TRIM( verbosity ) )
        CASE( 'minimal' )
          !
-         iverbosity_ = 0
+         iverbosity_ =-1
          !
        CASE( 'low', 'default' )
          !
-         iverbosity_ = 1
+         iverbosity_ = 0
          timing_ = .TRUE.
          !
        CASE( 'medium' )
          !
-         iverbosity_   = 2
+         iverbosity_ = 1
          timing_   = .TRUE.
          tprnsfac_ = .TRUE.
          !
        CASE( 'high' )
          !
-         iverbosity_   = 3
+         iverbosity_ = 2
          memchk_   = .TRUE.
          timing_   = .TRUE.
          tprnsfac_ = .TRUE.
          !
        CASE( 'debug' )
          !
-         iverbosity_   = 4
+         iverbosity_ = 3
          memchk_   = .TRUE.
          timing_   = .TRUE.
          !
@@ -742,10 +755,6 @@ MODULE input
       IF( .NOT. tavel .AND. TRIM(ion_velocities)=='from_input' ) &
         CALL errore(' iosys ',' ION_VELOCITIES not present in stdin ', 1 )
 
-      IF( ( TRIM( calculation ) == 'smd' ) .AND. ( TRIM( cell_dynamics ) /= 'none' ) ) THEN
-        CALL errore(' smiosys ',' cell_dynamics not implemented : '//TRIM(cell_dynamics), 1 )
-      END IF
-
       RETURN
    END SUBROUTINE set_control_flags
    !
@@ -753,7 +762,7 @@ MODULE input
    SUBROUTINE modules_setup()
      !-------------------------------------------------------------------------
      !
-     USE control_flags,    ONLY : lconstrain, lneb, tpre, thdyn, tksw
+     USE control_flags,    ONLY : lconstrain, tpre, thdyn, tksw
 
      USE constants,        ONLY : amu_au, pi
      !
@@ -764,7 +773,7 @@ MODULE input
            ecutrho, ecfixed, qcutz, q2sigma, tk_inp, wmass,                   &
            ion_radius, emass, emass_cutoff, temph, fnoseh, nr1b, nr2b, nr3b,  &
            tempw, fnosep, nr1, nr2, nr3, nr1s, nr2s, nr3s, ekincw, fnosee,    &
-           outdir, prefix, nkstot, xk,                                        &
+           outdir, prefix, nkstot, xk, vdw_table_name,                        &
            occupations, n_inner, fermi_energy, rotmass, occmass,              &
            rotation_damping, occupation_damping, occupation_dynamics,         &
            rotation_dynamics, degauss, smearing, nhpcl, nhptyp, ndega,        &
@@ -796,6 +805,7 @@ MODULE input
                                   axis
      USE input_parameters, ONLY : lda_plus_u, Hubbard_U
      USE input_parameters, ONLY : step_pen, A_pen, alpha_pen, sigma_pen
+     USE input_parameters, ONLY : london, london_s6, london_rcut
      !
      USE ions_base,        ONLY : zv
      USE cell_base,        ONLY : cell_base_init, cell_dyn_init, at, cell_alat
@@ -818,6 +828,10 @@ MODULE input
      USE ensemble_dft,     ONLY : ensemble_initval,tens
      USE wannier_base,     ONLY : wannier_init
      USE efield_module,    ONLY : tefield
+     USE funct,            ONLY : dft_is_nonlocc
+     USE kernel_table,     ONLY : vdw_table_name_ => vdw_table_name, &
+                                  initialize_kernel_table
+     USE london_module,    ONLY : init_london, scal6, lon_rcut
      !
      IMPLICIT NONE
      !
@@ -946,6 +960,19 @@ MODULE input
      CALL ldaU_init0 ( ntyp, lda_plus_u, Hubbard_U )
      CALL ldaUpen_init( SIZE(sigma_pen), step_pen, sigma_pen, alpha_pen, A_pen )
      !
+     IF ( london) THEN
+        lon_rcut    = london_rcut
+        scal6       = london_s6
+        CALL init_london ( )
+     END IF
+     !
+     ! ... initialize kernel table for nonlocal functionals
+     !
+     IF ( dft_is_nonlocc( ) ) THEN
+        vdw_table_name_ = vdw_table_name
+        CALL initialize_kernel_table()
+     ENDIF
+     !
      RETURN
      !
   END SUBROUTINE modules_setup
@@ -963,14 +990,14 @@ MODULE input
     USE control_flags,      ONLY: nbeg, iprint, ndr, ndw, nomore
     USE time_step,          ONLY: delt
     USE cp_electronic_mass, ONLY: emass, emass_cutoff
-    USE io_global,          ONLY: meta_ionode, stdout
+    USE io_global,          ONLY: ionode, stdout
 
     IMPLICIT NONE
 
     IF( .NOT. has_been_read ) &
       CALL errore( ' iosys ', ' input file has not been read yet! ', 1 )
 
-    IF( meta_ionode ) THEN
+    IF( ionode ) THEN
       WRITE( stdout, 500) nbeg, restart_mode, nomore, iprint, ndr, ndw
       WRITE( stdout, 505) delt
       WRITE( stdout, 510) emass
@@ -1010,7 +1037,7 @@ MODULE input
     USE cell_nose,            ONLY: cell_nose_info
     USE cell_base,            ONLY: frich
     USE efield_module,        ONLY: tefield, efield_info, tefield2, efield_info2
-    USE io_global,            ONLY: meta_ionode, stdout
+    USE io_global,            ONLY: ionode, stdout
     USE time_step,            ONLY: delt
     !
     !
@@ -1021,7 +1048,7 @@ MODULE input
     IF( .NOT. has_been_read ) &
       CALL errore( ' iosys ', ' input file has not been read yet! ', 1 )
 
-    IF( meta_ionode ) THEN
+    IF( ionode ) THEN
       !
       CALL cutoffs_print_info( )
       !
