@@ -26,12 +26,6 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img3.png"
  ALT="$ \psi^{*}_{}$">|; 
 
-$key = q/{Phi(G)-Phi^*(G)over2i};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="88" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img8.png"
- ALT="$ {\Phi(G) - \Phi^*(G)\over 2i}$">|; 
-
 $key = q/tau_{i}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="18" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
@@ -62,11 +56,11 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img14.png"
  ALT="$\displaystyle {\frac{{k-1}}{{nr3}}}$">|; 
 
-$key = q/{Phi(G)+Phi^*(G)over2};MSF=1.6;LFS=12;AAT/;
+$key = q/{Phi(G)-Phi^*(-G)over2i};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="88" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img7.png"
- ALT="$ {\Phi(G) + \Phi^*(G)\over 2}$">|; 
+ WIDTH="98" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img8.png"
+ ALT="$ {\Phi(G) - \Phi^*(-G)\over 2i}$">|; 
 
 $key = q/displaystyletau_{2}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
@@ -86,18 +80,24 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img10.png"
  ALT="$\displaystyle {\frac{{i-1}}{{nr1}}}$">|; 
 
-$key = q/Gamma;MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="16" HEIGHT="16" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img1.png"
- ALT="$ \Gamma$">|; 
-
 $key = q/le;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="19" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
  SRC="|."$dir".q|img9.png"
  ALT="$ \le$">|; 
 
+$key = q/{Phi(G)+Phi^*(-G)over2};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="98" HEIGHT="47" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img7.png"
+ ALT="$ {\Phi(G) + \Phi^*(-G)\over 2}$">|; 
+
+$key = q/Gamma;MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="16" HEIGHT="16" ALIGN="BOTTOM" BORDER="0"
+ SRC="|."$dir".q|img1.png"
+ ALT="$ \Gamma$">|; 
+
 $key = q/psi_{j}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="24" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
