@@ -1,4 +1,4 @@
-# LaTeX2HTML 2008 (1.71)
+# LaTeX2HTML 2002-2-1 (1.71)
 # Associate internals original text with physical files.
 
 
@@ -6,30 +6,22 @@ $key = q/SubSec:para/;
 $ref_files{$key} = "$dir".q|node17.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/pw2casino_info/;
-$ref_files{$key} = "$dir".q|node28.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/Sec:Installation/;
 $ref_files{$key} = "$dir".q|node9.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/SubSec:Contacts/;
-$ref_files{$key} = "$dir".q|node5.html|; 
+$ref_files{$key} = "$dir".q|node4.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:pseudo/;
-$ref_files{$key} = "$dir".q|node24.html|; 
+$key = q/SubSec:Terms/;
+$ref_files{$key} = "$dir".q|node6.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/SubSec:Examples/;
 $ref_files{$key} = "$dir".q|node13.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:badpara/;
-$ref_files{$key} = "$dir".q|node50.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:Guidelines/;
 $ref_files{$key} = "$dir".q|node5.html|; 
 $noresave{$key} = "$nosave";
