@@ -69,8 +69,8 @@
 
         INTEGER   :: iforceh(3,3) = 1  ! if iforceh( i, j ) = 0 then h( i, j ) 
                                        ! is not allowed to move
-        LOGICAL   :: fix_volume = .FALSE.  ! True if cell volume is kept fixed
-
+        LOGICAL   :: fix_volume = .FALSE.! True if cell volume is kept fixed
+        LOGICAL   :: fix_area = .FALSE.  ! True if area in xy plane is kept constant
         REAL(DP) :: wmass = 0.0_DP     ! cell fictitious mass
         REAL(DP) :: press = 0.0_DP     ! external pressure 
 
@@ -130,8 +130,12 @@
       !
       SELECT CASE ( TRIM( cell_units ) )
         CASE ( 'bohr' )
+          IF( celldm( 1 ) /= 0.0_DP .OR. a /= 0.0_dp ) CALL errore &
+              ('cell_base_init','lattice vectors in Bohr or in a0 units?',1)
           units = 1.0_DP
         CASE ( 'angstrom' )
+          IF( celldm( 1 ) /= 0.0_DP .OR. a /= 0.0_dp ) CALL errore &
+              ('cell_base_init','lattice vectors in A or in a0 units?',2)
           units = 1.0_DP / bohr_radius_angs
         CASE DEFAULT
           IF( celldm( 1 ) /= 0.0_DP ) THEN
@@ -156,6 +160,9 @@
      ELSE
         alat = SQRT ( at(1,1)**2+at(2,1)**2+at(3,1)**2 )
      END IF
+     ! for compatibility: celldm still used in phonon etc 
+     celldm(1) = alat
+     !
      at(:,:) = at(:,:) / alat
      CALL volume( alat, at(1,1), at(1,2), at(1,3), omega )
      !
@@ -622,6 +629,19 @@
             CASE ( 'shape' )
               iforceh = 1
               fix_volume = .true.
+! 2DSHAPE: CASE FOR SHAPE CHANGE IN xy PLANE WITH CONST AREA
+! contribution from Richard Charles Andrew
+! Physics Department, University of Pretoria
+! South Africa, august 2012.
+            CASE ( '2Dshape' )
+              iforceh = 1
+              iforceh(3,3) = 0
+              iforceh(1,3) = 0
+              iforceh(3,1) = 0
+              iforceh(2,3) = 0
+              iforceh(3,2) = 0
+              fix_area = .true.
+! 2DSHAPE
             CASE ( 'volume' )
               CALL errore(' init_dofree ', &
                  ' cell_dofree = '//TRIM(cell_dofree)//' not yet implemented ', 1 )
@@ -641,6 +661,14 @@
               ! ... if you want the entire xy plane to be free, uncomment:
               ! iforceh(1,2) = 1
               ! iforceh(2,1) = 1
+! 2DSHAPE THE ENTIRE xy PLANE IS FREE
+            CASE ('2Dxy')
+              iforceh      = 0
+              iforceh(1,1) = 1
+              iforceh(2,2) = 1
+              iforceh(1,2) = 1
+              iforceh(2,1) = 1
+! 2DSHAPE
             CASE ('xz')
               iforceh      = 0
               iforceh(1,1) = 1
@@ -675,7 +703,7 @@
     tpiba  = 2.0_DP * pi / alat
     tpiba2 = tpiba * tpiba
     !
-    IF( iverbosity > 3 ) THEN
+    IF( iverbosity > 2 ) THEN
       WRITE( stdout, 210 )
       WRITE( stdout, 220 ) ( ht( 1, j ), j = 1, 3 )
       WRITE( stdout, 220 ) ( ht( 2, j ), j = 1, 3 )
@@ -697,7 +725,7 @@
     ainv(2,:) = bg(:,2)/alat
     ainv(3,:) = bg(:,3)/alat
     !
-    IF( iverbosity > 3 ) THEN
+    IF( iverbosity > 2 ) THEN
       WRITE( stdout, 305 ) alat
       WRITE( stdout, 310 ) at(:,1)*alat
       WRITE( stdout, 320 ) at(:,2)*alat
