@@ -40,7 +40,7 @@ MODULE control_flags
             t_diis_rot, tnosee, tnosep, tnoseh, tcp, tcap, tdamp, tdampions, &
             tconvthrs, tolp, convergence_criteria, tionstep, nstepe,         &
             tsteepdesc, tatomicwfc, tscreen, gamma_only, force_pairing,      &
-            lecrpa, tddfpt
+            lecrpa, tddfpt, smallmem
   !
   PUBLIC :: fix_dependencies, check_flags
   PUBLIC :: tksw, trhor, thdyn, trhow
@@ -85,6 +85,7 @@ MODULE control_flags
   LOGICAL :: force_pairing = .FALSE. ! Force pairing
   LOGICAL :: lecrpa        = .FALSE. ! RPA correlation energy request
   LOGICAL :: tddfpt        = .FALSE. ! use tddfpt specific tweaks to ph.x routines
+  LOGICAL :: smallmem      = .FALSE. ! the memory per task is small
   !
   TYPE (convergence_criteria) :: tconvthrs
                               !  thresholds used to check GS convergence
@@ -213,6 +214,7 @@ MODULE control_flags
     tr2_init,         &! initial value of tr2 for adaptive thresholds
     tr2_multi          ! the dexx multiplier for adaptive thresholds
                        ! tr2 = tr2_multi * dexx after each V_exx update 
+  LOGICAL, PUBLIC :: scf_must_converge
   !
   ! ... pw diagonalization
   !
@@ -263,7 +265,7 @@ MODULE control_flags
   INTEGER, PUBLIC :: &
     io_level = 1       ! variable controlling the amount of I/O to file
   INTEGER, PUBLIC :: & ! variable controlling the amount of I/O to output
-    iverbosity = 0     ! -2 minimal, -1 low, 0 medium, 1 high, 2 debug
+    iverbosity = 0     ! -1 minimal, 0 low, 1 medium, 2 high, 3 debug
   !
   ! ... miscellany
   !
