@@ -14,8 +14,8 @@ MODULE environment
   USE io_files, ONLY: crash_file, crashunit, nd_nmbr
   USE io_global, ONLY: stdout, meta_ionode
   USE mp_global, ONLY: me_image, my_image_id, root_image, nimage, &
-      nproc_image, nproc, npool, nproc_pool, nbgrp, get_ntask_groups
-  USE global_version, ONLY: version_number
+      nproc_image, nproc, npool, nproc_bgrp, nbgrp, get_ntask_groups
+  USE global_version, ONLY: version_number, svn_revision
 
   IMPLICIT NONE
 
@@ -54,6 +54,8 @@ CONTAINS
     CALL start_clock( TRIM(code) )
 
     code_version = TRIM (code) // " v." // TRIM (version_number)
+    IF ( TRIM (svn_revision) /= "unknown" ) code_version = &
+         TRIM (code_version) // " (svn rev. " // TRIM (svn_revision) // ")"
 
     ! ... for compatibility with PWSCF
 
@@ -117,6 +119,7 @@ CONTAINS
        WRITE( stdout,3335)
     END IF
 3335 FORMAT('=',78('-'),'=')
+    CALL flush_unit(stdout)
 
     RETURN
   END SUBROUTINE environment_end
@@ -130,8 +133,8 @@ CONTAINS
 
     CALL date_and_tim( cdate, ctime )
     !
-    WRITE( stdout, '(/5X,"Program ",A18," starts on ",A9," at ",A9)' ) &
-         code_version, cdate, ctime
+    WRITE( stdout, '(/5X,"Program ",A," starts on ",A9," at ",A9)' ) &
+         TRIM(code_version), cdate, ctime
     !
     WRITE( stdout, '(/5X,"This program is part of the open-source Quantum ",&
          &    "ESPRESSO suite", &
@@ -177,11 +180,11 @@ CONTAINS
     !
 #if defined(__OPENMP)
     WRITE( stdout, '(/5X,"Parallel version (MPI & OpenMP), running on ",&
-         &I5," processor cores")' ) nproc * omp_get_max_threads()
+         &I7," processor cores")' ) nproc * omp_get_max_threads()
     !
-    WRITE( stdout, '(5X,"Number of MPI processes:           ",I5)' ) nproc
+    WRITE( stdout, '(5X,"Number of MPI processes:           ",I7)' ) nproc
     !
-    WRITE( stdout, '(5X,"Threads/MPI process:               ",I4)' ) &
+    WRITE( stdout, '(5X,"Threads/MPI process:               ",I7)' ) &
          omp_get_max_threads()
 #else
     WRITE( stdout, '(/5X,"Parallel version (MPI), running on ",&
@@ -189,15 +192,15 @@ CONTAINS
 #endif
     !
     IF ( nimage > 1 ) WRITE( stdout, &
-         '(5X,"path-images division:  nimage    = ",I4)' ) nimage
-    IF ( nbgrp > 1 ) WRITE( stdout, &
-         '(5X,"band groups division:  nbgrp     = ",I4)' ) nbgrp
+         '(5X,"path-images division:  nimage    = ",I7)' ) nimage
     IF ( npool > 1 ) WRITE( stdout, &
-         '(5X,"K-points division:     npool     = ",I4)' ) npool
-    IF ( nproc_pool > 1 ) WRITE( stdout, &
-         '(5X,"R & G space division:  proc/pool = ",I4)' ) nproc_pool
+         '(5X,"K-points division:     npool     = ",I7)' ) npool
+    IF ( nbgrp > 1 ) WRITE( stdout, &
+         '(5X,"band groups division:  nbgrp     = ",I7)' ) nbgrp
+    IF ( nproc_bgrp > 1 ) WRITE( stdout, &
+         '(5X,"R & G space division:  proc/nbgrp/npool/nimage = ",I7)' ) nproc_bgrp
     IF ( get_ntask_groups() > 1 ) WRITE( stdout, &
-         '(5X,"wavefunctions fft division:  fft/group = ",I4)' ) &
+         '(5X,"wavefunctions fft division:  fft/group = ",I7)' ) &
          get_ntask_groups()
     !
   END SUBROUTINE parallel_info
@@ -211,7 +214,7 @@ CONTAINS
     !
 #if defined(__OPENMP)
     WRITE( stdout, '(/5X,"Serial multi-threaded version, running on ",&
-         &I3," processor cores")' ) omp_get_max_threads()
+         &I4," processor cores")' ) omp_get_max_threads()
     !
 #else
     WRITE( stdout, '(/5X,"Serial version")' )
