@@ -1095,9 +1095,9 @@
 #endif
 
      IF ( nx < 1 ) &
-         call errore('cfft3',' nx is less than 1 ', 1)
+         call errore('cfft3d',' nx is less than 1 ', 1)
      IF ( ny < 1 ) &
-         call errore('cfft3',' ny is less than 1 ', 1)
+         call errore('cfft3d',' ny is less than 1 ', 1)
      IF ( nz < 1 ) &
          call errore('cfft3',' nz is less than 1 ', 1)
 
