@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2004-2011 Quantum ESPRESSO group
+! Copyright (C) 2004-2012 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -40,7 +40,6 @@ module funct
 !
   USE io_global, ONLY: stdout
   USE kinds,     ONLY: DP
-  USE control_flags,  ONLY : lwfpbe0, lwfpbe0nscf ! Lingzhu Kong
   IMPLICIT NONE
   PRIVATE
   SAVE
@@ -59,7 +58,7 @@ module funct
   PUBLIC  :: dft_has_finite_size_correction, set_finite_size_volume
   ! driver subroutines computing XC
   PUBLIC  :: xc, xc_spin, gcxc, gcx_spin, gcc_spin, gcc_spin_more
-  PUBLIC  :: dmxc, dmxc_spin, dmxc_nc
+  PUBLIC  :: tau_xc , tau_xc_spin, dmxc, dmxc_spin, dmxc_nc
   PUBLIC  :: dgcxc, dgcxc_spin
   PUBLIC  :: nlc
   ! general XC driver
@@ -88,17 +87,21 @@ module funct
   !              "pbe"   = "sla+pw+pbx+pbc"    = PBE
   !              "revpbe"= "sla+pw+rpb+pbc"    = revPBE (Zhang-Yang)
   !              "pbesol"= "sla+pw+psx+psc"    = PBEsol
+  !              "q2d"   = "sla+pw+q2dx+q2dc"  = PBEQ2D
   !              "hcth"  = "nox+noc+hcth+hcth" = HCTH/120
   !              "olyp"  = "nox+lyp+optx+blyp" = OLYP
   !              "wc"    = "sla+pw+wcx+pbc"    = Wu-Cohen
+  !              "sogga  = "sla+pw+sox+pbec"   = SOGGA
   !              "tpss"  = "sla+pw+tpss+tpss"  = TPSS Meta-GGA
+  !              "m06l"  = "nox+noc+m6lx+m6lc" = M06L Meta-GGA
   !              "pbe0"  = "pb0x+pw+pb0x+pbc"  = PBE0
-  !              "hse"   = "sla+pw+hse+pbc"    = Heyd-Scuseria-Ernzerhof HSE 06
+  !              "hse"   = "sla+pw+hse+pbc"    = Heyd-Scuseria-Ernzerhof 
+  !                                              (HSE 06, see note below)
   !              "b3lyp" = "b3lp+vwn+b3lp+b3lp"= B3LYP
   !              "vdw-df"= "sla+pw+rpb+vdw1"   = vdW-DF
-  !              "vdw-df2"="sla+pw+rpb+vdw2"   = vdW-DF2
-  !              "vdw-df-c09"
-  !              "vdw-df2-c09"
+  !              "vdw-df2"="sla+pw+rw86+vdw2"  = vdW-DF2
+  !              "vdw-df-c09"="sla+pw+c09x+vdw1"
+  !              "vdw-df2-c09"="sla+pw+c09x+vdw2"
   ! or by any nonconflicting combination of the following keywords
   ! (case-insensitive):
   !
@@ -132,8 +135,8 @@ module funct
   !              "pbx"    Perdew-Burke-Ernzenhof exch    igcx =3
   !              "rpb"    revised PBE by Zhang-Yang      igcx =4
   !              "hcth"   Cambridge exch, Handy et al    igcx =5
-  !              "optx"   Handy's exchange functional    igcx =6
   !              "tpss"   TPSS meta-gga                  igcx =7
+  !              "optx"   Handy's exchange functional    igcx =6
   !              "pb0x"   PBE0 (PBE exchange*0.75)       igcx =8
   !              "b3lp"   B3LYP (Becke88*0.72)           igcx =9
   !              "psx"    PBEsol exchange                igcx =10
@@ -141,8 +144,11 @@ module funct
   !              "hse"    HSE screened exchange          igcx =12
   !              "rw86"   revised PW86                   igcx =13
   !              "pbe"    same as PBX, back-comp.        igcx =14
-  !              "tpss"   same as META, back-comp.       igcx =15
+  !              "meta"   same as TPSS, back-comp.       igcx =15
   !              "c09x"   Cooper 09                      igcx =16
+  !              "sox"    sogga                          igcx =17
+  !              "m6lx"   M06L exchange Meta-GGA         igcx =18
+  !              "q2dx"   Q2D exchange grad corr         igcx =19
   !
   ! Gradient Correction on Correlation:
   !              "nogc"   none                           igcc =0 (default)
@@ -154,9 +160,13 @@ module funct
   !              "tpss"   TPSS meta-gga                  igcc =6
   !              "b3lp"   B3LYP (Lee-Yang-Parr*0.81)     igcc =7
   !              "psc"    PBEsol corr                    igcc =8
+  !              "pbe"    same as PBX, back-comp.        igcc =9
+  !              "meta"   same as TPSS, back-comp.       igcc =10
+  !              "m6lc"   M06L corr  Meta-GGA            igcc =11
+  !              "q2dc"   Q2D correlation grad corr      igcc =12
   !
   ! Van der Waals functionals (nonlocal term only)
-  !	         "nonlc"   none                           inlc =0 (default)
+  !             "nonlc"   none                           inlc =0 (default)
   !              "vdw1"    vdW-DF1                        inlc =1
   !              "vdw2"    vdW-DF2                        inlc =2
   !
@@ -177,21 +187,35 @@ module funct
   !              hcth    Handy et al, JCP 109, 6264 (1998)
   !              olyp    Handy et al, JCP 116, 5411 (2002)
   !              revPBE  Zhang and Yang, PRL 80, 890 (1998)
+  !              pbesol  J.P. Perdew et al., PRL 100, 136406 (2008)
+  !              q2d     L. Chiodo et al., PRL 108, 126402 (2012)
   !              rw86    E. Amonn D. Murray et al, J. Chem. Theory comp. 5, 2754 (2009) 
   !              wc      Z. Wu and R. E. Cohen, PRB 73, 235116 (2006)
-  !              tpss    J.Tao, J.P.Perdew, V.N.Staroverov, G.E. Scuseria, 
-  !                      PRL 91, 146401 (2003)
   !              kzk     H.Kwee, S. Zhang, H. Krakauer, PRL 100, 126404 (2008)
   !              pbe0    J.P.Perdew, M. Ernzerhof, K.Burke, JCP 105, 9982 (1996)
   !              hse     Heyd, Scuseria, Ernzerhof, J. Chem. Phys. 118, 8207 (2003)
   !                      Heyd, Scuseria, Ernzerhof, J. Chem. Phys. 124, 219906 (2006).
   !              b3lyp   P.J. Stephens,F.J. Devlin,C.F. Chabalowski,M.J. Frisch
   !                      J.Phys.Chem 98, 11623 (1994)
-  !              pbesol  J.P. Perdew et al., PRL 100, 136406 (2008)
   !              vdW-DF  M. Dion et al., PRL 92, 246401 (2004)
   !                      T. Thonhauser et al., PRB 76, 125112 (2007)
   !              vdw-DF2 Lee et al., Phys. Rev. B 82, 081101 (2010)
   !              c09x    V. R. Cooper, Phys. Rev. B 81, 161104(R) (2010)
+  !              tpss    J.Tao, J.P.Perdew, V.N.Staroverov, G.E. Scuseria, 
+  !                      PRL 91, 146401 (2003)
+  !              sogga   Y. Zhao and D. G. Truhlar, JCP 128, 184109 (2008)
+  !              m06l    Y. Zhao and D. G. Truhlar, JCP 125, 194101 (2006)
+  !
+  ! NOTE ABOUT HSE: there are two slight deviations with respect to the HSE06 
+  ! functional as it is in Gaussian code (that is considered as the reference
+  ! in the chemistry community):
+  ! - The range separation in Gaussian is precisely 0.11 bohr^-1, 
+  !   instead of 0.106 bohr^-1 in this implementation
+  ! - The gradient scaling relation is a bit more complicated 
+  !   [ see: TM Henderson, AF Izmaylov, G Scalmani, and GE Scuseria,
+  !          J. Chem. Phys. 131, 044108 (2009) ]
+  ! These two modifications accounts only for a 1e-5 Ha difference for a 
+  ! single He atom. Info by Fabien Bruneval
   !
   integer, parameter:: notset = -1
   !
@@ -230,7 +254,7 @@ module funct
   ! data
   integer :: nxc, ncc, ngcx, ngcc, ncnl
 
-  parameter (nxc = 8, ncc =11, ngcx =16, ngcc = 10, ncnl=2)
+  parameter (nxc = 8, ncc =11, ngcx =19, ngcc = 12, ncnl=2)
 
   character (len=4) :: exc, corr
   character (len=4) :: gradx, gradc, nonlocc
@@ -239,12 +263,13 @@ module funct
   data exc / 'NOX', 'SLA', 'SL1', 'RXC', 'OEP', 'HF', 'PB0X', 'B3LP', 'KZK' /
   data corr / 'NOC', 'PZ', 'VWN', 'LYP', 'PW', 'WIG', 'HL', 'OBZ', &
               'OBW', 'GL' , 'B3LP', 'KZK' /
-  data gradx / 'NOGX', 'B88', 'GGX', 'PBX',  'RPB', 'HCTH', 'OPTX',&
-               'META', 'PB0X', 'B3LP','PSX', 'WCX', 'HSE', 'RW86', 'PBE', &
-               'TPSS', 'C09X' / 
 
-  data gradc / 'NOGC', 'P86', 'GGC', 'BLYP', 'PBC', 'HCTH', 'META',&
-                'B3LP', 'PSC', 'PBE', 'TPSS' / 
+  data gradx / 'NOGX', 'B88', 'GGX', 'PBX',  'RPB', 'HCTH', 'OPTX',&
+               'TPSS', 'PB0X', 'B3LP','PSX', 'WCX', 'HSE', 'RW86', 'PBE', &
+               'META', 'C09X', 'SOX', 'M6LX', 'Q2DX' / 
+
+  data gradc / 'NOGC', 'P86', 'GGC', 'BLYP', 'PBC', 'HCTH', 'TPSS',&
+                'B3LP', 'PSC', 'PBE', 'META', 'M6LC', 'Q2DC' / 
 
   data nonlocc / '    ', 'VDW1', 'VDW2' / 
 
@@ -288,6 +313,7 @@ CONTAINS
     do l = 1, len
        dftout (l:l) = capital (dft_(l:l) )
     enddo
+
     !
     ! ----------------------------------------------
     ! FIRST WE CHECK ALL THE SPECIAL NAMES
@@ -295,6 +321,7 @@ CONTAINS
     !       not using function "matches"
     ! ----------------------------------------------
     !
+
     if ( 'REVPBE' .EQ. TRIM(dftout) ) then
     ! special case : revPBE
        call set_dft_value (iexch,1) !Default
@@ -335,6 +362,15 @@ CONTAINS
        call set_dft_value (igcc, 8)
        call set_dft_value (inlc,0) !Default       
        dft_defined = .true.
+
+    else if ('PBEQ2D' .EQ. TRIM(dftout) .OR. 'Q2D'.EQ. TRIM(dftout) ) then
+    ! special case : PBEQ2D
+       call set_dft_value (iexch,1) !Default    
+       call set_dft_value (icorr,4)
+       call set_dft_value (igcx,19)
+       call set_dft_value (igcc,12)
+       call set_dft_value (inlc,0) !Default       
+       dft_defined = .true.
        
     else if ('VDW-DF2-C09' .EQ. TRIM(dftout) ) then
     ! Special case vdW-DF2 with C09 exchange
@@ -356,7 +392,7 @@ CONTAINS
 
    else if ('VDW-DF2' .EQ. TRIM(dftout) ) then
     ! Special case vdW-DF2
-       call set_dft_value (iexch, 0)
+       call set_dft_value (iexch, 1)
        call set_dft_value (icorr, 4)
        call set_dft_value (igcx, 13)
        call set_dft_value (igcc, 0)
@@ -403,7 +439,7 @@ CONTAINS
     ! special case : PBC  = PW + PBC 
        call set_dft_value (iexch,1) !Default
        call set_dft_value (icorr,4)
-       call set_dft_value (igcx, 0) !Default       
+       call set_dft_value (igcx,0) !Default       
        call set_dft_value (igcc, 4)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
@@ -420,13 +456,13 @@ CONTAINS
     ! special case : PW91 = GGX + GGC
     else if ('PW91'.EQ. TRIM(dftout) ) then
        call set_dft_value (iexch,1) !Default
-       call set_dft_value (icorr,4) 
+       call set_dft_value (icorr,4)
        call set_dft_value (igcx, 2)
        call set_dft_value (igcc, 2)
        call set_dft_value (inlc,0) !Default    
        dft_defined = .true.
        
-    ! special case : HCTH 
+    ! special case : HCTH
     else if ('HCTH'.EQ. TRIM(dftout)) then
        call set_dft_value(iexch,0) ! contained in hcth
        call set_dft_value(icorr,0) ! contained in hcth
@@ -488,7 +524,26 @@ CONTAINS
        call set_dft_value (igcc, 0)      
        call set_dft_value (inlc,0)    
        dft_defined = .true.
-           
+
+    ! special case : SOGGA = SOX + PBEc       
+    else if (matches ('SOGGA', dftout) ) then
+       call set_dft_value (iexch, 1)
+       call set_dft_value (icorr,4)
+       call set_dft_value (igcx,17)
+       call set_dft_value (igcc, 4)
+       call set_dft_value (inlc,0) ! Default    
+       dft_defined = .true.
+     
+    ! special case : M06L Meta GGA
+    else if ( matches( 'M06L', dftout ) ) THEN
+       !
+       CALL set_dft_value( iexch, 0 ) ! contained in m6lx
+       CALL set_dft_value( icorr, 0 ) ! contained in m6lc
+       CALL set_dft_value( igcx,  18 )
+       CALL set_dft_value( igcc,  11)
+       call set_dft_value (inlc,0) ! Default  
+       dft_defined = .true.
+    
     END IF
 
     !
@@ -543,9 +598,6 @@ CONTAINS
     ! No more defaults, the code exit if the dft is not defined
     ! ----------------------------------------------------------------
 
-    if (igcx == 13 .and. iexch > 0 ) &
-          call errore('set_dft_from_name','revPW86 already contains LDA contribution',iexch)
-   
     ! Back compatibility - TO BE REMOVED
  
     if (igcx == 14) igcx = 3 ! PBE -> PBX
@@ -614,7 +666,7 @@ CONTAINS
 
     isnonlocc = (inlc > 0)
 
-    ismeta     =  (igcx == 7)
+    ismeta     =  (igcx == 7) .or. (igcx == 18)
 
     ! PBE0
     IF ( iexch==6 .or. igcx ==8 ) exx_fraction = 0.25_DP
@@ -900,6 +952,8 @@ CONTAINS
      shortname_ = 'revPBE'
   else if (iexch_==1.and.icorr_==4.and.igcx_==10.and.igcc_==8) then
      shortname_ = 'PBESOL'
+  else if (iexch_==1.and.icorr_==4.and.igcx_==19.and.igcc_==12) then
+     shortname_ = 'Q2D'
   else if (iexch_==1.and.icorr_==4.and.igcx_==12.and.igcc_==4) then
      shortname_ = 'HSE'
   else if (iexch_==1.and.icorr_==4.and.igcx_==11.and.igcc_==4) then
@@ -911,12 +965,16 @@ CONTAINS
      shortname_ = 'OLYP'
   else if (iexch_==1.and.icorr_==4.and.igcx_==4.and.igcc_==0.and.inlc_==1) then
      shortname_ = 'VDW-DF'
-  else if (iexch_==1.and.icorr_==4.and.igcx_==12.and.igcc_==0.and.inlc_==2) then
+  else if (iexch_==1.and.icorr_==4.and.igcx_==13.and.igcc_==0.and.inlc_==2) then
      shortname_ = 'VDW-DF2'
   else if (iexch_==1.and.icorr_==4.and.igcx_==16.and.igcc_==0.and.inlc_==1) then
      shortname_ = 'VDW-DF-C09'
   else if (iexch_==1.and.icorr_==4.and.igcx_==16.and.igcc_==0.and.inlc_==2) then
      shortname_ = 'VDW-DF2-C09'
+  else if (iexch_==0.and.icorr_==0.and.igcx_==18.and.igcc_==11) then
+     shortname_ = 'M06L'
+  else if (iexch_==1.and.icorr_==4.and.igcx_==17.and.igcc_==4) then
+     shortname_ = 'SOGGA'
   else
      shortname_ = ' '
   end if
@@ -983,13 +1041,6 @@ subroutine xc (rho, ex, ec, vx, vc)
   !..exchange
   if (iexch == 1) THEN             !  'sla'
      call slater (rs, ex, vx)
-!==================================================================
-!Lingzhu Kong
-     IF(lwfpbe0 .or. lwfpbe0nscf)THEN
-         ex = 0.75d0 * ex
-         vx = 0.75d0 * vx
-     END IF
-!==================================================================
   ELSEIF (iexch == 2) THEN         !  'sl1'
      call slater1(rs, ex, vx)
   ELSEIF (iexch == 3) THEN         !  'rxc'
@@ -1088,14 +1139,6 @@ subroutine xc_spin (rho, zeta, ex, ec, vxup, vxdw, vcup, vcdw)
   !..exchange
   IF (iexch == 1) THEN      ! 'sla'
      call slater_spin (rho, zeta, ex, vxup, vxdw)
-!==============================================================
-!Lingzhu Kong
-     IF( lwfpbe0 .or. lwfpbe0nscf )THEN
-          ex   = 0.75d0 * ex
-          vxup = 0.75d0 * vxup
-          vxdw = 0.75d0 * vxdw
-     END IF
-!==============================================================
   ELSEIF (iexch == 2) THEN  ! 'sl1'
      call slater1_spin (rho, zeta, ex, vxup, vxdw)
   ELSEIF (iexch == 3) THEN  ! 'rxc'
@@ -1176,12 +1219,6 @@ subroutine xc_spin_vec (rho, zeta, length, evx, evc)
   select case (iexch)
   case(1)            ! 'sla'
      call slater_spin_vec (rho, zeta, evx, length)
-!=========================================================
-!Lingzhu Kong
-     if (lwfpbe0 .or. lwfpbe0nscf) then
-        evx = 0.75_DP * evx
-     end if
-!=========================================================
   case(2)            ! 'sl1'
      do i=1,length
         call slater1_spin (rho(i), zeta(i), evx(i,3), evx(i,1), evx(i,2))
@@ -1285,14 +1322,6 @@ subroutine gcxc (rho, grho, sx, sc, v1x, v2x, v1c, v2c)
      call ggax (rho, grho, sx, v1x, v2x)
   elseif (igcx == 3) then
      call pbex (rho, grho, 1, sx, v1x, v2x)
-!==============================================================
-!Lingzhu Kong
-     IF( lwfpbe0 .or. lwfpbe0nscf )THEN
-          sx  = 0.75d0 * sx
-          v1x = 0.75d0 * v1x
-          v2x = 0.75d0 * v2x
-     END IF
-!==============================================================
   elseif (igcx == 4) then
      call pbex (rho, grho, 2, sx, v1x, v2x)
   elseif (igcx == 5 .and. igcc == 5) then
@@ -1331,6 +1360,8 @@ subroutine gcxc (rho, grho, sx, sc, v1x, v2x, v1c, v2c)
      call rPW86 (rho, grho, sx, v1x, v2x)
   elseif (igcx ==16) then ! 'C09x'
      call c09x (rho, grho, sx, v1x, v2x)
+  elseif (igcx ==19) then ! 'pbesol'
+     call pbex (rho, grho, 4, sx, v1x, v2x)
   else
      sx = 0.0_DP
      v1x = 0.0_DP
@@ -1360,6 +1391,11 @@ subroutine gcxc (rho, grho, sx, sc, v1x, v2x, v1c, v2c)
      end if
   elseif (igcc == 8) then ! 'PBEsol'
      call pbec (rho, grho, 2, sc, v1c, v2c)
+  ! igcc == 9 set to 5, back-compatibility
+  ! igcc ==10 set to 6, back-compatibility
+  ! igcc ==11 M06L calculated in another routine
+  else if (igcc == 12) then ! 'Q2D'
+     call pbec (rho, grho, 3, sc, v1c, v2c)
   else
      sc = 0.0_DP
      v1c = 0.0_DP
@@ -1438,7 +1474,7 @@ subroutine gcx_spin (rhoup, rhodw, grhoup2, grhodw2, &
      v2xdw = 2.0_DP * v2xdw
   elseif (igcx == 3 .or. igcx == 4 .or. igcx == 8 .or. &
           igcx == 10 .or. igcx == 12) then
-     ! igcx=3: PBE, igcx=4: revised PBE, igcx=8 PBE0, igcx=10: PBEsol
+     ! igcx=3: PBE, igcx=4: revised PBE, igcx=8: PBE0, igcx=10: PBEsol
      ! igcx=12: HSE
      if (igcx == 4) then
         iflag = 2
@@ -1471,16 +1507,6 @@ subroutine gcx_spin (rhoup, rhodw, grhoup2, grhodw2, &
        v2xup = (1.0_DP - exx_fraction) * v2xup
        v2xdw = (1.0_DP - exx_fraction) * v2xdw
      end if
-!=============================================================
-!Lingzhu Kong
-     if (igcx == 3 .and. (lwfpbe0 .or. lwfpbe0nscf) ) then
-       sx =    0.75D0 * sx
-       v1xup = 0.75D0 * v1xup
-       v1xdw = 0.75D0 * v1xdw
-       v2xup = 0.75D0 * v2xup
-       v2xdw = 0.75D0 * v2xdw
-     end if
-!=============================================================
      if (igcx == 12 .and. exx_started ) then
 
         call pbexsr_lsd (rhoup, rhodw, grhoup2, grhodw2, sxsr,  &
@@ -1645,16 +1671,6 @@ subroutine gcx_spin_vec(rhoup, rhodw, grhoup2, grhodw2, &
         v2xup = (1.0_DP - exx_fraction) * v2xup
         v2xdw = (1.0_DP - exx_fraction) * v2xdw
      end if
-!=============================================================
-!Lingzhu Kong
-     if (igcx == 3 .and. (lwfpbe0 .or. lwfpbe0nscf) ) then
-       sx =    0.75D0 * sx
-       v1xup = 0.75D0 * v1xup
-       v1xdw = 0.75D0 * v1xdw
-       v2xup = 0.75D0 * v2xup
-       v2xdw = 0.75D0 * v2xdw
-     end if
-!=============================================================
   case(9)
      do i=1,length
         if (rhoup(i) > small .and. sqrt(abs(grhoup2(i)) ) > small) then
@@ -1739,7 +1755,6 @@ subroutine gcc_spin (rho, zeta, grho, sc, v1cup, v1cdw, v2c)
 
   real(DP), parameter :: small = 1.E-10_DP, epsr=1.E-6_DP
   !
-  !
   if ( abs(zeta) > 1.0_DP ) then
      sc = 0.0_DP
      v1cup = 0.0_DP
@@ -1860,6 +1875,125 @@ subroutine nlc (rho_valence, rho_core, enl, vnl, v)
   return
 end subroutine nlc
 
+!
+!-----------------------------------------------------------------------
+!------- META CORRECTIONS DRIVERS ----------------------------------
+!-----------------------------------------------------------------------
+!
+!-----------------------------------------------------------------------
+subroutine tau_xc (rho, grho, tau, ex, ec, v1x, v2x, v3x, v1c, v2c, v3c)
+  !-----------------------------------------------------------------------
+  !     gradient corrections for exchange and correlation - Hartree a.u.
+  !     See comments at the beginning of module for implemented cases
+  !
+  !     input:  rho, grho=|\nabla rho|^2
+  !
+  !     definition:  E_x = \int e_x(rho,grho) dr
+  !
+  !     output: sx = e_x(rho,grho) = grad corr
+  !             v1x= D(E_x)/D(rho)
+  !             v2x= D(E_x)/D( D rho/D r_alpha ) / |\nabla rho|
+  !             v3x= D(E_x)/D(tau)
+  !
+  !             sc, v1c, v2c as above for correlation
+  !
+  implicit none
+
+  real(DP) :: rho, grho, tau, ex, ec, v1x, v2x, v3x, v1c, v2c, v3c
+  
+  !_________________________________________________________________________
+  
+  if     (igcx == 7  .and. igcc == 6) then
+  
+     call tpsscxc (rho, grho, tau, ex, ec, v1x, v2x, v3x, v1c, v2c, v3c)
+     
+  elseif (igcx == 18 .and. igcc == 11) then
+  
+     call   m06lxc (rho, grho, tau, ex, ec, v1x, v2x, v3x, v1c, v2c, v3c)
+     
+  else
+  
+     call errore('v_xc_meta','wrong igcx and/or igcc',1)
+     
+  end if
+  
+  return
+  
+end subroutine tau_xc
+
+!
+!
+!-----------------------------------------------------------------------
+subroutine tau_xc_spin (rhoup, rhodw, grhoup, grhodw, tauup, taudw, ex, ec,       &
+           &            v1xup, v1xdw, v2xup, v2xdw, v3xup, v3xdw, v1cup, v1cdw,   &
+           &            v2cup, v2cdw, v2cup_vec, v2cdw_vec, v3cup, v3cdw)
+
+!-----------------------------------------------------------------------
+  !
+  !
+  
+  implicit none
+
+  real(dp), intent(in)                :: rhoup, rhodw, tauup, taudw
+  real(dp), dimension (3), intent(in) :: grhoup, grhodw
+  
+  real(dp), intent(out)               :: ex, ec, v1xup, v1xdw, v2xup, v2xdw, v3xup, v3xdw,  &
+                                      &  v1cup, v1cdw, v2cup, v2cdw, v3cup, v3cdw
+  real(dp), dimension(3), intent(out) :: v2cup_vec, v2cdw_vec
+  
+  !
+  !  Local variables
+  !
+  integer                 :: ipol
+  real(dp)                :: rh, zeta, atau, grhoup2, grhodw2
+  real(dp), parameter     :: epsr=1.0d-08, zero=0._dp
+  !
+  !_____________________________
+
+  grhoup2 = zero
+  grhodw2 = zero
+  
+  v2cup         = zero
+  v2cdw         = zero
+  v2cup_vec (:) = zero
+  v2cdw_vec (:) = zero
+  
+  
+  do ipol=1,3
+     grhoup2 = grhoup2 + grhoup(ipol)**2
+     grhodw2 = grhodw2 + grhodw(ipol)**2
+  end do
+
+  
+  if (igcx == 7 .and. igcc == 6) then
+
+     call tpsscx_spin(rhoup, rhodw, grhoup2, grhodw2, tauup,   &
+              &  taudw, ex, v1xup,v1xdw,v2xup,v2xdw,v3xup,v3xdw)
+  
+     rh   =  rhoup + rhodw
+        
+     zeta = (rhoup - rhodw) / rh
+     atau =  tauup + taudw    ! KE-density in Hartree
+
+     call tpsscc_spin(rh,zeta,grhoup,grhodw, atau,ec,              &
+     &                v1cup,v1cdw,v2cup_vec,v2cdw_vec,v3cup, v3cdw) 
+  
+  
+  elseif (igcx == 18 .and. igcc == 11) then
+  
+     call   m06lxc_spin (rhoup, rhodw, grhoup2, grhodw2, tauup, taudw,      &
+            &            ex, ec, v1xup, v1xdw, v2xup, v2xdw, v3xup, v3xdw,  &
+            &            v1cup, v1cdw, v2cup, v2cdw, v3cup, v3cdw)
+     
+  else
+  
+     call errore('v_xc_meta','wrong igcx and/or igcc',1)
+     
+  end if
+  
+end subroutine tau_xc_spin                
+                
+
 !-----------------------------------------------------------------------
 !------- DRIVERS FOR DERIVATIVES OF XC POTENTIAL -----------------------
 !-----------------------------------------------------------------------
