@@ -201,6 +201,9 @@ MODULE input_parameters
           ! if .TRUE. a static homogeneous electric field is present
           ! via the modern theory of polarizability - differs from tefield!
 
+        LOGICAL :: lorbm = .false.
+          ! if .TRUE. an orbital magnetization is computed (Kubo terms)
+
         LOGICAL :: dipfield = .false.
           ! if .TRUE. the dipole field is subtracted
           ! only used in PW for surface calculations
@@ -248,6 +251,15 @@ MODULE input_parameters
 
         CHARACTER(len=256) :: vdw_table_name = ' '
 
+        CHARACTER(len=80) :: memory = 'default' 
+          ! controls memory usage 
+        CHARACTER(len=80) :: memory_allowed(3)
+        DATA memory_allowed / 'small', 'default', 'large' /
+          ! if memory = 'small' then QE tries to use (when implemented) algorithms using less memory,
+          !                     even if they are slower than the default
+          ! if memory = 'large' then QE tries to use (when implemented) algorithms using more memory
+          !                     to enhance performance.
+
 #if defined (__MS2)
         LOGICAL :: MS2_enabled = .false.       ! Enable the shared memory exchange in MS2
         CHARACTER(len=256) :: MS2_handler = ' '! Name for the shared memory handler in MS2
@@ -259,7 +271,7 @@ MODULE input_parameters
           forc_conv_thr, pseudo_dir, disk_io, tefield, dipfield, lberry,  &
           gdir, nppstr, wf_collect, printwfc, lelfield, nberrycyc, refg,  &
           tefield2, saverho, tabps, lkpoint_dir, use_wannier, lecrpa,     &
-          vdw_table_name
+          vdw_table_name, lorbm, memory
 
 
 #if defined ( __MS2)
@@ -374,8 +386,10 @@ MODULE input_parameters
         INTEGER, PARAMETER :: nspinx=2
         REAL(DP) :: starting_ns_eigenvalue(lqmax,nspinx,nsx) = -1.0_DP
         REAL(DP) :: hubbard_u(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_j0(nsx) = 0.0_DP
         REAL(DP) :: hubbard_j(3,nsx) = 0.0_DP
         REAL(DP) :: hubbard_alpha(nsx) = 0.0_DP
+        REAL(DP) :: hubbard_beta(nsx) = 0.0_DP
         CHARACTER(len=80) :: U_projection_type = 'atomic'
 
         LOGICAL :: la2F = .false.
@@ -487,6 +501,7 @@ MODULE input_parameters
              occupations, degauss, nspin, ecfixed,                            &
              qcutz, q2sigma, lda_plus_U, lda_plus_u_kind,                     &
              Hubbard_U, Hubbard_J, Hubbard_alpha,                             &
+             Hubbard_J0, Hubbard_beta,                                        &
              edir, emaxpos, eopreg, eamp, smearing, starting_ns_eigenvalue,   &
              U_projection_type, input_dft, la2F, assume_isolated,             &
              nqx1, nqx2, nqx3, ecutfock,                                      &
@@ -530,7 +545,7 @@ MODULE input_parameters
         ! type of switching functions used in the solvation models
         !    0: original Fattebert-Gygi
         !    1: ultrasoft dielectric function as defined in Andreussi et al.
-        REAL(DP) :: rhomax = 0.001
+        REAL(DP) :: rhomax = 0.005
         ! first parameter of the sw function, roughly corresponding 
         ! to the density threshold of the solvation model
         REAL(DP) :: rhomin = 0.0001
@@ -599,7 +614,7 @@ MODULE input_parameters
 !
 ! Ionic countercharge parameters
 !
-        REAL(DP) :: cion = 0.D0
+        REAL(DP) :: env_ioncc_concentration = 0.D0
         ! molar concentration of ionic countercharge (M=mol/L)
         REAL(DP) :: zion = 1.D0
         ! valence of ionic countercharge
@@ -617,7 +632,8 @@ MODULE input_parameters
              mixtype, ndiis, mixrhopol, tolrhopol,                     &
              env_surface_tension, delta,                               &
              env_pressure,                                             &
-             cion, zion, rhopb, solvent_temperature
+             env_ioncc_concentration, zion, rhopb,                     &
+             solvent_temperature
 #endif
 !
 !=----------------------------------------------------------------------------=!
@@ -697,6 +713,8 @@ MODULE input_parameters
           ! maximum number of steps in electronic minimization
           ! This parameter apply only when using 'cg' electronic or
           ! ionic dynamics
+        LOGICAL :: scf_must_converge = .true.
+          ! stop or continue if SCF does not converge
 
         CHARACTER(len=80) :: electron_dynamics = 'none'
           ! set how electrons should be moved
@@ -967,7 +985,7 @@ MODULE input_parameters
           ! to be used together with penalty functional ...
 
         NAMELIST / electrons / emass, emass_cutoff, orthogonalization, &
-          electron_maxstep, ortho_eps, ortho_max, electron_dynamics,   &
+          electron_maxstep, scf_must_converge, ortho_eps, ortho_max, electron_dynamics,   &
           electron_damping, electron_velocities, electron_temperature, &
           ekincw, fnosee, ampre, grease,                               &
           diis_size, diis_nreset, diis_hcut,                           &
