@@ -257,7 +257,7 @@ MODULE london_module
          !
          ! ... define a gross maximum bound of the mxr size
          !
-         mxr = INT ( ( 2 * ( lon_rcut + alat ) ) ** 3 / omega )
+         mxr = 1 + INT ( ( 2 * ( lon_rcut + alat ) ) ** 3 / omega )
          !
       END IF
       !
@@ -377,8 +377,7 @@ MODULE london_module
             dist6 = dist ** 6
             !
             f_damp = 1.d0 / ( 1.d0 + &
-            !
-            exp ( -beta * ( dist / ( R_sum ( ityp ( atb ) , ityp ( ata ) ) ) - 1 )))
+            exp ( -beta * ( dist / ( R_sum ( ityp (atb) , ityp (ata) ) ) - 1 )))
             !
             energy_london = energy_london - &
                   ( C6_ij ( ityp ( atb ) , ityp ( ata ) ) / dist6 ) * &
