@@ -69,7 +69,7 @@
         MODULE PROCEDURE mp_alltoall_c3d, mp_alltoall_i3d
       END INTERFACE
       INTERFACE mp_circular_shift_left
-        MODULE PROCEDURE mp_circular_shift_left_d2d_int,mp_circular_shift_left_d2d_double
+        MODULE PROCEDURE mp_circular_shift_left_d2d_int,mp_circular_shift_left_d2d_double,mp_circular_shift_left_d2d_complex
       END INTERFACE
 
 
@@ -2258,6 +2258,41 @@ SUBROUTINE mp_circular_shift_left_d2d_double( buf, itag, gid )
    RETURN
 END SUBROUTINE mp_circular_shift_left_d2d_double
 
+SUBROUTINE mp_circular_shift_left_d2d_complex( buf, itag, gid )
+   IMPLICIT NONE
+   COMPLEX(DP) :: buf( :, : )
+   INTEGER, INTENT(IN) :: itag
+   INTEGER, OPTIONAL, INTENT(IN) :: gid
+   INTEGER :: nsiz, group, ierr, npe, sour, dest, mype
+
+#if defined (__MPI)
+
+   INTEGER :: istatus( mpi_status_size )
+   !
+   group = mpi_comm_world
+   IF( PRESENT( gid ) ) group = gid
+   !
+   CALL mpi_comm_size( group, npe, ierr )
+   IF (ierr/=0) CALL mp_stop( 8100 )
+   CALL mpi_comm_rank( group, mype, ierr )
+   IF (ierr/=0) CALL mp_stop( 8101 )
+   !
+   sour = mype + 1
+   IF( sour == npe ) sour = 0
+   dest = mype - 1
+   IF( dest == -1 ) dest = npe - 1
+   !
+   CALL MPI_Sendrecv_replace( buf, SIZE(buf), MPI_DOUBLE_COMPLEX, &
+        dest, itag, sour, itag, group, istatus, ierr)
+   !
+   IF (ierr/=0) CALL mp_stop( 8102 )
+   !
+#else
+   ! do nothing
+#endif
+   RETURN
+END SUBROUTINE mp_circular_shift_left_d2d_complex
+
 
       FUNCTION mp_get_comm_null( )
         IMPLICIT NONE
