@@ -23,6 +23,7 @@ MODULE mp_global
   INTEGER :: root  = 0  ! index of the root processor
   INTEGER :: nproc = 1  ! number of processors
   INTEGER :: world_comm = 0  ! communicator
+  INTEGER :: nproc_file = 1  ! saved on file
   !
   ! ... Image groups (processors within an image)
   !
@@ -33,6 +34,18 @@ MODULE mp_global
   INTEGER :: nproc_image=1 ! number of processors within an image
   INTEGER :: inter_image_comm = 0  ! inter image communicator
   INTEGER :: intra_image_comm = 0  ! intra image communicator  
+  INTEGER :: nproc_image_file = 1  ! value saved on file
+  !
+  ! ... Pot groups (processors within a cooking-pot)
+  !
+  INTEGER :: npot       = 1  ! number of pots
+  INTEGER :: me_pot     = 0  ! index of the processor within a pot
+  INTEGER :: root_pot   = 0  ! index of the root processor within a pot
+  INTEGER :: my_pot_id  = 0  ! index of my pot
+  INTEGER :: nproc_pot  = 1  ! number of processors within a pot
+  INTEGER :: inter_pot_comm  = 0  ! inter pot communicator
+  INTEGER :: intra_pot_comm  = 0  ! intra pot communicator
+  INTEGER :: nproc_pot_file = 1  ! value saved on file
   !
   ! ... Pool groups (processors within a pool of k-points)
   !
@@ -43,6 +56,7 @@ MODULE mp_global
   INTEGER :: nproc_pool  = 1  ! number of processors within a pool
   INTEGER :: inter_pool_comm  = 0  ! inter pool communicator
   INTEGER :: intra_pool_comm  = 0  ! intra pool communicator
+  INTEGER :: nproc_pool_file  = 1  ! saved on file
   !
   ! ... Band groups (processors within a pool of bands)
   !
@@ -55,6 +69,7 @@ MODULE mp_global
   INTEGER :: intra_bgrp_comm  = 0  ! intra band group communicator  
   INTEGER :: ibnd_start = 0 !starting band index
   INTEGER :: ibnd_end = 0   !ending band index
+  INTEGER :: nproc_bgrp_file = 1  ! value saved on file
   !
   ! ... ortho (or linear-algebra) groups
   !
@@ -65,7 +80,10 @@ MODULE mp_global
   INTEGER :: leg_ortho   = 1  ! the distance in the father communicator
                               ! of two neighbour processors in ortho_comm
   INTEGER :: ortho_comm  = 0  ! communicator for the ortho group
+  INTEGER :: ortho_row_comm  = 0  ! communicator for the ortho row group
+  INTEGER :: ortho_col_comm  = 0  ! communicator for the ortho col group
   INTEGER :: ortho_comm_id= 0 ! id of the ortho_comm
+  INTEGER :: nproc_ortho_file = 1  ! value saved on file
   !
 #if defined __SCALAPACK
   INTEGER :: me_blacs   =  0  ! BLACS processor index starting from 0
@@ -77,14 +95,12 @@ MODULE mp_global
   ! ... "task" groups (for band parallelization of FFT)
   !
   INTEGER :: ntask_groups = 1  ! number of proc. in an orbital "task group" 
+  INTEGER :: ntask_groups_file  = 1  ! number of task groups 
   !
   ! ... Misc parallelization info
   ! 
   INTEGER :: kunit = 1  ! granularity of k-point distribution
   ! ... number of processors written in the data file for checkin purposes:
-  INTEGER :: nproc_file = 1        ! world group
-  INTEGER :: nproc_image_file = 1  ! in an image
-  INTEGER :: nproc_pool_file  = 1  ! in a pool
   !
   PRIVATE :: init_pools, init_bands, init_ortho
   PRIVATE :: ntask_groups
@@ -92,23 +108,19 @@ MODULE mp_global
 CONTAINS
   !
   !-----------------------------------------------------------------------
-  SUBROUTINE mp_startup ( )
+  SUBROUTINE mp_startup ( start_images )
     !-----------------------------------------------------------------------
-    ! ... This subroutine initializes MPI
-    ! ... Processes are organized in NIMAGE images each dealing with a subset of
-    ! ... images used to discretize the "path" (only in "path" optimizations)
-    ! ... Within each image processes are organized in NPOOL pools each dealing
-    ! ... with a subset of kpoints.
-    ! ... Within each pool R & G space distribution is performed.
-    ! ... NPROC is read from command line or can be set with the appropriate
-    ! ... environment variable ( for example use 'setenv MP_PROCS 8' on IBM SP
-    ! ... machine to run on NPROC=8 processors ); NIMAGE and NPOOL are read from
-    ! ... command line.
-    ! ... NPOOL must be a whole divisor of NPROC
+    ! ... This wrapper subroutine initializes MPI
+    ! ... If option with_images=.true., processes are organized into images,
+    ! ... each of which performing a quasi-indipendent calculation, uch as
+    ! ... a point in configuration space (NEB) or a phonon irrep (PHonon)
+    ! ... Within each image processes are further subdivided into various
+    ! ... groups and parallelization levels
     !
     IMPLICIT NONE
-    INTEGER :: world, nproc_ortho_in = 0
-    INTEGER :: root = 0
+    LOGICAL, INTENT(IN), OPTIONAL :: start_images
+    INTEGER :: world, nproc_ortho_in = 0, root = 0
+    LOGICAL :: do_images
     !
     !
     ! ... get the basic parameters from communications sub-system
@@ -119,131 +131,56 @@ CONTAINS
     !
     CALL mp_start( nproc, mpime, world )
     !
+    ! ... for compatibility: initialize images
+    !
+    meta_ionode = ( mpime == root )
+    meta_ionode_id = root
+    do_images = PRESENT(start_images) 
+    IF ( do_images ) do_images = start_images
+    IF ( do_images ) THEN
+       !
+       ! ... get nimage from command line
+       !
+       IF ( meta_ionode ) THEN
+          !
+          CALL get_arg_nimage( nimage )
+          nimage = MAX( nimage, 1 )
+          nimage = MIN( nimage, nproc )
+          !
+       END IF
+       CALL mp_barrier(world)
+       CALL mp_bcast( nimage, meta_ionode_id )
+    ELSE
+       nimage = 1
+    END IF
+    !
+    CALL init_images ( world )
     !
     ! ... now initialize processors and groups variables
     ! ... set global coordinate for this processor
     ! ... root  = index of the root processor
     !
-    CALL mp_global_start( root, mpime, world, nproc )
-    !
-    ! ... initialize input/output, set (and get) the I/O nodes
-    !
-    CALL io_global_start( mpime, root )
-    !
-    meta_ionode = ( mpime == root )
-    meta_ionode_id = root
-    !
-    IF ( meta_ionode ) THEN
-       !
-       ! ... How many parallel images ?
-       !
-       CALL get_arg_nimage( nimage )
-       !
-       nimage = MAX( nimage, 1 )
-       nimage = MIN( nimage, nproc )
-       !
-       ! ... How many band groups?
-       !
-       CALL get_arg_nbgrp( nbgrp )
-       !
-       nbgrp = MAX( nbgrp, 1 )
-       nbgrp = MIN( nbgrp, nproc )
-       !
-       ! ... How many k-point pools ?
-       !
-       CALL get_arg_npool( npool )
-       !
-       npool = MAX( npool, 1 )
-       npool = MIN( npool, nproc )
-       !
-       ! ... How many task groups ?
-       !
-       CALL get_arg_ntg( ntask_groups )
-       !
-       ! ... How many processors involved in diagonalization of Hamiltonian ?
-       !
-       CALL get_arg_northo( nproc_ortho_in )
-       !
-       ! ... nproc_ortho_in = 0 if no command-line option -ndiag or -northo
-       ! ... nproc_ortho_in = N if -ndiag N or -northo N comm-line opt present
-       !
-    END IF
-    !
-    CALL mp_barrier()
-    !
-    ! ... broadcast input parallelization options to all processors
-    !
-    CALL mp_bcast( npool,  meta_ionode_id )
-    CALL mp_bcast( nimage, meta_ionode_id )
-    CALL mp_bcast( nbgrp, meta_ionode_id )
-    CALL mp_bcast( ntask_groups, meta_ionode_id )
-    CALL mp_bcast( nproc_ortho_in, meta_ionode_id )
-    !
-    ! ... initialize images, band, k-point, ortho groups in sequence
-    !
-    CALL init_images( world_comm )
-    !
-    CALL init_pools( intra_image_comm )
-    !
-    CALL init_bands( intra_pool_comm )
-    !
-    CALL init_ortho( nproc_ortho_in, intra_bgrp_comm )
-    !
+    CALL mp_startup_new (root_image, intra_image_comm ) 
     !
     RETURN
     !
   END SUBROUTINE mp_startup
-  !
-  !-----------------------------------------------------------------------
-  SUBROUTINE mp_global_start( root_i, mpime_i, group_i, nproc_i )
-    !-----------------------------------------------------------------------
-    !
-    IMPLICIT NONE
-    !
-    INTEGER, INTENT(IN) :: root_i, mpime_i, group_i, nproc_i
-    !
-    root             = root_i
-    mpime            = mpime_i
-    world_comm       = group_i
-    nproc            = nproc_i
-    nproc_pool       = nproc_i
-    nproc_image      = nproc_i
-    nproc_bgrp       = nproc_i
-    my_pool_id       = 0
-    my_image_id      = 0
-    my_bgrp_id       = 0
-    me_pool          = mpime
-    me_image         = mpime
-    me_bgrp          = mpime
-    root_pool        = root
-    root_image       = root
-    root_bgrp        = root
-    inter_pool_comm  = group_i
-    intra_pool_comm  = group_i
-    inter_image_comm = group_i
-    intra_image_comm = group_i
-    inter_bgrp_comm  = group_i
-    intra_bgrp_comm  = group_i
-    ortho_comm       = group_i
-    !
-    RETURN
-    !
-  END SUBROUTINE mp_global_start
-  !
   !-----------------------------------------------------------------------
   SUBROUTINE mp_startup_new (root, world ) 
     !-----------------------------------------------------------------------
-    ! ... This subroutine initializes MPI
-    ! ... Processes are organized in NIMAGE images each dealing with a subset of
-    ! ... images used to discretize the "path" (only in "path" optimizations)
-    ! ... Within each image processes are organized in NPOOL pools each dealing
-    ! ... with a subset of kpoints.
-    ! ... Within each pool R & G space distribution is performed.
-    ! ... NPROC is read from command line or can be set with the appropriate
-    ! ... environment variable ( for example use 'setenv MP_PROCS 8' on IBM SP
-    ! ... machine to run on NPROC=8 processors ); NIMAGE and NPOOL are read from
-    ! ... command line.
-    ! ... NPOOL must be a whole divisor of NPROC
+    ! ... This subroutine initializes the various parallelization levels
+    ! ... inside an image. On input:
+    ! ...     root =root processor for this image
+    ! ...     world=communicator for this image (intra_image_comm)
+    ! ... Within each image processes are subdivided into:
+    ! ...     npool k-points groups, for parallelization over k-points
+    ! ... Each k-point group is subdivided into
+    ! ...     nbgr      band groups, for parallelization over bands
+    ! ... Each band group performs parallelization over plane waves 
+    ! ... and is subdivided into
+    ! ...    ntg        task groups, for parallelization of H*psi
+    ! ...    ndiag      linear-algebra groups, for parallelization of LA
+    ! ... Npool, nbgr, ntg, ndiag are read from command line 
     !
     IMPLICIT NONE
     !
@@ -266,18 +203,22 @@ CONTAINS
     CALL io_global_start( myrank, root )
     !
     IF ( myrank == root ) THEN
+       !
+       ! ... How many parallel pots ?
+       !
+       CALL get_arg_npot( npot )
+       npot = MAX( npot, 1 )
+       npot = MIN( npot, nproc )
        !
        ! ... How many band groups?
        !
        CALL get_arg_nbgrp( nbgrp )
-       !
        nbgrp = MAX( nbgrp, 1 )
        nbgrp = MIN( nbgrp, nproc )
        !
        ! ... How many k-point pools ?
        !
        CALL get_arg_npool( npool )
-       !
        npool = MAX( npool, 1 )
        npool = MIN( npool, nproc )
        !
@@ -302,17 +243,15 @@ CONTAINS
     CALL mp_bcast( nbgrp, root, world )
     CALL mp_bcast( ntask_groups, root, world )
     CALL mp_bcast( nproc_ortho_in, root, world )
+    CALL mp_bcast( npot, root, world )
     !
-    ! ... initialize images, band, k-point, ortho groups in sequence
-    !
-    !
-    CALL init_pools( intra_image_comm )
+    ! ... initialize k-point, band, ortho groups in sequence
     !
+    CALL init_pots( world )
+    CALL init_pools( world )
     CALL init_bands( intra_pool_comm )
-    !
     CALL init_ortho( nproc_ortho_in, intra_bgrp_comm )
     !
-    !
     RETURN
     !
   END SUBROUTINE mp_startup_new
@@ -342,6 +281,8 @@ CONTAINS
     inter_bgrp_comm  = group_i
     intra_bgrp_comm  = group_i
     ortho_comm       = group_i
+    ortho_row_comm   = group_i
+    ortho_col_comm   = group_i
     !
     RETURN
     !
@@ -489,6 +430,61 @@ CONTAINS
   END SUBROUTINE init_bands
   !
   !----------------------------------------------------------------------------
+  SUBROUTINE init_pots( parent_comm )
+    !---------------------------------------------------------------------------
+    !
+    ! ... This routine divides the parent group into pots
+    !
+    IMPLICIT NONE
+    !
+    INTEGER, INTENT(IN) :: parent_comm
+    !
+    INTEGER :: ierr = 0
+    INTEGER :: parent_nproc = 1
+    INTEGER :: parent_mype  = 0
+    !
+#if defined (__MPI)
+    !
+    parent_nproc = mp_size( parent_comm )
+    parent_mype  = mp_rank( parent_comm )
+    !
+    ! ... number of cpus per pot (they are created inside each parent group)
+    !
+    nproc_pot = parent_nproc / npot
+    !
+    IF ( MOD( parent_nproc, npot ) /= 0 ) &
+         CALL errore( 'init_pots', 'invalid number of pots, parent_nproc /= nproc_pot * npot', 1 )  
+    !
+    ! ... my_pot_id  =  pot index for this processor    ( 0 : npot - 1 )
+    ! ... me_pot     =  processor index within the pot  ( 0 : nproc_pot - 1 )
+    !
+    my_pot_id = parent_mype / nproc_pot    
+    me_pot    = MOD( parent_mype, nproc_pot )
+    !
+    CALL mp_barrier( parent_comm )
+    !
+    ! ... the intra_pot_comm communicator is created
+    !
+    CALL MPI_COMM_SPLIT( parent_comm, my_pot_id, parent_mype, intra_pot_comm, ierr )
+    !
+    IF ( ierr /= 0 ) &
+       CALL errore( 'init_pots', 'intra pot communicator initialization', ABS(ierr) )
+    !
+    CALL mp_barrier( parent_comm )
+    !
+    ! ... the inter_pot_comm communicator is created
+    !
+    CALL MPI_COMM_SPLIT( parent_comm, me_pot, parent_mype, inter_pot_comm, ierr )
+    !
+    IF ( ierr /= 0 ) &
+       CALL errore( 'init_pots', 'inter pot communicator initialization', ABS(ierr) )
+    !
+#endif
+    !
+    RETURN
+  END SUBROUTINE init_pots
+  !
+  !----------------------------------------------------------------------------
   SUBROUTINE init_pools( parent_comm )
     !---------------------------------------------------------------------------
     !
@@ -705,6 +701,10 @@ CONTAINS
             CALL errore( " init_ortho_group ", " wrong task coordinates in ortho group ", ierr )
        IF( me_ortho1*leg_ortho /= me_all ) &
             CALL errore( " init_ortho_group ", " wrong rank assignment in ortho group ", ierr )
+
+       CALL MPI_COMM_SPLIT( ortho_comm, me_ortho(2), me_ortho(1), ortho_col_comm, ierr )
+       CALL MPI_COMM_SPLIT( ortho_comm, me_ortho(1), me_ortho(2), ortho_row_comm, ierr )
+
     else
        ortho_comm_id = 0
        me_ortho(1) = me_ortho1
