@@ -114,11 +114,13 @@ MODULE read_namelists_module
        IF( prog == 'CP' ) wf_collect = .TRUE.  ! default for CP is true
        printwfc = -1
        lelfield = .FALSE.
+       lorbm = .FALSE.
        nberrycyc  = 1
        lkpoint_dir = .TRUE.
        lecrpa   = .FALSE.   
        !
        saverho = .TRUE.
+       memory = 'default'
        !
        RETURN
        !
@@ -194,8 +196,10 @@ MODULE read_namelists_module
        lda_plus_U = .FALSE.
        lda_plus_u_kind = 0
        Hubbard_U = 0.0_DP
+       Hubbard_J0 = 0.0_DP
        Hubbard_J = 0.0_DP
        Hubbard_alpha = 0.0_DP
+       Hubbard_beta = 0.0_DP
        step_pen=.false.
        A_pen=0.0_DP
        sigma_pen=0.01_DP
@@ -300,7 +304,7 @@ MODULE read_namelists_module
        !
        env_pressure = 0.D0
        !
-       cion = 0.0D0
+       env_ioncc_concentration = 0.0D0
        zion = 1.0D0
        rhopb = 0.0001D0
        solvent_temperature = 300.0D0
@@ -379,6 +383,7 @@ MODULE read_namelists_module
        ortho_eps = 1.E-8_DP
        ortho_max = 20
        electron_maxstep = 100
+       scf_must_converge = .true.
        !
        ! ... ( 'sd' | 'cg' | 'damp' | 'verlet' | 'none' | 'diis' )
        !
@@ -768,10 +773,12 @@ MODULE read_namelists_module
        CALL mp_bcast( wf_collect,    ionode_id )
        CALL mp_bcast( printwfc,      ionode_id )
        CALL mp_bcast( lelfield,      ionode_id )
+       CALL mp_bcast( lorbm,         ionode_id )
        CALL mp_bcast( nberrycyc,     ionode_id )
        CALL mp_bcast( saverho,       ionode_id )
        CALL mp_bcast( lecrpa,        ionode_id )
-       CALL mp_bcast( vdw_table_name,        ionode_id )
+       CALL mp_bcast( vdw_table_name,ionode_id )
+       CALL mp_bcast( memory,        ionode_id )
        !
        RETURN
        !
@@ -845,8 +852,10 @@ MODULE read_namelists_module
        CALL mp_bcast( lda_plus_U,             ionode_id )
        CALL mp_bcast( lda_plus_u_kind,        ionode_id )
        CALL mp_bcast( Hubbard_U,              ionode_id )
+       CALL mp_bcast( Hubbard_J0,             ionode_id )
        CALL mp_bcast( Hubbard_J,              ionode_id )
        CALL mp_bcast( Hubbard_alpha,          ionode_id )
+       CALL mp_bcast( Hubbard_beta,           ionode_id )
        CALL mp_bcast( step_pen,               ionode_id )
        CALL mp_bcast( A_pen,                  ionode_id )
        CALL mp_bcast( sigma_pen,              ionode_id )
@@ -939,7 +948,7 @@ MODULE read_namelists_module
        !
        CALL mp_bcast( env_pressure,               ionode_id )
        !
-       CALL mp_bcast( cion,                       ionode_id )
+       CALL mp_bcast( env_ioncc_concentration,    ionode_id )
        CALL mp_bcast( zion,                       ionode_id )
        CALL mp_bcast( rhopb,                      ionode_id )
        CALL mp_bcast( solvent_temperature,        ionode_id )
@@ -1005,6 +1014,7 @@ MODULE read_namelists_module
        CALL mp_bcast( emass_cutoff,         ionode_id )
        CALL mp_bcast( orthogonalization,    ionode_id )
        CALL mp_bcast( electron_maxstep,     ionode_id )
+       CALL mp_bcast( scf_must_converge,    ionode_id )
        CALL mp_bcast( ortho_eps,            ionode_id )
        CALL mp_bcast( ortho_max,            ionode_id )
        CALL mp_bcast( electron_dynamics,    ionode_id )
@@ -1392,12 +1402,16 @@ MODULE read_namelists_module
        IF( refg < 0 ) &
          CALL errore( sub_name, ' wrong table interval refg ', 1 )
        !
-#ifdef __LOWMEM
-       IF( wf_collect .EQ. .true. ) &
-         CALL errore( sub_name, ' wf_collect = .true. is not allowed with LOWMEM build ', 1 )
-       IF( prog /= 'CP' ) &
-         CALL errore( sub_name, ' LOWMEM not available in '//prog//' yet ', 1 )
-#endif
+       IF( ( prog == 'CP' ) .AND. ( TRIM(memory) == 'small' ) .AND. wf_collect ) &
+         CALL errore( sub_name, ' wf_collect = .true. is not allowed with memory = small ', 1 )
+
+       allowed = .FALSE.
+       DO i = 1, SIZE( memory_allowed )
+          IF( TRIM(memory) == memory_allowed(i) ) allowed = .TRUE.
+       END DO
+       IF( .NOT. allowed ) &
+          CALL errore( sub_name, ' memory '''// &
+                       & TRIM(memory)//''' not allowed ',1)
 
        RETURN
        !
@@ -1967,7 +1981,7 @@ MODULE read_namelists_module
                TRIM( calculation ) == 'cp-wf-nscf' .OR. &   !Lingzhu Kong
                TRIM( calculation ) == 'cp-wf-pbe0' .OR. &   !Lingzhu Kong
                TRIM( calculation ) == 'pbe0-nscf'  .OR. &   !Lingzhu Kong
-               TRIM( calculation ) == 'cp-wf' ) READ( 5, ions, iostat = ios )
+               TRIM( calculation ) == 'cp-wf' ) READ( unit_loc, ions, iostat = ios )
   
        END IF
        CALL mp_bcast( ios, ionode_id )
@@ -2018,7 +2032,7 @@ MODULE read_namelists_module
        !
        IF ( do_environ ) THEN
           ios = 0
-          IF( ionode ) READ( 5, environ, iostat = ios )
+          IF( ionode ) READ( unit_loc, environ, iostat = ios )
           CALL mp_bcast( ios, ionode_id )
           IF( ios /= 0 ) CALL errore( ' read_namelists ', &
                                     & ' reading namelist environ ', ABS(ios) )
