@@ -32,11 +32,7 @@ CONTAINS
 !=----------------------------------------------------------------------=
 !
    !-----------------------------------------------------------------------
-#ifdef __LOWMEM
-   SUBROUTINE ggen ( gamma_only, at, bg, comm )
-#else
-   SUBROUTINE ggen ( gamma_only, at, bg )
-#endif
+   SUBROUTINE ggen ( gamma_only, at, bg, comm, no_global_sort )
    !----------------------------------------------------------------------
    !
    !     This routine generates all the reciprocal lattice vectors
@@ -50,35 +46,63 @@ CONTAINS
    !
    LOGICAL,  INTENT(IN) :: gamma_only
    REAL(DP), INTENT(IN) :: at(3,3), bg(3,3)
-#ifdef __LOWMEM
-   INTEGER,  INTENT(IN) :: comm
-#endif
+   INTEGER,  OPTIONAL, INTENT(IN) :: comm
+   LOGICAL,  OPTIONAL, INTENT(IN) :: no_global_sort
+   !  if no_global_sort is present (and it is true) G vectors are sorted only locally and not globally.
+   !  In this case no global array should be allocated and sorted (save memory
+   !  and a lot of time for large systems).
+   !
    !     here a few local variables
    !
    REAL(DP) ::  t (3), tt
-   INTEGER :: ngm_, ngms_, n1, n2, n3, n1s, n2s, n3s, ngm_offset
+   INTEGER :: ngm_save, ngms_save, n1, n2, n3, n1s, n2s, n3s, ngm_offset, ngm_max, ngms_max
    !
    REAL(DP), ALLOCATABLE :: g2sort_g(:)
    ! array containing all g vectors, on all processors: replicated data
-   ! when compiling with __LOWMEM only g vectors for the current processor are stored
+   ! when no_global_sort is present (and it is true) only g vectors for the current processor are stored
    INTEGER, ALLOCATABLE :: mill_g(:,:), mill_unsorted(:,:)
    ! array containing all g vectors generators, on all processors: replicated data
-   ! when compiling with __LOWMEM only g vectors for the current processor are stored
+   ! when no_global_sort is present (and it is true) only g vectors for the current processor are stored
    INTEGER, ALLOCATABLE :: igsrt(:)
    !
-#ifdef __MPI
    INTEGER :: m1, m2, mc
-#endif
    INTEGER :: ni, nj, nk, i, j, k, ipol, ng, igl, indsw
    INTEGER :: mype, npe
+   LOGICAL :: global_sort
    INTEGER, ALLOCATABLE :: ngmpe(:)
    !
-#ifdef __LOWMEM
-   mype = mp_rank( comm )
-   npe  = mp_size( comm )
-   ALLOCATE( ngmpe( npe ) )
-   ngmpe = 0
-#endif
+   IF( PRESENT( no_global_sort ) .AND. .NOT. PRESENT( comm ) ) THEN
+      CALL errore ('ggen', ' wrong subroutine arguments, communicator is missing ', 1)
+   END IF
+   IF( .NOT. PRESENT( no_global_sort ) .AND. PRESENT( comm ) ) THEN
+      CALL errore ('ggen', ' wrong subroutine arguments, parameter no_global_sort is missing ', 1)
+   END IF
+   !
+   global_sort = .TRUE.
+   !
+   IF( PRESENT( no_global_sort ) ) THEN
+      global_sort = .NOT. no_global_sort
+   END IF
+   !
+   IF( .NOT. global_sort ) THEN
+      mype = mp_rank( comm )
+      npe  = mp_size( comm )
+      ALLOCATE( ngmpe( npe ) )
+      ngmpe = 0
+      ngm_max = ngm
+      ngms_max = ngms
+   ELSE
+      ngm_max = ngm_g
+      ngms_max = ngms_g
+   END IF
+   !
+   ! save current value of ngm and ngms
+   !
+   ngm_save  = ngm
+   ngms_save = ngms
+   !
+   ngm = 0
+   ngms = 0
    !
    ! counters
    !
@@ -90,25 +114,12 @@ CONTAINS
    !
    !    and computes all the g vectors inside a sphere
    !
-#ifdef __LOWMEM
-   ALLOCATE( mill_g( 3, ngm ),mill_unsorted( 3, ngm ) )
-   ALLOCATE( igsrt( ngm ) )
-   ALLOCATE( g2sort_g( ngm ) )
-#else
-   ALLOCATE( mill_g( 3, ngm_g ),mill_unsorted( 3, ngm_g ) )
-   ALLOCATE( igsrt( ngm_g ) )
-   ALLOCATE( g2sort_g( ngm_g ) )
-#endif
+   ALLOCATE( mill_g( 3, ngm_max ),mill_unsorted( 3, ngm_max ) )
+   ALLOCATE( igsrt( ngm_max ) )
+   ALLOCATE( g2sort_g( ngm_max ) )
+   !
    g2sort_g(:) = 1.0d20
    !
-   ! save current value of ngm and ngms
-   !
-   ngm_  = ngm
-   ngms_ = ngms
-   !
-   ngm = 0
-   ngms = 0
-   !
    ! max miller indices (same convention as in module stick_set)
    !
    ni = (dfftp%nr1-1)/2
@@ -126,14 +137,15 @@ CONTAINS
          !
          IF ( gamma_only .and. i == 0 .and. j < 0) CYCLE jloop
 
-#if defined (__MPI) && defined (__LOWMEM)
-         m1 = mod (i, dfftp%nr1) + 1
-         IF (m1 < 1) m1 = m1 + dfftp%nr1
-         m2 = mod (j, dfftp%nr2) + 1
-         IF (m2 < 1) m2 = m2 + dfftp%nr2
-         mc = m1 + (m2 - 1) * dfftp%nr1x
-         IF ( dfftp%isind ( mc ) == 0) CYCLE jloop
-#endif
+         IF( .NOT. global_sort ) THEN
+            m1 = mod (i, dfftp%nr1) + 1
+            IF (m1 < 1) m1 = m1 + dfftp%nr1
+            m2 = mod (j, dfftp%nr2) + 1
+            IF (m2 < 1) m2 = m2 + dfftp%nr2
+            mc = m1 + (m2 - 1) * dfftp%nr1x
+            IF ( dfftp%isind ( mc ) == 0) CYCLE jloop
+         END IF
+
          kloop: DO k = -nk, nk
             !
             ! gamma-only: exclude line with x = 0, y = 0, z < 0
@@ -144,11 +156,7 @@ CONTAINS
             IF (tt <= gcutm) THEN
                ngm = ngm + 1
                IF (tt <= gcutms) ngms = ngms + 1
-#ifdef __LOWMEM
-               IF (ngm > ngm_) CALL errore ('ggen', 'too many g-vectors', ngm)
-#else
-               IF (ngm > ngm_g) CALL errore ('ggen', 'too many g-vectors', ngm)
-#endif
+               IF (ngm > ngm_max) CALL errore ('ggen', 'too many g-vectors', ngm)
                mill_unsorted( :, ngm ) = (/ i,j,k /)
                IF ( tt > eps8 ) THEN
                   g2sort_g(ngm) = tt
@@ -160,62 +168,54 @@ CONTAINS
       ENDDO jloop
    ENDDO iloop
 
-#ifdef __LOWMEM
-   ngmpe( mype + 1 ) = ngm
-   CALL mp_sum( ngmpe, comm )
-   IF (ngm  /= ngm_ ) &
-         CALL errore ('ggen', 'g-vectors missing !', abs(ngm - ngm_))
-   IF (ngms /= ngms_) &
-         CALL errore ('ggen', 'smooth g-vectors missing !', abs(ngms - ngms_))
-#else
-   IF (ngm  /= ngm_g ) &
-         CALL errore ('ggen', 'g-vectors missing !', abs(ngm - ngm_g))
-   IF (ngms /= ngms_g) &
-         CALL errore ('ggen', 'smooth g-vectors missing !', abs(ngms - ngms_g))
-#endif
-
+   IF( .NOT. global_sort ) THEN
+      ngmpe( mype + 1 ) = ngm
+      CALL mp_sum( ngmpe, comm )
+   END IF
+   IF (ngm  /= ngm_max) &
+         CALL errore ('ggen', 'g-vectors missing !', abs(ngm - ngm_max))
+   IF (ngms /= ngms_max) &
+         CALL errore ('ggen', 'smooth g-vectors missing !', abs(ngms - ngms_max))
 
    igsrt(1) = 0
-#ifdef __LOWMEM
-   CALL hpsort_eps( ngm, g2sort_g, igsrt, eps8 )
-#else
-   CALL hpsort_eps( ngm_g, g2sort_g, igsrt, eps8 )
-#endif
+   IF( .NOT. global_sort ) THEN
+      CALL hpsort_eps( ngm, g2sort_g, igsrt, eps8 )
+   ELSE
+      CALL hpsort_eps( ngm_g, g2sort_g, igsrt, eps8 )
+   END IF
    mill_g(1,:) = mill_unsorted(1,igsrt(:))
    mill_g(2,:) = mill_unsorted(2,igsrt(:))
    mill_g(3,:) = mill_unsorted(3,igsrt(:))
    DEALLOCATE( g2sort_g, igsrt, mill_unsorted )
 
-#ifdef __LOWMEM
-   ! compute adeguate offsets in order to avoid overlap between
-   ! g vectors once they are gathered on a single (global) array
-   !
-   ngm_offset = 0
-   DO ng = 1, mype
-      ngm_offset = ngm_offset + ngmpe( ng )
-   END DO
-#endif
+   IF( .NOT. global_sort ) THEN
+      ! compute adeguate offsets in order to avoid overlap between
+      ! g vectors once they are gathered on a single (global) array
+      !
+      ngm_offset = 0
+      DO ng = 1, mype
+         ngm_offset = ngm_offset + ngmpe( ng )
+      END DO
+   END IF
 
    ngm = 0
    ngms = 0
    !
-#ifdef __LOWMEM
-   ngloop: DO ng = 1, ngm_
-#else
-   ngloop: DO ng = 1, ngm_g
-#endif
+   ngloop: DO ng = 1, ngm_max
 
       i = mill_g(1, ng)
       j = mill_g(2, ng)
       k = mill_g(3, ng)
 
-#if defined (__MPI) && !defined (__LOWMEM)
-      m1 = mod (i, dfftp%nr1) + 1
-      IF (m1 < 1) m1 = m1 + dfftp%nr1
-      m2 = mod (j, dfftp%nr2) + 1
-      IF (m2 < 1) m2 = m2 + dfftp%nr2
-      mc = m1 + (m2 - 1) * dfftp%nr1x
-      IF ( dfftp%isind ( mc ) == 0) CYCLE ngloop
+#if defined(__MPI)
+      IF( global_sort ) THEN
+         m1 = mod (i, dfftp%nr1) + 1
+         IF (m1 < 1) m1 = m1 + dfftp%nr1
+         m2 = mod (j, dfftp%nr2) + 1
+         IF (m2 < 1) m2 = m2 + dfftp%nr2
+         mc = m1 + (m2 - 1) * dfftp%nr1x
+         IF ( dfftp%isind ( mc ) == 0) CYCLE ngloop
+      END IF
 #endif
 
       ngm = ngm + 1
@@ -223,21 +223,23 @@ CONTAINS
       !  Here map local and global g index !!!
       !  N.B. the global G vectors arrangement depends on the number of processors
       !
-#if defined (__LOWMEM)
-      ig_l2g( ngm ) = ng + ngm_offset
-#else
-      ig_l2g( ngm ) = ng
-#endif
+      IF( .NOT. global_sort ) THEN
+         ig_l2g( ngm ) = ng + ngm_offset
+      ELSE
+         ig_l2g( ngm ) = ng
+      END IF
 
       g (1:3, ngm) = i * bg (:, 1) + j * bg (:, 2) + k * bg (:, 3)
       gg (ngm) = sum(g (1:3, ngm)**2)
 
       IF (gg (ngm) <= gcutms) ngms = ngms + 1
-      IF (ngm > ngm_) CALL errore ('ggen', 'too many g-vectors', ngm)
+      IF (ngm > ngm_save) CALL errore ('ggen', 'too many g-vectors', ngm)
    ENDDO ngloop
 
-   IF (ngm /= ngm_) &
-      CALL errore ('ggen', 'g-vectors missing !', abs(ngm - ngm_))
+   IF (ngm /= ngm_save) &
+      CALL errore ('ggen', 'g-vectors (ngm) missing !', abs(ngm - ngm_save))
+   IF (ngms /= ngms_save) &
+      CALL errore ('ggen', 'g-vectors (ngms) missing !', abs(ngm - ngms_save))
    !
    !     determine first nonzero g vector
    !
@@ -396,7 +398,7 @@ SUBROUTINE gshells ( vc )
          ENDIF
       ENDDO
 
-      IF (igl /= ngl) CALL errore ('setup', 'igl <> ngl', ngl)
+      IF (igl /= ngl) CALL errore ('gshells', 'igl <> ngl', ngl)
 
    ENDIF
 
