@@ -1,5 +1,5 @@
-!j
-! Copyright (C) 2008-2011 Quantum ESPRESSO group
+!
+! Copyright (C) 2008-2012 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -36,15 +36,17 @@ MODULE write_upf_v2_module
 CONTAINS
 
   !-------------------------------+
-  SUBROUTINE write_upf_v2(u, upf, conf) !
+  SUBROUTINE write_upf_v2(u, upf, conf, u_input)
     !----------------------------+
     ! Write pseudopotential in UPF format version 2, uses iotk
     !
     IMPLICIT NONE
-    INTEGER,INTENT(IN)                    :: u   ! i/o unit
-    TYPE(pseudo_upf),INTENT(IN)           :: upf ! the pseudo data
+    INTEGER,INTENT(IN)          :: u   ! unit for writing
+    TYPE(pseudo_upf),INTENT(IN) :: upf ! the pseudo data
     ! optional: configuration used to generate the pseudopotential
     TYPE(pseudo_config),OPTIONAL,INTENT(IN) :: conf
+    ! optional: unit pointing to input file containing generation data
+    INTEGER, OPTIONAL, INTENT(IN) :: u_input
     !
     CHARACTER(len=iotk_attlenx) :: attr
     !
@@ -53,8 +55,7 @@ CONTAINS
     CALL iotk_open_write(u, attr=attr, root='UPF', skip_head=.true.)
     !
     ! Write human-readable header
-    CALL write_info(u, upf, conf)
-    !
+    CALL write_info(u, upf, conf, u_input)
     ! Write machine-readable header
     CALL write_header(u, upf)
     ! Write radial grid mesh
@@ -92,21 +93,22 @@ CONTAINS
 
   CONTAINS
     !
-    SUBROUTINE write_info(u, upf, conf)
+    SUBROUTINE write_info(u, upf, conf, u_input)
       ! Write human-readable header
       ! The header is written directly, not via iotk
       IMPLICIT NONE
-      INTEGER,INTENT(IN)          :: u    ! i/o unit
+      INTEGER,INTENT(IN)          :: u    ! i/o unit: write to unit u
       TYPE(pseudo_upf),INTENT(IN) :: upf  ! the pseudo data
       ! optional: configuration used to generate the pseudopotential
       TYPE(pseudo_config),OPTIONAL,INTENT(IN) :: conf
+      INTEGER, OPTIONAL, INTENT(IN) :: u_input ! read input data from u_input
       !
       INTEGER :: nb ! aux counter
       INTEGER :: ierr ! /= 0 if something went wrong
+      CHARACTER(len=256) :: line
+      LOGICAL :: opnd
       !
       CALL iotk_write_begin(u,'PP_INFO')
-      ! All the section has to fit in a comment, otherwise iotk will complain:
-      !WRITE(u, '(2x,a)', err=100) '<!--'
       !
       WRITE(u, '(4x,a)', err=100) TRIM(CHECK(upf%generated))
       WRITE(u, '(4x,a)', err=100) &
@@ -192,7 +194,24 @@ CONTAINS
          WRITE(u, '(4x,"Comment:",/,4x,a)', err=100) TRIM(CHECK(upf%comment))
       END IF
       !
-      !WRITE(u, '(2x,a)', err=100) '-->'
+      IF ( PRESENT(u_input) ) THEN
+         !
+         ! copy content of input file used in pseudopotential generation
+         !
+         INQUIRE (unit=u_input, opened=opnd)
+         IF (opnd) THEN
+            WRITE (u,'("<PP_INPUTFILE>")')
+            REWIND (unit=u_input)
+10          READ (u_input, '(A)',end=20,err=25) line
+            WRITE (u, '(A)') TRIM(CHECK(line))
+            GO TO 10
+25          CALL infomsg('write_upf_v2::write_inputfile', 'problem writing input data')
+20          WRITE (u,'("</PP_INPUTFILE>")')
+         ELSE
+            CALL infomsg('write_upf_v2::write_inputfile', 'input file not open')
+         END IF
+         !
+      END IF
       !
       CALL iotk_write_end(u,'PP_INFO')
       CALL iotk_write_comment(u,'                               ')
@@ -201,8 +220,10 @@ CONTAINS
       !
       RETURN
 100   CALL errore('write_upf_v2::write_info', 'Writing pseudo file', 1)
+      !
     END SUBROUTINE write_info
     !
+    !
     SUBROUTINE write_header(u, upf)
       IMPLICIT NONE
       INTEGER,INTENT(IN)           :: u    ! i/o unit
@@ -556,22 +577,6 @@ CONTAINS
    CALL iotk_write_dat(u, 'PP_AE_VLOC', upf%paw%ae_vloc,columns=4)
    !
    CALL iotk_write_end(u, 'PP_PAW')
-   !CORE DATA FOR GIPAW
-   !EMINE
-   CALL iotk_write_attr(attr, 'gipaw_data_format', upf%gipaw_data_format, first=.true.)
-   CALL iotk_write_begin(u, 'PP_GIPAW', attr=attr)
-   CALL iotk_write_attr(attr, 'number_of_core_orbitals', upf%gipaw_ncore_orbitals, first=.true.)
-   CALL iotk_write_begin(u, 'PP_GIPAW_CORE_ORBITALS', attr=attr)
-   DO nb = 1,upf%gipaw_ncore_orbitals
-      CALL iotk_write_attr(attr, 'index', nb, first=.true.)
-      CALL iotk_write_attr(attr, 'label', upf%gipaw_core_orbital_el(nb))
-      CALL iotk_write_attr(attr, 'n',     upf%gipaw_core_orbital_n(nb))
-      CALL iotk_write_attr(attr, 'l',     upf%gipaw_core_orbital_l(nb))
-      CALL iotk_write_dat(u, 'PP_GIPAW_CORE_ORBITAL'//iotk_index(nb), &
-           upf%gipaw_core_orbital(:,nb), columns=4, attr=attr)
-   ENDDO
-   CALL iotk_write_end(u, 'PP_GIPAW_CORE_ORBITALS')
-   CALL iotk_write_end(u, 'PP_GIPAW')
    !
    RETURN
  END SUBROUTINE write_paw
@@ -585,7 +590,7 @@ CONTAINS
    CHARACTER(len=iotk_attlenx) :: attr
    !
    INTEGER :: nb
-   IF (.not. upf%has_gipaw .or. upf%paw_as_gipaw) RETURN
+   IF (.not. upf%has_gipaw) RETURN
 
    CALL iotk_write_attr(attr, 'gipaw_data_format', upf%gipaw_data_format, first=.true.)
    CALL iotk_write_begin(u, 'PP_GIPAW', attr=attr)
@@ -602,6 +607,12 @@ CONTAINS
    ENDDO
    CALL iotk_write_end(u, 'PP_GIPAW_CORE_ORBITALS')
    !
+   ! Only write core orbitals in the PAW as GIPAW case
+   IF (upf%paw_as_gipaw) THEN
+      CALL iotk_write_end(u, 'PP_GIPAW')
+      RETURN
+   ENDIF
+   !
    ! Write valence all-electron and pseudo orbitals
    CALL iotk_write_attr(attr, 'number_of_valence_orbitals', upf%gipaw_wfs_nchannels, first=.true.)
    CALL iotk_write_begin(u, 'PP_GIPAW_ORBITALS', attr=attr)
@@ -638,7 +649,12 @@ CONTAINS
  ! or iotk will complain while read-skipping PP_INFO section.
  FUNCTION CHECK(in) RESULT (out)
    CHARACTER(len=*) :: in
+#if defined(__PGI)
+   INTEGER, PARAMETER :: length = 255
+   CHARACTER(len=length) :: out
+#else
    CHARACTER(len=len(in)) :: out
+#endif   
    INTEGER :: i
    DO i = 1,len(in)
       IF ( in(i:i) == '<' .or. in(i:i) == '>' ) THEN
