@@ -853,8 +853,9 @@ MODULE xml_io_base
       INTEGER,          INTENT(IN) :: if_pos(:,:)
       REAL(DP),         INTENT(IN) :: pos_unit
       !
-      INTEGER            :: i, flen
-      CHARACTER(LEN=256) :: file_pseudo
+      INTEGER            :: i, flen, flen2
+      LOGICAL            :: exst
+      CHARACTER(LEN=256) :: file_pseudo_in, file_pseudo_out
       !
       !
       CALL iotk_write_begin( iunpun, "IONS" )
@@ -864,6 +865,7 @@ MODULE xml_io_base
       CALL iotk_write_dat( iunpun, "NUMBER_OF_SPECIES", nsp )
       !
       flen = LEN_TRIM( pseudo_dir )
+      flen2= LEN_TRIM( dirname )
       !
       CALL iotk_write_attr ( attr, "UNITS", "a.m.u.", FIRST = .TRUE. )
       CALL iotk_write_empty( iunpun, "UNITS_FOR_ATOMIC_MASSES", ATTR = attr )
@@ -874,20 +876,30 @@ MODULE xml_io_base
          !
          CALL iotk_write_dat( iunpun, "ATOM_TYPE", atm(i) )
          !
+         ! ... Copy PP file into the .save directory - verify that the
+         ! ...  sourcefile exists and is not the same as the destination file
+         !
          IF ( pseudo_dir(flen:flen) /= '/' ) THEN
-            !
-            file_pseudo = pseudo_dir(1:flen) // '/' // psfile(i)
-            !
+            file_pseudo_in = pseudo_dir(1:flen) // '/' // TRIM(psfile(i))
          ELSE
-            !
-            file_pseudo = pseudo_dir(1:flen) // psfile(i)
-            !
+            file_pseudo_in = pseudo_dir(1:flen) // TRIM(psfile(i))
          END IF
          !
-         IF (TRIM( file_pseudo ).ne. TRIM( dirname ) // "/" // &
-                           TRIM(psfile(i))) &
-         CALL copy_file( TRIM( file_pseudo ), &
-                            TRIM( dirname ) // "/" // TRIM( psfile(i) ) )
+         IF ( dirname(flen2:flen2) /= '/' ) THEN
+            file_pseudo_out = dirname(1:flen2) // '/' // TRIM(psfile(i))
+         ELSE
+            file_pseudo_out = dirname(1:flen2) // TRIM(psfile(i))
+         END IF
+         !
+         IF ( file_pseudo_in .ne. file_pseudo_out ) THEN
+            INQUIRE ( FILE=file_pseudo_in, EXIST=exst )
+            IF ( exst ) THEN
+               CALL copy_file( TRIM( file_pseudo_in ), TRIM( file_pseudo_out ) )
+            ELSE
+               CALL infomsg( 'write_ions', &
+                   'file ' // TRIM( file_pseudo_in) // ' not present' )
+            END IF
+         END IF
          !
          CALL iotk_write_dat( iunpun, "MASS", amass(i) )
          !
@@ -1202,18 +1214,20 @@ MODULE xml_io_base
     END SUBROUTINE write_magnetization
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_xc( dft, nsp, lda_plus_u, lda_plus_u_kind, Hubbard_lmax, &
-                         Hubbard_l, Hubbard_U, Hubbard_J, Hubbard_alpha,      &
-                         inlc, vdw_table_name, pseudo_dir, dirname )
+    SUBROUTINE write_xc( dft, nsp, lda_plus_u, lda_plus_u_kind, U_projection, Hubbard_lmax, &
+                         Hubbard_l, Hubbard_U, Hubbard_J, Hubbard_J0, Hubbard_beta, &
+                         Hubbard_alpha, inlc, vdw_table_name, pseudo_dir, dirname )
       !------------------------------------------------------------------------
       !
       CHARACTER(LEN=*),   INTENT(IN) :: dft
       LOGICAL,            INTENT(IN) :: lda_plus_u
       INTEGER,  OPTIONAL, INTENT(IN) :: lda_plus_u_kind
       INTEGER,  OPTIONAL, INTENT(IN) :: nsp
+      CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: U_projection
       INTEGER,  OPTIONAL, INTENT(IN) :: Hubbard_lmax
       INTEGER,  OPTIONAL, INTENT(IN) :: Hubbard_l(:)
-      REAL(DP), OPTIONAL, INTENT(IN) :: Hubbard_U(:), Hubbard_J(:,:), Hubbard_alpha(:)
+      REAL(DP), OPTIONAL, INTENT(IN) :: Hubbard_U(:), Hubbard_J(:,:), Hubbard_alpha(:), &
+                                        Hubbard_J0(:), Hubbard_beta(:)
       INTEGER,  OPTIONAL, INTENT(IN) :: inlc
       CHARACTER(LEN=*), OPTIONAL,   INTENT(IN) :: vdw_table_name, pseudo_dir, dirname
       !
@@ -1230,17 +1244,22 @@ MODULE xml_io_base
          !
          IF ( .NOT. PRESENT( Hubbard_lmax ) .OR. &
               .NOT. PRESENT( Hubbard_l )    .OR. & 
+              .NOT. PRESENT( U_projection ) .OR. &
               .NOT. PRESENT( Hubbard_U )    .OR. &
+              .NOT. PRESENT( Hubbard_J0 )    .OR. &
               .NOT. PRESENT( Hubbard_J )    .OR. &
               .NOT. PRESENT( nsp )          .OR. &
-              .NOT. PRESENT( Hubbard_alpha ) ) &
-            CALL errore( 'write_exchange_correlation', &
+              .NOT. PRESENT( Hubbard_alpha )    .OR. &
+              .NOT. PRESENT( Hubbard_beta ) ) &
+            CALL errore( 'write_xc', &
                          ' variables for LDA+U not present', 1 )
          !
          CALL iotk_write_dat( iunpun, "NUMBER_OF_SPECIES", nsp )
          !
          CALL iotk_write_dat( iunpun, "LDA_PLUS_U_KIND", lda_plus_u_kind )
          ! 
+         CALL iotk_write_dat( iunpun, "U_PROJECTION_TYPE", trim(U_projection) )
+         !
          CALL iotk_write_dat( iunpun, "HUBBARD_LMAX", Hubbard_lmax )
          !
          CALL iotk_write_dat( iunpun, "HUBBARD_L", &
@@ -1250,8 +1269,12 @@ MODULE xml_io_base
          !
          CALL iotk_write_dat( iunpun, "HUBBARD_J", Hubbard_J(1:3,1:nsp), COLUMNS = 3 )
          !
+         CALL iotk_write_dat( iunpun, "HUBBARD_J0", Hubbard_J0(1:nsp) )
+         !
          CALL iotk_write_dat( iunpun, "HUBBARD_ALPHA", Hubbard_alpha(1:nsp) )
          !
+         CALL iotk_write_dat( iunpun, "HUBBARD_BETA", Hubbard_beta(1:nsp) )
+         !
       END IF
 
       !
@@ -1264,7 +1287,7 @@ MODULE xml_io_base
          IF ( .NOT. PRESENT( vdw_table_name ) .OR. &
               .NOT. PRESENT( pseudo_dir ) .OR. &
               .NOT. PRESENT( dirname )) &
-            CALL errore( 'write_exchange_correlation', &
+            CALL errore( 'write_xc', &
                          ' variable vdw_table_name not present', 1 )
         
          CALL iotk_write_dat( iunpun, "VDW_KERNEL_NAME", vdw_table_name )
@@ -1450,10 +1473,12 @@ MODULE xml_io_base
     END SUBROUTINE write_bz
     !
     !------------------------------------------------------------------------
-    SUBROUTINE write_para( kunit, nproc, nproc_pool, nproc_image )
+    SUBROUTINE write_para( kunit, nproc, nproc_pool, nproc_image, &
+                    ntask_groups, nproc_pot, nproc_bgrp, nproc_ortho ) 
       !------------------------------------------------------------------------
       !
-      INTEGER,  INTENT(IN) :: kunit, nproc, nproc_pool, nproc_image
+      INTEGER,  INTENT(IN) :: kunit, nproc, nproc_pool, nproc_image, &
+                              ntask_groups, nproc_pot, nproc_bgrp, nproc_ortho 
       !
       !
       CALL iotk_write_begin( iunpun, "PARALLELISM" )
@@ -1464,6 +1489,14 @@ MODULE xml_io_base
                               "NUMBER_OF_PROCESSORS_PER_POOL", nproc_pool )
       CALL iotk_write_dat( iunpun, &
                               "NUMBER_OF_PROCESSORS_PER_IMAGE", nproc_image )
+      CALL iotk_write_dat( iunpun, "NUMBER_OF_PROCESSORS_PER_TASKGROUP", &
+                                              ntask_groups )
+      CALL iotk_write_dat( iunpun, "NUMBER_OF_PROCESSORS_PER_POT", &
+                                              nproc_pot )
+      CALL iotk_write_dat( iunpun, "NUMBER_OF_PROCESSORS_PER_BAND_GROUP", &
+                                              nproc_bgrp )
+      CALL iotk_write_dat( iunpun, "NUMBER_OF_PROCESSORS_PER_DIAGONALIZATION", &
+                                              nproc_ortho )
       CALL iotk_write_end( iunpun, "PARALLELISM" )
       !
       !
@@ -1623,17 +1656,16 @@ MODULE xml_io_base
     END SUBROUTINE write_rho_xml
     !
     !------------------------------------------------------------------------
-    SUBROUTINE read_rho_xml( rho_file_base, rho, &
-                             nr1, nr2, nr3, nr1x, nr2x, ipp, npp, &
-                             ionode, intra_group_comm, inter_group_comm )
+    SUBROUTINE read_rho_xml( rho_file_base, nr1, nr2, nr3, nr1x, nr2x, &
+                             ipp, npp, rho )
       !------------------------------------------------------------------------
       !
-      ! ... Reads charge density rho, one plane at a time.
-      ! ... If ipp and npp are specified, planes are collected one by one from
-      ! ... all processors, avoiding an overall collect of the charge density
-      ! ... on a single proc.
+      ! ... Reads charge density rho, one plane at a time, to avoid 
+      ! ... collecting the entire charge density on a single processor
       !
       USE io_files,  ONLY : rhounit
+      USE io_global, ONLY : ionode, ionode_id
+      USE mp_global, ONLY : intra_bgrp_comm, intra_image_comm
       USE mp,        ONLY : mp_put, mp_sum, mp_rank, mp_size
       !
       IMPLICIT NONE
@@ -1642,23 +1674,19 @@ MODULE xml_io_base
       INTEGER,           INTENT(IN)  :: nr1, nr2, nr3
       INTEGER,           INTENT(IN)  :: nr1x, nr2x
       REAL(DP),          INTENT(OUT) :: rho(:)
-      INTEGER, OPTIONAL, INTENT(IN)  :: ipp(:)
-      INTEGER, OPTIONAL, INTENT(IN)  :: npp(:)
-      LOGICAL,           INTENT(IN)  :: ionode
-      INTEGER,           INTENT(IN)  :: intra_group_comm, inter_group_comm
+      INTEGER,           INTENT(IN)  :: ipp(:)
+      INTEGER,           INTENT(IN)  :: npp(:)
       !
       INTEGER               :: ierr, i, j, k, kk, ldr, ip
       INTEGER               :: nr( 3 )
+      INTEGER               :: me_group, nproc_group
       CHARACTER(LEN=256)    :: rho_file
       REAL(DP), ALLOCATABLE :: rho_plane(:)
       INTEGER,  ALLOCATABLE :: kowner(:)
       LOGICAL               :: exst
-      INTEGER               :: ngroup, my_group_id, me_group, nproc_group, io_group_id, io_group
       !
-      me_group    = mp_rank( intra_group_comm )
-      nproc_group = mp_size( intra_group_comm )
-      my_group_id = mp_rank( inter_group_comm )
-      ngroup      = mp_size( inter_group_comm )
+      me_group     = mp_rank ( intra_bgrp_comm )
+      nproc_group  = mp_size ( intra_bgrp_comm )
       !
       rho_file = TRIM( rho_file_base ) // ".dat"
       exst = check_file_exst( TRIM(rho_file) ) 
@@ -1695,26 +1723,6 @@ MODULE xml_io_base
       ALLOCATE( rho_plane( nr1*nr2 ) )
       ALLOCATE( kowner( nr3 ) )
       !
-      ! ... find the index of the pool that will write rho
-      !
-      io_group_id = 0
-      !
-      IF ( ionode ) io_group_id = my_group_id
-      !
-      CALL mp_sum( io_group_id, intra_group_comm )
-      CALL mp_sum( io_group_id, inter_group_comm )
-      !
-      ! ... find the index of the ionode within its own pool
-      !
-      io_group = 0
-      !
-      IF ( ionode ) io_group = me_group
-      !
-      CALL mp_sum( io_group, intra_group_comm )
-      CALL mp_sum( io_group, inter_group_comm )
-      !
-      ! ... find out the owner of each "z" plane
-      !
       DO ip = 1, nproc_group
          !
          kowner((ipp(ip)+1):(ipp(ip)+npp(ip))) = ip - 1
@@ -1737,36 +1745,15 @@ MODULE xml_io_base
          !
          ! ... planes are sent to the destination processor
          !
-         IF( ngroup > 1 ) THEN
-            !
-            !  send to all proc/pools
-            !
-            IF( io_group_id == my_group_id ) THEN
-               CALL mp_bcast( rho_plane, io_group, intra_group_comm )
-            END IF
-            CALL mp_bcast( rho_plane, io_group_id, inter_group_comm )
-            !
-         ELSE
-            !
-            !  send to the destination proc
-            !
-            IF ( kowner(k) /= io_group ) &
-               CALL mp_put( rho_plane, rho_plane, me_group, io_group, kowner(k), k, intra_group_comm )
-            !
-         END IF
+         CALL mp_bcast( rho_plane, ionode_id, intra_image_comm )
          !
          IF( kowner(k) == me_group ) THEN
             !
             kk = k - ipp( me_group + 1 )
-            ! 
             DO j = 1, nr2
-               !
                DO i = 1, nr1
-                  !
                   rho(i+(j-1)*nr1x+(kk-1)*ldr) = rho_plane(i+(j-1)*nr1)
-                  !
                END DO
-               !
             END DO
             !
          END IF
@@ -1788,6 +1775,7 @@ MODULE xml_io_base
       !
     END SUBROUTINE read_rho_xml
     !
+    !------------------------------------------------------------------------
     ! ... methods to write and read wavefunctions
     !
     !------------------------------------------------------------------------
@@ -2096,5 +2084,4 @@ MODULE xml_io_base
       !
     END SUBROUTINE write_eig
          
-
 END MODULE xml_io_base
