@@ -56,14 +56,15 @@ SUBROUTINE chdens (filplot,plot_num)
   real(DP), ALLOCATABLE :: aux(:)
 
   CHARACTER (len=256) :: fileout
-  CHARACTER (len=13), DIMENSION(0:6) :: formatname = &
+  CHARACTER (len=13), DIMENSION(0:7) :: formatname = &
        (/ 'gnuplot      ', &
           'contour.x    ', &
           'plotrho.x    ', &
           'XCrySDen     ', &
           'gOpenMol     ', &
           'XCrySDen     ', &
-          'Gaussian cube' /)
+          'Gaussian cube', & 
+          'gnuplot x,y,f' /)
   CHARACTER (len=20), DIMENSION(0:4) :: plotname = &
        (/ '1D spherical average', &
           '1D along a line     ', &
@@ -199,9 +200,9 @@ SUBROUTINE chdens (filplot,plot_num)
   !
   IF (plot_num==-1) THEN
      IF (ionode) &
-        CALL read_io_header(filepp (1), title, dfftp%nr1x, dfftp%nr2x, dfftp%nr3x, dfftp%nr1, dfftp%nr2, &
-                dfftp%nr3, nat, ntyp, ibrav, celldm, at, gcutm, dual, ecutwfc, &
-                idum )
+        CALL read_io_header(filepp (1), title, dfftp%nr1x, dfftp%nr2x, &
+                dfftp%nr3x, dfftp%nr1, dfftp%nr2, dfftp%nr3, nat, ntyp,&
+                ibrav, celldm, at, gcutm, dual, ecutwfc, idum )
      CALL mp_bcast( title, ionode_id )
      CALL mp_bcast( dfftp%nr1x, ionode_id )
      CALL mp_bcast( dfftp%nr2x, ionode_id )
@@ -275,10 +276,12 @@ SUBROUTINE chdens (filplot,plot_num)
      IF (abs(gcutmsa-gcutm)>1.d-8.or.abs(duals-dual)>1.d-8.or.&
          abs(ecuts-ecutwfc)>1.d-8) &
           CALL errore ('chdens', 'incompatible gcutm or dual or ecut', 1)
-     DO i = 1, 6
-        IF (abs( celldm (i)-celldms (i) ) > 1.0d-7 ) CALL errore &
-             ('chdens', 'incompatible celldm', 1)
-     ENDDO
+     IF (ibravs /= 0 ) THEN
+        DO i = 1, 6
+           IF (abs( celldm (i)-celldms (i) ) > 1.0d-7 ) &
+              CALL errore ('chdens', 'incompatible celldm', 1)
+        ENDDO
+     ENDIF
      !
      rhor (:) = rhor (:) + weight (ifile) * rhos (:)
   ENDDO
@@ -364,7 +367,7 @@ SUBROUTINE chdens (filplot,plot_num)
                             &" not implemented for Gamma only!",/, &
                             &"SOLUTION: restart this calculation with", &
                             &" emtpy namelist &inputpp")')
-             CALL errore ('punch_plot','Not implemented, please read above',1)
+             CALL errore ('chdens','Not implemented, please read above',1)
         ENDIF
         !
      ENDIF
@@ -733,6 +736,17 @@ SUBROUTINE plot_2d (nx, ny, m1, m2, x0, e1, e2, ngm, g, rhog, alat, &
         !
         CALL xsf_struct (alat, at, nat, tau, atm, ityp, ounit)
         CALL xsf_datagrid_2d (carica, nx, ny, m1, m2, x0, e1, e2, alat, ounit)
+     ELSEIF (output_format == 7) THEN
+        !
+        !     gnuplot format : x, y, f(x,y)
+        !
+        DO i=1, nx
+           DO j=1, ny 
+              WRITE (ounit, '(3e20.8)')  alat*deltax * (i - 1), &
+                      alat*deltay * (j - 1), dble(carica(i,j))
+           ENDDO
+           WRITE(ounit, *)
+        ENDDO
      ELSE
         CALL errore('plot_2d', 'wrong output_format', 1)
      ENDIF
