@@ -62,17 +62,19 @@
 ! write_wfng  - generates complex wavefunctions in G-space (normalized to 1)
 ! real_wfng   - constructs real wavefunctions by applying the Gram-Schmidt
 !               process (called from write_wfng)
-! write_rhog  - generates complex charge density in G-space
+! write_rhog  - generates real/complex charge density in G-space
 !               (units of the number of electronic states per unit cell)
-! write_vxcg  - generates complex exchange-correlation potential in G-space
+! write_vxcg  - generates real/complex exchange-correlation potential in G-space
 !               (units of Rydberg) [only local part of Vxc]
-! write_vxc0  - prints complex exchange-correlation potential at G=0
+! write_vxc0  - prints real/complex exchange-correlation potential at G=0
 !               (units of eV) [only local part of Vxc]
 ! write_vxc_r - calculates matrix elements of exchange-correlation potential
 !               in R-space (units of eV) [only local part of Vxc]
 ! write_vxc_g - calculates matrix elements of exchange-correlation potential
 !               in G-space (units of eV) [supports non-local Vxc]
-! write_vnlg  - generates Kleinman-Bylander projectors in G-space
+! write_vscg  - generates real/complex self-consistent potential in G-space
+!               (units of Rydberg) [only local part of Vsc]
+! write_vkbg  - generates complex Kleinman-Bylander projectors in G-space
 !               (units of Rydberg)
 ! check_inversion - checks whether real/complex version is appropriate
 !
@@ -91,16 +93,18 @@
 
 PROGRAM pw2bgw
 
-  USE environment, ONLY : environment_start, environment_end
+  USE constants, ONLY : eps12
   USE control_flags, ONLY : gamma_only
+  USE environment, ONLY : environment_start, environment_end
   USE io_files, ONLY : prefix, tmp_dir, outdir
   USE io_global, ONLY : ionode, ionode_id
   USE kinds, ONLY : DP
+  USE lsda_mod, ONLY : nspin
   USE mp, ONLY : mp_bcast
   USE mp_global, ONLY : mp_startup
-  USE uspp, ONLY : okvan
   USE paw_variables, ONLY : okpaw
-  USE lsda_mod, ONLY : nspin
+  USE scf, ONLY : rho_core, rhog_core
+  USE uspp, ONLY : okvan
 
   IMPLICIT NONE
 
@@ -133,10 +137,11 @@ PROGRAM pw2bgw
   integer :: vxc_diag_nmax
   integer :: vxc_offdiag_nmin
   integer :: vxc_offdiag_nmax
-  character ( len = 20 ) :: input_dft
-  logical :: exx_flag
-  logical :: vnlg_flag
-  character ( len = 256 ) :: vnlg_file
+  logical :: vxc_zero_rho_core
+  logical :: vscg_flag
+  character ( len = 256 ) :: vscg_file
+  logical :: vkbg_flag
+  character ( len = 256 ) :: vkbg_file
 
   NAMELIST / input_pw2bgw / prefix, outdir, &
     real_or_complex, symm_type, wfng_flag, wfng_file, wfng_kgrid, &
@@ -144,8 +149,8 @@ PROGRAM pw2bgw
     wfng_occupation, wfng_nvmin, wfng_nvmax, rhog_flag, rhog_file, &
     vxcg_flag, vxcg_file, vxc0_flag, vxc0_file, vxc_flag, &
     vxc_file, vxc_integral, vxc_diag_nmin, vxc_diag_nmax, &
-    vxc_offdiag_nmin, vxc_offdiag_nmax, input_dft, exx_flag, &
-    vnlg_flag, vnlg_file
+    vxc_offdiag_nmin, vxc_offdiag_nmax, vxc_zero_rho_core, &
+    vscg_flag, vscg_file, vkbg_flag, vkbg_file
 
   integer :: ii, ios
   character ( len = 256 ) :: output_file_name
@@ -188,10 +193,11 @@ PROGRAM pw2bgw
   vxc_diag_nmax = 0
   vxc_offdiag_nmin = 0
   vxc_offdiag_nmax = 0
-  input_dft = 'sla+pz'
-  exx_flag = .FALSE.
-  vnlg_flag = .FALSE.
-  vnlg_file = 'VNL'
+  vxc_zero_rho_core = .FALSE.
+  vscg_flag = .FALSE.
+  vscg_file = 'VSC'
+  vkbg_flag = .FALSE.
+  vkbg_file = 'VKB'
 
   IF ( ionode ) THEN
     CALL input_from_file ( )
@@ -244,20 +250,31 @@ PROGRAM pw2bgw
   CALL mp_bcast ( vxc_diag_nmax, ionode_id )
   CALL mp_bcast ( vxc_offdiag_nmin, ionode_id )
   CALL mp_bcast ( vxc_offdiag_nmax, ionode_id )
-  CALL mp_bcast ( input_dft, ionode_id )
-  CALL mp_bcast ( exx_flag, ionode_id )
-  CALL mp_bcast ( vnlg_flag, ionode_id )
-  CALL mp_bcast ( vnlg_file, ionode_id )
+  CALL mp_bcast ( vxc_zero_rho_core, ionode_id )
+  CALL mp_bcast ( vscg_flag, ionode_id )
+  CALL mp_bcast ( vscg_file, ionode_id )
+  CALL mp_bcast ( vkbg_flag, ionode_id )
+  CALL mp_bcast ( vkbg_file, ionode_id )
 
   CALL read_file ( )
 
-  if(okvan) call errore ( 'pw2bgw', 'BGW cannot use ultrasoft pseudopotentials.', 3 )
-  if(okpaw) call errore ( 'pw2bgw', 'BGW cannot use PAW.', 4 )
-  if(gamma_only) call errore ( 'pw2bgw', 'BGW cannot use gamma-only run.', 5 )
-  if(nspin == 4) call errore ( 'pw2bgw', 'BGW cannot use spinors.', 6 )
+  if (ionode) then
+    if (MAX (MAXVAL (ABS (rho_core (:) ) ), MAXVAL (ABS (rhog_core (:) ) ) ) &
+      .LT. eps12) then
+      WRITE ( 6, '(/,5x,"NLCC is absent")' )
+    else
+      WRITE ( 6, '(/,5x,"NLCC is present")' )
+    endif
+  endif
+  if (okvan) call errore ( 'pw2bgw', 'BGW cannot use USPP.', 3 )
+  if (okpaw) call errore ( 'pw2bgw', 'BGW cannot use PAW.', 4 )
+  if (gamma_only) call errore ( 'pw2bgw', 'BGW cannot use gamma-only run.', 5 )
+  if (nspin == 4) call errore ( 'pw2bgw', 'BGW cannot use spinors.', 6 )
 
   CALL openfil_pp ( )
 
+  if ( ionode ) WRITE ( 6, '("")' )
+
   IF ( wfng_flag ) THEN
     output_file_name = TRIM ( outdir ) // '/' // TRIM ( wfng_file )
     IF ( ionode ) WRITE ( 6, '(5x,"call write_wfng")' )
@@ -283,7 +300,7 @@ PROGRAM pw2bgw
     IF ( ionode ) WRITE ( 6, '(5x,"call write_vxcg")' )
     CALL start_clock ( 'write_vxcg' )
     CALL write_vxcg ( output_file_name, real_or_complex, symm_type, &
-      input_dft, exx_flag )
+      vxc_zero_rho_core )
     CALL stop_clock ( 'write_vxcg' )
     IF ( ionode ) WRITE ( 6, '(5x,"done write_vxcg",/)' )
   ENDIF
@@ -292,7 +309,7 @@ PROGRAM pw2bgw
     output_file_name = TRIM ( outdir ) // '/' // TRIM ( vxc0_file )
     IF ( ionode ) WRITE ( 6, '(5x,"call write_vxc0")' )
     CALL start_clock ( 'write_vxc0' )
-    CALL write_vxc0 ( output_file_name, input_dft, exx_flag )
+    CALL write_vxc0 ( output_file_name, vxc_zero_rho_core )
     CALL stop_clock ( 'write_vxc0' )
     IF ( ionode ) WRITE ( 6, '(5x,"done write_vxc0",/)' )
   ENDIF
@@ -305,7 +322,7 @@ PROGRAM pw2bgw
       CALL write_vxc_r ( output_file_name, &
         vxc_diag_nmin, vxc_diag_nmax, &
         vxc_offdiag_nmin, vxc_offdiag_nmax, &
-        input_dft, exx_flag )
+        vxc_zero_rho_core )
       CALL stop_clock ( 'write_vxc_r' )
       IF ( ionode ) WRITE ( 6, '(5x,"done write_vxc_r",/)' )
     ENDIF
@@ -315,20 +332,29 @@ PROGRAM pw2bgw
       CALL write_vxc_g ( output_file_name, &
         vxc_diag_nmin, vxc_diag_nmax, &
         vxc_offdiag_nmin, vxc_offdiag_nmax, &
-        input_dft, exx_flag )
+        vxc_zero_rho_core )
       CALL stop_clock ( 'write_vxc_g' )
       IF ( ionode ) WRITE ( 6, '(5x,"done write_vxc_g",/)' )
     ENDIF
   ENDIF
 
-  IF ( vnlg_flag ) THEN
-    output_file_name = TRIM ( outdir ) // '/' // TRIM ( vnlg_file )
-    IF ( ionode ) WRITE ( 6, '(5x,"call write_vnlg")' )
-    CALL start_clock ( 'write_vnlg' )
-    CALL write_vnlg ( output_file_name, symm_type, wfng_kgrid, wfng_nk1, &
+  IF ( vscg_flag ) THEN
+    output_file_name = TRIM ( outdir ) // '/' // TRIM ( vscg_file )
+    IF ( ionode ) WRITE ( 6, '(5x,"call write_vscg")' )
+    CALL start_clock ( 'write_vscg' )
+    CALL write_vscg ( output_file_name, real_or_complex, symm_type )
+    CALL stop_clock ( 'write_vscg' )
+    IF ( ionode ) WRITE ( 6, '(5x,"done write_vscg",/)' )
+  ENDIF
+
+  IF ( vkbg_flag ) THEN
+    output_file_name = TRIM ( outdir ) // '/' // TRIM ( vkbg_file )
+    IF ( ionode ) WRITE ( 6, '(5x,"call write_vkbg")' )
+    CALL start_clock ( 'write_vkbg' )
+    CALL write_vkbg ( output_file_name, symm_type, wfng_kgrid, wfng_nk1, &
       wfng_nk2, wfng_nk3, wfng_dk1, wfng_dk2, wfng_dk3 )
-    CALL stop_clock ( 'write_vnlg' )
-    IF ( ionode ) WRITE ( 6, '(5x,"done write_vnlg",/)' )
+    CALL stop_clock ( 'write_vkbg' )
+    IF ( ionode ) WRITE ( 6, '(5x,"done write_vkbg",/)' )
   ENDIF
 
   IF ( ionode ) WRITE ( 6, * )
@@ -340,7 +366,8 @@ PROGRAM pw2bgw
     IF ( vxc_integral .EQ. 'r' ) CALL print_clock ( 'write_vxc_r' )
     IF ( vxc_integral .EQ. 'g' ) CALL print_clock ( 'write_vxc_g' )
   ENDIF
-  IF ( vnlg_flag ) CALL print_clock ( 'write_vnlg' )
+  IF ( vscg_flag ) CALL print_clock ( 'write_vscg' )
+  IF ( vkbg_flag ) CALL print_clock ( 'write_vkbg' )
   IF ( wfng_flag .AND. real_or_complex .EQ. 1 ) THEN
     IF ( ionode ) WRITE ( 6, '(/,5x,"Called by write_wfng:")' )
     CALL print_clock ( 'real_wfng' )
@@ -1366,14 +1393,13 @@ END SUBROUTINE write_rhog
 !-------------------------------------------------------------------------------
 
 SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
-  input_dft, exx_flag )
+  vxc_zero_rho_core )
 
   USE cell_base, ONLY : omega, alat, tpiba, tpiba2, at, bg, ibrav
   USE constants, ONLY : pi, tpi, eps6
   USE ener, ONLY : etxc, vtxc
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : fwfft
-  USE funct, ONLY : enforce_input_dft
   USE gvect, ONLY : ngm, ngm_g, ig_l2g, nl, mill, ecutrho
   USE io_global, ONLY : ionode
   USE ions_base, ONLY : nat, atm, ityp, tau 
@@ -1384,17 +1410,13 @@ SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
   USE scf, ONLY : rho, rho_core, rhog_core
   USE symm_base, ONLY : s, ftau, nsym
   USE wavefunctions_module, ONLY : psic
-#ifdef EXX
-  USE funct, ONLY : start_exx, stop_exx
-#endif
 
   IMPLICIT NONE
 
   character ( len = 256 ), intent (in) :: output_file_name
   integer, intent (in) :: real_or_complex
   character ( len = 9 ), intent (in) :: symm_type
-  character ( len = 20 ), intent (in) :: input_dft
-  logical, intent (in) :: exx_flag
+  logical, intent (in) :: vxc_zero_rho_core
 
   character :: cdate*9, ctime*9, sdate*32, stime*32, stitle*32
   integer :: unit, id, is, ir, ig, i, j, k, ierr
@@ -1522,27 +1544,17 @@ SUBROUTINE write_vxcg ( output_file_name, real_or_complex, symm_type, &
     ENDDO
   ENDDO
 
-! In GW calculations, the exchange-correlation
-! potential should not include partial core correction.
-! Assign rho_core and rhog_core to zero.
-
-  rho_core ( : ) = 0.0D0
-  rhog_core ( : ) = ( 0.0D0, 0.0D0 )
-
   DO ig = 1, ng_l
     g_g ( 1, ig_l2g ( ig ) ) = mill ( 1, ig )
     g_g ( 2, ig_l2g ( ig ) ) = mill ( 2, ig )
     g_g ( 3, ig_l2g ( ig ) ) = mill ( 3, ig )
   ENDDO
-  CALL enforce_input_dft ( input_dft )
-#ifdef EXX
-  IF ( exx_flag ) CALL start_exx ( )
-#endif
   vxcr_g ( :, : ) = 0.0D0
+  IF ( vxc_zero_rho_core ) THEN
+    rho_core ( : ) = 0.0D0
+    rhog_core ( : ) = ( 0.0D0, 0.0D0 )
+  ENDIF
   CALL v_xc ( rho, rho_core, rhog_core, etxc, vtxc, vxcr_g )
-#ifdef EXX
-  IF ( exx_flag ) CALL stop_exx ( )
-#endif
   DO is = 1, ns
     DO ir = 1, nr
       psic ( ir ) = CMPLX ( vxcr_g ( ir, is ), 0.0D0 )
@@ -1594,13 +1606,12 @@ END SUBROUTINE write_vxcg
 
 !-------------------------------------------------------------------------------
 
-SUBROUTINE write_vxc0 ( output_file_name, input_dft, exx_flag )
+SUBROUTINE write_vxc0 ( output_file_name, vxc_zero_rho_core )
 
   USE constants, ONLY : RYTOEV
   USE ener, ONLY : etxc, vtxc
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : fwfft
-  USE funct, ONLY : enforce_input_dft
   USE gvect, ONLY : ngm, nl, mill
   USE io_global, ONLY : ionode
   USE kinds, ONLY : DP
@@ -1609,15 +1620,11 @@ SUBROUTINE write_vxc0 ( output_file_name, input_dft, exx_flag )
   USE mp_global, ONLY : intra_pool_comm
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : psic
-#ifdef EXX
-  USE funct, ONLY : start_exx, stop_exx
-#endif
 
   IMPLICIT NONE
 
   character ( len = 256 ), intent (in) :: output_file_name
-  character ( len = 20 ), intent (in) :: input_dft
-  logical, intent (in) :: exx_flag
+  logical, intent (in) :: vxc_zero_rho_core
 
   integer :: unit
   integer :: is, ir, ig
@@ -1639,22 +1646,12 @@ SUBROUTINE write_vxc0 ( output_file_name, input_dft, exx_flag )
     vxc0_g ( is ) = ( 0.0D0, 0.0D0 )
   ENDDO
 
-! In GW calculations, the exchange-correlation
-! potential should not include partial core correction.
-! Assign rho_core and rhog_core to zero.
-
-  rho_core ( : ) = 0.0D0
-  rhog_core ( : ) = ( 0.0D0, 0.0D0 )
-
-  CALL enforce_input_dft ( input_dft )
-#ifdef EXX
-  IF ( exx_flag ) CALL start_exx ( )
-#endif
   vxcr_g ( :, : ) = 0.0D0
+  IF ( vxc_zero_rho_core ) THEN
+    rho_core ( : ) = 0.0D0
+    rhog_core ( : ) = ( 0.0D0, 0.0D0 )
+  ENDIF
   CALL v_xc ( rho, rho_core, rhog_core, etxc, vtxc, vxcr_g )
-#ifdef EXX
-  IF ( exx_flag ) CALL stop_exx ( )
-#endif
   DO is = 1, ns
     DO ir = 1, nr
       psic ( ir ) = CMPLX ( vxcr_g ( ir, is ), 0.0D0 )
@@ -1699,7 +1696,7 @@ END SUBROUTINE write_vxc0
 !-------------------------------------------------------------------------------
 
 SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
-  offdiag_nmin, offdiag_nmax, input_dft, exx_flag)
+  offdiag_nmin, offdiag_nmax, vxc_zero_rho_core)
 
   USE kinds, ONLY : DP
   USE constants, ONLY : rytoev
@@ -1707,7 +1704,6 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   USE ener, ONLY : etxc, vtxc
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : invfft
-  USE funct, ONLY : enforce_input_dft
   USE gvect, ONLY : ngm, g, nl
   USE io_files, ONLY : nwordwfc, iunwfc
   USE io_global, ONLY : ionode
@@ -1719,9 +1715,6 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : evc, psic
   USE wvfct, ONLY : npw, nbnd, igk, g2kin, ecutwfc
-#ifdef EXX
-  USE funct, ONLY : start_exx, stop_exx
-#endif
 
   IMPLICIT NONE
 
@@ -1730,8 +1723,7 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   integer, intent (inout) :: diag_nmax
   integer, intent (inout) :: offdiag_nmin
   integer, intent (inout) :: offdiag_nmax
-  character (len = 20), intent (in) :: input_dft
-  logical, intent (in) :: exx_flag
+  logical, intent (in) :: vxc_zero_rho_core
 
   integer :: ik, is, ib, ig, ir, unit, nkbl, nkl, nkr, iks, ike, &
     ndiag, noffdiag, ib2
@@ -1742,18 +1734,26 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   real (DP), allocatable :: vxcr (:, :)
   complex (DP), allocatable :: psic2 (:)
 
+  if(diag_nmin > diag_nmax) then
+    call errore ( 'write_vxc_r', 'diag_nmin > diag_nmax', diag_nmin )
+  endif
   IF (diag_nmin .LT. 1) diag_nmin = 1
   IF (diag_nmax .GT. nbnd) then
-    write(0,*) 'WARNING: resetting diag_nmax to max number of bands'
+    write(0,'(a,i6)') 'WARNING: resetting diag_nmax to max number of bands', nbnd
     diag_nmax = nbnd
   ENDIF
   ndiag = MAX (diag_nmax - diag_nmin + 1, 0)
+
+  if(offdiag_nmin > offdiag_nmax) then
+    call errore ( 'write_vxc_r', 'offdiag_nmin > offdiag_nmax', offdiag_nmin )
+  endif
   IF (offdiag_nmin .LT. 1) offdiag_nmin = 1
   IF (offdiag_nmax .GT. nbnd)  then
-    write(0,*) 'WARNING: resetting offdiag_nmax to max number of bands'
+    write(0,'(a,i6)') 'WARNING: resetting offdiag_nmax to max number of bands', nbnd
     offdiag_nmax = nbnd
   ENDIF
   noffdiag = MAX (offdiag_nmax - offdiag_nmin + 1, 0)
+
   IF (ndiag .EQ. 0 .AND. noffdiag .EQ. 0) RETURN
 
   unit = 4
@@ -1778,19 +1778,11 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   ALLOCATE (vxcr (dfftp%nnr, nspin))
   IF (noffdiag .GT. 0) ALLOCATE (psic2 (dfftp%nnr))
 
-  CALL enforce_input_dft (input_dft)
-#ifdef EXX
-  IF (exx_flag) CALL start_exx ()
-#endif
-
-! In GW calculations, the exchange-correlation
-! potential should not include partial core correction.
-! Assign rho_core and rhog_core to zero.
-
-  rho_core (:) = 0.0D0
-  rhog_core (:) = (0.0D0, 0.0D0)
-
   vxcr (:, :) = 0.0D0
+  IF ( vxc_zero_rho_core ) THEN
+    rho_core ( : ) = 0.0D0
+    rhog_core ( : ) = ( 0.0D0, 0.0D0 )
+  ENDIF
   CALL v_xc (rho, rho_core, rhog_core, etxc, vtxc, vxcr)
 
   DO ik = iks, ike
@@ -1842,10 +1834,6 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
     ENDIF
   ENDDO
 
-#ifdef EXX
-  IF (exx_flag) CALL stop_exx ()
-#endif
-
   DEALLOCATE (vxcr)
   IF (noffdiag .GT. 0) DEALLOCATE (psic2)
 
@@ -1898,14 +1886,15 @@ END SUBROUTINE write_vxc_r
 !-------------------------------------------------------------------------------
 
 SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
-  offdiag_nmin, offdiag_nmax, input_dft, exx_flag)
+  offdiag_nmin, offdiag_nmax, vxc_zero_rho_core)
 
   USE constants, ONLY : rytoev
   USE cell_base, ONLY : tpiba2, at, bg
   USE ener, ONLY : etxc, vtxc
+  USE exx, ONLY : vexx
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : fwfft, invfft
-  USE funct, ONLY : enforce_input_dft
+  USE funct, ONLY : exx_is_active
   USE gvect, ONLY : ngm, g, nl
   USE io_files, ONLY : nwordwfc, iunwfc
   USE io_global, ONLY : ionode
@@ -1918,10 +1907,6 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   USE scf, ONLY : rho, rho_core, rhog_core
   USE wavefunctions_module, ONLY : evc, psic
   USE wvfct, ONLY : npwx, npw, nbnd, igk, g2kin, ecutwfc
-#ifdef EXX
-  USE funct, ONLY : start_exx, stop_exx, exx_is_active
-  USE exx, ONLY : vexx
-#endif
 
   IMPLICIT NONE
 
@@ -1930,8 +1915,7 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   integer, intent (inout) :: diag_nmax
   integer, intent (inout) :: offdiag_nmin
   integer, intent (inout) :: offdiag_nmax
-  character (len = 20), intent (in) :: input_dft
-  logical, intent (in) :: exx_flag
+  logical, intent (in) :: vxc_zero_rho_core
 
   integer :: ik, is, ib, ig, ir, unit, nkbl, nkl, nkr, iks, ike, &
     ndiag, noffdiag, ib2
@@ -1942,12 +1926,26 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   complex (DP), allocatable :: psic2 (:)
   complex (DP), allocatable :: hpsi (:)
 
+  if(diag_nmin > diag_nmax) then
+    call errore ( 'write_vxc_g', 'diag_nmin > diag_nmax', diag_nmin )
+  endif
   IF (diag_nmin .LT. 1) diag_nmin = 1
-  IF (diag_nmax .GT. nbnd) diag_nmax = nbnd
+  IF (diag_nmax .GT. nbnd) then
+    write(0,'(a,i6)') 'WARNING: resetting diag_nmax to max number of bands', nbnd
+    diag_nmax = nbnd
+  ENDIF
   ndiag = MAX (diag_nmax - diag_nmin + 1, 0)
+
+  if(offdiag_nmin > offdiag_nmax) then
+    call errore ( 'write_vxc_g', 'offdiag_nmin > offdiag_nmax', offdiag_nmin )
+  endif
   IF (offdiag_nmin .LT. 1) offdiag_nmin = 1
-  IF (offdiag_nmax .GT. nbnd) offdiag_nmax = nbnd
+  IF (offdiag_nmax .GT. nbnd)  then
+    write(0,'(a,i6)') 'WARNING: resetting offdiag_nmax to max number of bands', nbnd
+    offdiag_nmax = nbnd
+  ENDIF
   noffdiag = MAX (offdiag_nmax - offdiag_nmin + 1, 0)
+
   IF (ndiag .EQ. 0 .AND. noffdiag .EQ. 0) RETURN
 
   unit = 4
@@ -1973,19 +1971,11 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   IF (noffdiag .GT. 0) ALLOCATE (psic2 (dfftp%nnr))
   ALLOCATE (hpsi (dfftp%nnr))
 
-  CALL enforce_input_dft (input_dft)
-#ifdef EXX
-  IF (exx_flag) CALL start_exx ()
-#endif
-
-! In GW calculations, the exchange-correlation
-! potential should not include partial core correction.
-! Assign rho_core and rhog_core to zero.
-
-  rho_core (:) = 0.0D0
-  rhog_core (:) = (0.0D0, 0.0D0)
-
   vxcr (:, :) = 0.0D0
+  IF ( vxc_zero_rho_core ) THEN
+    rho_core ( : ) = 0.0D0
+    rhog_core ( : ) = ( 0.0D0, 0.0D0 )
+  ENDIF
   CALL v_xc (rho, rho_core, rhog_core, etxc, vtxc, vxcr)
 
   DO ik = iks, ike
@@ -2011,10 +2001,8 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
         DO ig = 1, npw
           psic (ig) = evc (ig, ib)
         ENDDO
-#ifdef EXX
         IF (exx_is_active ()) CALL vexx (npwx, npw, 1, &
           psic, hpsi)
-#endif
         dummy = (0.0D0, 0.0D0)
         DO ig = 1, npw
           dummy = dummy + conjg (psic (ig)) * hpsi (ig)
@@ -2043,10 +2031,8 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
         DO ig = 1, npw
           psic (ig) = evc (ig, ib)
         ENDDO
-#ifdef EXX
         IF (exx_is_active ()) CALL vexx (npwx, npw, 1, &
           psic, hpsi)
-#endif
         DO ib2 = offdiag_nmin, offdiag_nmax
           psic2 (:) = (0.0D0, 0.0D0)
           DO ig = 1, npw
@@ -2065,10 +2051,6 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
     ENDIF
   ENDDO
 
-#ifdef EXX
-  IF (exx_flag) CALL stop_exx ()
-#endif
-
   DEALLOCATE (vxcr)
   IF (noffdiag .GT. 0) DEALLOCATE (psic2)
   DEALLOCATE (hpsi)
@@ -2120,7 +2102,216 @@ END SUBROUTINE write_vxc_g
 
 !-------------------------------------------------------------------------------
 
-SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
+SUBROUTINE write_vscg ( output_file_name, real_or_complex, symm_type )
+
+  USE cell_base, ONLY : omega, alat, tpiba, tpiba2, at, bg, ibrav
+  USE constants, ONLY : pi, tpi, eps6
+  USE ener, ONLY : etxc, vtxc
+  USE fft_base, ONLY : dfftp
+  USE fft_interfaces, ONLY : fwfft
+  USE gvect, ONLY : ngm, ngm_g, ig_l2g, nl, mill, ecutrho
+  USE io_global, ONLY : ionode
+  USE ions_base, ONLY : nat, atm, ityp, tau 
+  USE kinds, ONLY : DP
+  USE lsda_mod, ONLY : nspin
+  USE mp, ONLY : mp_sum
+  USE mp_global, ONLY : intra_pool_comm
+  USE scf, ONLY : vltot, v
+  USE symm_base, ONLY : s, ftau, nsym
+  USE wavefunctions_module, ONLY : psic
+
+  IMPLICIT NONE
+
+  character ( len = 256 ), intent (in) :: output_file_name
+  integer, intent (in) :: real_or_complex
+  character ( len = 9 ), intent (in) :: symm_type
+
+  character :: cdate*9, ctime*9, sdate*32, stime*32, stitle*32
+  integer :: unit, id, is, ir, ig, i, j, k, ierr
+  integer :: nd, ns, nr, ng_l, ng_g
+  integer :: ntran, cell_symmetry, nrecord
+  real (DP) :: alat2, recvol, dr1, t1 ( 3 ), t2 ( 3 )
+  real (DP) :: r1 ( 3, 3 ), r2 ( 3, 3 ), adot ( 3, 3 )
+  real (DP) :: bdot ( 3, 3 ), translation ( 3, 48 )
+  integer, allocatable :: g_g ( :, : )
+  real (DP), allocatable :: vscr_g ( :, : )
+  complex (DP), allocatable :: vscg_g ( :, : )
+
+  INTEGER, EXTERNAL :: atomic_number
+
+  CALL check_inversion ( real_or_complex, nsym, s, nspin, .true., .true. )
+
+  CALL date_and_tim ( cdate, ctime )
+  WRITE ( sdate, '(A2,"-",A3,"-",A4,21X)' ) cdate(1:2), cdate(3:5), cdate(6:9)
+  WRITE ( stime, '(A8,24X)' ) ctime(1:8)
+  ! this is supposed to be VSC-Real/Complex but BGW wfn_rho_vxc IO
+  ! does not recognize VSC header so we are using VXC instead
+  IF ( real_or_complex .EQ. 1 ) THEN
+    WRITE ( stitle, '("VXC-Real",24X)' )
+  ELSE
+    WRITE ( stitle, '("VXC-Complex",21X)' )
+  ENDIF
+
+  unit = 4
+  nrecord = 1
+  nd = 3
+
+  ns = nspin
+  nr = dfftp%nnr
+  ng_l = ngm
+  ng_g = ngm_g
+
+  ierr = 0
+  IF ( ibrav .EQ. 0 ) THEN
+    IF ( TRIM ( symm_type ) .EQ. 'cubic' ) THEN
+      cell_symmetry = 0
+    ELSEIF ( TRIM ( symm_type ) .EQ. 'hexagonal' ) THEN
+      cell_symmetry = 1
+    ELSE
+      ierr = 1
+    ENDIF
+  ELSEIF ( abs ( ibrav ) .GE. 1 .AND. abs ( ibrav ) .LE. 3 ) THEN
+    cell_symmetry = 0
+  ELSEIF ( abs ( ibrav ) .GE. 4 .AND. abs ( ibrav ) .LE. 5 ) THEN
+    cell_symmetry = 1
+  ELSEIF ( abs ( ibrav ) .GE. 6 .AND. abs ( ibrav ) .LE. 14 ) THEN
+    cell_symmetry = 0
+  ELSE
+    ierr = 1
+  ENDIF
+  IF ( ierr .GT. 0 ) &
+    CALL errore ( 'write_vscg', 'cell_symmetry', ierr )
+
+  ntran = nsym
+  DO i = 1, ntran
+    DO j = 1, nd
+      DO k = 1, nd
+        r1 ( k, j ) = dble ( s ( k, j, i ) )
+      ENDDO
+    ENDDO
+    CALL invmat ( 3, r1, r2, dr1 )
+    t1 ( 1 ) = dble ( ftau ( 1, i ) ) / dble ( dfftp%nr1 )
+    t1 ( 2 ) = dble ( ftau ( 2, i ) ) / dble ( dfftp%nr2 )
+    t1 ( 3 ) = dble ( ftau ( 3, i ) ) / dble ( dfftp%nr3 )
+    DO j = 1, nd
+      t2 ( j ) = 0.0D0
+      DO k = 1, nd
+        t2 ( j ) = t2 ( j ) + r2 ( k, j ) * t1 ( k )
+      ENDDO
+      IF ( t2 ( j ) .GE. eps6 + 0.5D0 ) &
+        t2 ( j ) = t2 ( j ) - dble ( int ( t2 ( j ) + 0.5D0 ) )
+      IF ( t2 ( j ) .LT. eps6 - 0.5D0 ) &
+        t2 ( j ) = t2 ( j ) - dble ( int ( t2 ( j ) - 0.5D0 ) )
+    ENDDO
+    DO j = 1, nd
+      translation ( j, i ) = t2 ( j ) * tpi
+    ENDDO
+  ENDDO
+
+  alat2 = alat ** 2
+  recvol = 8.0D0 * pi**3 / omega
+
+  DO i = 1, nd
+    DO j = 1, nd
+      adot ( j, i ) = 0.0D0
+    ENDDO
+  ENDDO
+  DO i = 1, nd
+    DO j = 1, nd
+      DO k = 1, nd
+        adot ( j, i ) = adot ( j, i ) + &
+          at ( k, j ) * at ( k, i ) * alat2
+      ENDDO
+    ENDDO
+  ENDDO
+
+  DO i = 1, nd
+    DO j = 1, nd
+      bdot ( j, i ) = 0.0D0
+    ENDDO
+  ENDDO
+  DO i = 1, nd
+    DO j = 1, nd
+      DO k = 1, nd
+        bdot ( j, i ) = bdot ( j, i ) + &
+          bg ( k, j ) * bg ( k, i ) * tpiba2
+      ENDDO
+    ENDDO
+  ENDDO
+
+  ALLOCATE ( g_g ( nd, ng_g ) )
+  ALLOCATE ( vscr_g ( ng_g, ns ) )
+  ALLOCATE ( vscg_g ( ng_g, ns ) )
+
+  DO ig = 1, ng_g
+    DO id = 1, nd
+      g_g ( id, ig ) = 0
+    ENDDO
+  ENDDO
+  DO is = 1, ns
+    DO ig = 1, ng_g
+      vscg_g ( ig, is ) = ( 0.0D0, 0.0D0 )
+    ENDDO
+  ENDDO
+
+  DO ig = 1, ng_l
+    g_g ( 1, ig_l2g ( ig ) ) = mill ( 1, ig )
+    g_g ( 2, ig_l2g ( ig ) ) = mill ( 2, ig )
+    g_g ( 3, ig_l2g ( ig ) ) = mill ( 3, ig )
+  ENDDO
+  vscr_g ( :, : ) = 0.0D0
+  DO is = 1, ns
+    DO ir = 1, nr
+      psic ( ir ) = CMPLX ( v%of_r ( ir, is ) + vltot ( ir ), 0.0D0 )
+    ENDDO
+    CALL fwfft ( 'Dense', psic, dfftp )
+    DO ig = 1, ng_l
+      vscg_g ( ig_l2g ( ig ), is ) = psic ( nl ( ig ) )
+    ENDDO
+  ENDDO
+
+  CALL mp_sum ( g_g, intra_pool_comm )
+  CALL mp_sum ( vscg_g, intra_pool_comm )
+
+  IF ( ionode ) THEN
+    OPEN ( unit = unit, file = TRIM ( output_file_name ), &
+      form = 'unformatted', status = 'replace' )
+    WRITE ( unit ) stitle, sdate, stime
+    WRITE ( unit ) ns, ng_g, ntran, cell_symmetry, nat, ecutrho
+    WRITE ( unit ) dfftp%nr1, dfftp%nr2, dfftp%nr3
+    WRITE ( unit ) omega, alat, ( ( at ( j, i ), j = 1, nd ), i = 1, nd ), &
+      ( ( adot ( j, i ), j = 1, nd ), i = 1, nd )
+    WRITE ( unit ) recvol, tpiba, ( ( bg ( j, i ), j = 1, nd ), i = 1, nd ), &
+      ( ( bdot ( j, i ), j = 1, nd ), i = 1, nd )
+    WRITE ( unit ) ( ( ( s ( k, j, i ), k = 1, nd ), j = 1, nd ), i = 1, ntran )
+    WRITE ( unit ) ( ( translation ( j, i ), j = 1, nd ), i = 1, ntran )
+    WRITE ( unit ) ( ( tau ( j, i ), j = 1, nd ), atomic_number ( atm ( ityp ( i ) ) ), i = 1, nat )
+    WRITE ( unit ) nrecord
+    WRITE ( unit ) ng_g
+    WRITE ( unit ) ( ( g_g ( id, ig ), id = 1, nd ), ig = 1, ng_g )
+    WRITE ( unit ) nrecord
+    WRITE ( unit ) ng_g
+    IF ( real_or_complex .EQ. 1 ) THEN
+      WRITE ( unit ) ( ( dble ( vscg_g ( ig, is ) ), &
+        ig = 1, ng_g ), is = 1, ns )
+    ELSE
+      WRITE ( unit ) ( ( vscg_g ( ig, is ), &
+        ig = 1, ng_g ), is = 1, ns )
+    ENDIF
+    CLOSE ( unit = unit, status = 'keep' )
+  ENDIF
+
+  DEALLOCATE ( vscg_g )
+  DEALLOCATE ( vscr_g )
+  DEALLOCATE ( g_g )
+
+  RETURN
+
+END SUBROUTINE write_vscg
+
+!-------------------------------------------------------------------------------
+
+SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   wfng_nk1, wfng_nk2, wfng_nk3, wfng_dk1, wfng_dk2, wfng_dk3)
 
   USE cell_base, ONLY : omega, alat, tpiba, tpiba2, at, bg, ibrav
@@ -2179,6 +2370,8 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
   CALL date_and_tim ( cdate, ctime )
   WRITE ( sdate, '(A2,"-",A3,"-",A4,21X)' ) cdate(1:2), cdate(3:5), cdate(6:9)
   WRITE ( stime, '(A8,24X)' ) ctime(1:8)
+  ! BGW wfn_rho_vxc IO does not recognize VKB header so this file
+  ! is read directly by SAPO code in BerkeleyGW
   WRITE ( stitle, '("VKB-Complex",21X)' )
 
   unit = 4
@@ -2212,7 +2405,7 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
     ierr = 1
   ENDIF
   IF ( ierr .GT. 0 ) &
-    CALL errore ( 'write_vnlg', 'cell_symmetry', ierr )
+    CALL errore ( 'write_vkbg', 'cell_symmetry', ierr )
 
   ntran = nsym
   DO i = 1, ntran
@@ -2289,7 +2482,7 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
   ENDDO
   CALL mp_max ( ierr )
   IF ( ierr .GT. 0 ) &
-    CALL errore ( 'write_vnlg', 'smap', ierr )
+    CALL errore ( 'write_vkbg', 'smap', ierr )
 
   ALLOCATE ( gvec ( 3, ngm_g ) )
   gvec = 0
@@ -2324,6 +2517,8 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
   CALL mp_max ( npw_g )
   npwx_g = MAXVAL ( ngk_g ( : ) )
 
+  CALL cryst_to_cart (nkstot, xk, at, -1)
+
   IF (ionode) THEN
     OPEN (unit = unit, file = TRIM (output_file_name), &
       form = 'unformatted', status = 'replace')
@@ -2356,6 +2551,8 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
     WRITE ( unit ) ( ( gvec ( id, ig ), id = 1, nd ), ig = 1, ngm_g )
   ENDIF
 
+  CALL cryst_to_cart (nkstot, xk, bg, 1)
+
   ALLOCATE ( igwk ( npwx_g ) )
 
   DO i = 1, nkstot
@@ -2440,7 +2637,7 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
       ierr = 1
     CALL mp_max ( ierr )
     IF ( ierr .GT. 0 ) &
-      CALL errore ( 'write_vnlg', 'igwx ngk_g', ierr )
+      CALL errore ( 'write_vkbg', 'igwx ngk_g', ierr )
 
     ALLOCATE ( vkb_g ( MAX ( 1, igwx ) ) )
 
@@ -2487,7 +2684,7 @@ SUBROUTINE write_vnlg (output_file_name, symm_type, wfng_kgrid, &
 
   RETURN
 
-END SUBROUTINE write_vnlg
+END SUBROUTINE write_vkbg
 
 !-------------------------------------------------------------------------------
 
@@ -2527,7 +2724,7 @@ subroutine check_inversion(real_or_complex, ntran, mtrx, nspin, warn, real_need_
   enddo
 
   if(real_or_complex .eq. 2) then
-    if(invflag .ne. 0 .and. warn) then
+    if(invflag .ne. 0 .and. warn .and. nspin == 1) then
       if(ionode) write(6, '(a)') 'WARNING: Inversion symmetry is present. The real version would be faster.'
     endif
   else
@@ -2542,7 +2739,7 @@ subroutine check_inversion(real_or_complex, ntran, mtrx, nspin, warn, real_need_
     endif
     if(nspin .eq. 2) then
       call errore('check_inversion', &
-        'Time-reversal symmetry is absent in spin-polarized calculation. Complex version must be used.', -2)
+        'Real version may only be used for spin-unpolarized calculations.', nspin)
     endif
   endif
 
