@@ -21,7 +21,7 @@ MODULE dynamics_module
    USE io_global, ONLY : stdout
    USE io_files,  ONLY : prefix, tmp_dir, seqopn
    USE constants, ONLY : tpi, fpi
-   USE constants, ONLY : amconv, ry_to_kelvin, au_ps, bohr_radius_cm, ry_kbar
+   USE constants, ONLY : amu_ry, ry_to_kelvin, au_ps, bohr_radius_cm, ry_kbar
    USE constants, ONLY : eps8
    USE control_flags, ONLY : tolp
    !
@@ -119,7 +119,7 @@ CONTAINS
       USE ions_base,      ONLY : nat, nsp, ityp, tau, if_pos, atm
       USE cell_base,      ONLY : alat, omega
       USE ener,           ONLY : etot
-      USE force_mod,      ONLY : force
+      USE force_mod,      ONLY : force, lstres
       USE control_flags,  ONLY : istep, nstep, conv_ions, lconstrain, &
                                  lfixatom
       !
@@ -140,6 +140,8 @@ CONTAINS
 #endif
       LOGICAL  :: file_exists, leof
       REAL(DP), EXTERNAL :: dnrm2
+      REAL(DP) :: kstress(3,3)
+      INTEGER :: i, j
       !
       ! ... the number of degrees of freedom
       !
@@ -289,7 +291,7 @@ CONTAINS
          ! ... check if the new positions satisfy the constrain equation
          !
          CALL check_constraint( nat, tau_new, tau, &
-                                force, if_pos, ityp, alat, dt**2, amconv )
+                                force, if_pos, ityp, alat, dt**2, amu_ry )
          !
 #if ! defined (__REDUCE_OUTPUT)
          !
@@ -309,7 +311,7 @@ CONTAINS
 
          IF (vel_defined) THEN
             CALL check_constraint( nat, tau_old, tau, &
-                                   force, if_pos, ityp, alat, dt**2, amconv )
+                                   force, if_pos, ityp, alat, dt**2, amu_ry )
          ENDIF
          !
       ENDIF
@@ -320,16 +322,23 @@ CONTAINS
       !
       ml   = 0.D0
       ekin = 0.D0
+      kstress = 0.d0
       !
       DO na = 1, nat
          !
          ml(:) = ml(:) + vel(:,na) * mass(na)
          ekin  = ekin + 0.5D0 * mass(na) * &
                         ( vel(1,na)**2 + vel(2,na)**2 + vel(3,na)**2 )
+         do i = 1, 3
+             do j = 1, 3
+                 kstress(i,j) = kstress(i,j) + mass(na)*vel(i,na)*vel(j,na)
+             enddo
+         enddo
          !
       ENDDO
       !
       ekin = ekin*alat**2
+      kstress = kstress * alat**2 / omega
       !
       ! ... find the new temperature and update the average
       !
@@ -384,6 +393,10 @@ CONTAINS
                      & 5X,"temperature           = ",F14.8," K ",/,  &
                      & 5X,"Ekin + Etot (const)   = ",F14.8," Ry")' ) &
           ekin, temp_new, ( ekin  + etot )
+      IF (lstres) WRITE ( stdout, &
+      '(5X,"Ions kinetic stress = ",F10.2," (kbar)",/3(27X,3F10.2/)/)') &
+              ((kstress(1,1)+kstress(2,2)+kstress(3,3))/3.d0*ry_kbar), &
+              (kstress(i,1)*ry_kbar,kstress(i,2)*ry_kbar,kstress(i,3)*ry_kbar, i=1,3)
       !
       IF ( .not.( lconstrain .or. lfixatom ) ) THEN
          !
@@ -479,7 +492,7 @@ CONTAINS
          !
          DO na = 1, nat
             !
-            mass(na) = amass( ityp(na) ) * amconv
+            mass(na) = amass( ityp(na) ) * amu_ry
             !
             total_mass = total_mass + mass(na)
             !
@@ -881,7 +894,7 @@ CONTAINS
       !
       CALL force_precond( istep, force, etotold )
       !
-      acc(:,:) = force(:,:) / alat / amconv
+      acc(:,:) = force(:,:) / alat / amu_ry
       !
       CALL project_velocity()
       !
@@ -917,7 +930,7 @@ CONTAINS
          ! ... check if the new positions satisfy the constrain equation
          !
          CALL check_constraint( nat, tau_new, tau, &
-                                force, if_pos, ityp, alat, dt**2, amconv )
+                                force, if_pos, ityp, alat, dt**2, amu_ry )
          !
       ENDIF
       !
@@ -1073,7 +1086,7 @@ CONTAINS
          ! ... check if the new positions satisfy the constrain equation
          !
          CALL check_constraint( nat, tau_new, tau, &
-                                force, if_pos, ityp, alat, dt**2, amconv )
+                                force, if_pos, ityp, alat, dt**2, amu_ry )
          !
 #if ! defined (__REDUCE_OUTPUT)
          !
