@@ -35,7 +35,7 @@ SUBROUTINE electrons()
   USE ener,                 ONLY : etot, hwf_energy, eband, deband, ehart, &
                                    vtxc, etxc, etxcc, ewld, demet, epaw, &
                                    elondon
-  USE scf,                  ONLY : scf_type, scf_type_COPY, &
+  USE scf,                  ONLY : scf_type, scf_type_COPY, bcast_scf_type,&
                                    create_scf_type, destroy_scf_type, &
                                    rho, rho_core, rhog_core, &
                                    v, vltot, vrs, kedtau, vnew
@@ -43,7 +43,7 @@ SUBROUTINE electrons()
                                    iprint, istep, lscf, lmd, conv_elec, &
                                    restart, io_level, do_makov_payne,  &
                                    gamma_only, iverbosity, textfor,     &
-                                   llondon
+                                   llondon, scf_must_converge
   USE io_files,             ONLY : iunwfc, iunocc, nwordwfc, output_drho, &
                                    iunefield, iunpaw
   USE buffers,              ONLY : save_buffer
@@ -61,9 +61,9 @@ SUBROUTINE electrons()
   USE funct,                ONLY : dft_is_hybrid, exx_is_active
   USE control_flags,        ONLY : adapt_thr, tr2_init, tr2_multi
   USE funct,                ONLY : dft_is_meta
-  USE mp_global,            ONLY : intra_bgrp_comm, nbgrp, mpime, &
-                                   inter_bgrp_comm, my_bgrp_id
-  USE mp,                   ONLY : mp_sum
+  USE mp_global,            ONLY : intra_bgrp_comm, inter_pool_comm, &
+                                   root_pool, my_pool_id
+  USE mp,                   ONLY : mp_sum, mp_bcast
   !
   USE london_module,        ONLY : energy_london
   !
@@ -76,8 +76,9 @@ SUBROUTINE electrons()
                                    vltot_zero, environ_thr,                 &
                                    env_static_permittivity,                 & 
                                    env_surface_tension, env_pressure,       &
-                                   env_slab_geometry, deenviron,            &
-                                   esolvent, ecavity, epressure, eslab
+                                   env_periodicity, env_ioncc_concentration,&
+                                   deenviron, esolvent, ecavity, epressure, &
+                                   eperiodic, eioncc
 #endif
   USE dfunct,                 only : newd
   USE esm,                  ONLY : do_comp_esm, esm_printpot
@@ -155,7 +156,6 @@ SUBROUTINE electrons()
   WRITE( stdout, 9000 ) get_clock( 'PWSCF' )
   !
   CALL memstat( kilobytes )
-  !
   IF ( kilobytes > 0 ) WRITE( stdout, 9001 ) kilobytes/1000.0
   !
   CALL flush_unit( stdout )
@@ -181,9 +181,9 @@ SUBROUTINE electrons()
 #ifdef __ENVIRON
   IF ( do_environ ) THEN
     vltot_zero = vltot
-    CALL environ_initions( dfftp%nnr, nat, nsp, ityp, zv, tau ) 
+    CALL environ_initions( dfftp%nnr, nat, nsp, ityp, zv, tau, alat ) 
     CALL environ_initcell( dfftp%nnr, dfftp%nr1*dfftp%nr2*dfftp%nr3, &
-                           omega, alat ) 
+                           omega, alat, at ) 
   END IF
 #endif
   !  
@@ -194,14 +194,10 @@ SUBROUTINE electrons()
   ewld = ewald( alat, nat, nsp, ityp, zv, at, bg, tau, &
                 omega, g, gg, ngm, gcutm, gstart, gamma_only, strf )
   !
-  !
-  !
-  elondon = 0.d0
-  !
   IF ( llondon ) THEN
-  !
-  elondon = energy_london ( alat , nat , ityp , at ,&
-                                         bg , tau )
+     elondon = energy_london ( alat , nat , ityp , at ,bg , tau )
+  ELSE
+     elondon = 0.d0
   END IF
   !
   call create_scf_type ( rhoin )
@@ -341,10 +337,22 @@ SUBROUTINE electrons()
         !
         deband = delta_e()
         !
-        ! ... mix_rho mixes several quantities: rho in g-space, tauk (for meta-gga)
-        ! ... ns (for lda+u) and becsum (for paw)
+        ! ... mix_rho mixes several quantities: rho in g-space, tauk (for
+        ! ... meta-gga), ns and ns_nc (for lda+u) and becsum (for paw)
+        ! ... Results are broadcast from pool 0 to others to prevent trouble
+        ! ... on machines unable to yield the same results from the same 
+        ! ... calculation on same data, performed on different procs
+        ! ... The mixing should be done on pool 0 only as well, but inside
+        ! ... mix_rho there is a call to rho_ddot that in the PAW case 
+        ! ... contains a hidden parallelization level on the entire image
+        ! IF ( my_pool_id == root_pool ) 
+        CALL mix_rho &
+             ( rho, rhoin, mixing_beta, dr2, tr2_min, iter, nmix, conv_elec )
+        CALL bcast_scf_type ( rhoin, root_pool, inter_pool_comm )
+        CALL mp_bcast ( dr2, root_pool, inter_pool_comm )
+        CALL mp_bcast ( conv_elec, root_pool, inter_pool_comm )
         !
-        CALL mix_rho( rho, rhoin, mixing_beta, dr2, tr2_min, iter, nmix, conv_elec )
+        if (.not. scf_must_converge .and. idum == niter) conv_elec = .true.
         !
         ! ... if convergence is achieved or if the self-consistency error
         ! ... (dr2) is smaller than the estimated error due to diagonalization
@@ -456,8 +464,8 @@ SUBROUTINE electrons()
        !
        vltot = vltot_zero
        !
-       CALL calc_eenviron( dfftp%nnr, nspin, rhoin%of_r, vltot_zero, &
-                           deenviron, esolvent, ecavity, epressure, eslab )
+       CALL calc_eenviron( dfftp%nnr, nspin, rhoin%of_r, deenviron, esolvent, &
+                           ecavity, epressure, eperiodic, eioncc )
        !
        update_venviron = .NOT. conv_elec .AND. dr2 .LT. environ_thr
        !
@@ -574,6 +582,13 @@ SUBROUTINE electrons()
         !
         dexx = fock1 - 0.5D0*( fock0 + fock2 )
         !
+        ! dexx is by definition positive definite. If it is less than
+        ! 0 there is some numerical problem. One such cause could be
+        ! that the exx divergence treatment has failed. 
+        !
+        IF ( dexx < 0d0 ) CALL errore( 'electrons', 'dexx is negative! &
+             & Check that exxdiv_treatment is appropriate for the system.', 1 )
+        !
         etot = etot  - dexx
         hwf_energy = hwf_energy - dexx
         !
@@ -594,7 +609,8 @@ SUBROUTINE electrons()
      !
      ! ... adds the external environment contribution to the energy
      !
-     IF ( do_environ ) etot = etot + deenviron + esolvent + ecavity + epressure + eslab
+     IF ( do_environ ) etot = etot + deenviron + esolvent + ecavity + & 
+                              epressure + eperiodic + eioncc
 #endif
      !
      IF ( ( conv_elec .OR. MOD( iter, iprint ) == 0 ) .AND. .NOT. lmd ) THEN
@@ -650,7 +666,11 @@ SUBROUTINE electrons()
         IF ( env_static_permittivity .GT. 1.D0 ) WRITE( stdout, 9201 ) esolvent
         IF ( env_surface_tension .GT. 0.D0 ) WRITE( stdout, 9202 ) ecavity
         IF ( env_pressure .NE. 0.D0 ) WRITE( stdout, 9203 ) epressure
-        IF ( env_slab_geometry ) WRITE( stdout, 9204 ) eslab
+        IF ( env_ioncc_concentration .GT. 0.D0 ) THEN 
+          WRITE( stdout, 9205 ) eioncc
+        ELSE IF ( env_periodicity .NE. 3 ) THEN
+          WRITE( stdout, 9204 ) eperiodic
+        ENDIF
      ENDIF
      !
 #endif
@@ -695,7 +715,11 @@ SUBROUTINE electrons()
         !
         IF ( do_comp_esm ) CALL esm_printpot()
         !
-        WRITE( stdout, 9110 ) iter
+        if (idum < niter) then
+          WRITE( stdout, 9110 ) iter
+        else
+          WRITE( stdout, 9122 ) iter
+        endif
         !
         ! ... jump to the end
         !
@@ -771,13 +795,15 @@ SUBROUTINE electrons()
 9101 FORMAT(/'     End of self-consistent calculation' )
 9110 FORMAT(/'     convergence has been achieved in ',i3,' iterations' )
 9120 FORMAT(/'     convergence NOT achieved after ',i3,' iterations: stopping' )
+9122 FORMAT(/'     WARNING: convergence NOT achieved after ',i3,' iterations' )
 9121 FORMAT(/'     scf convergence threshold =',1PE17.1,' Ry' )
 #ifdef __ENVIRON
 9200 FORMAT(/'     add environment contribution to local potential')
 9201 FORMAT( '     solvation energy          =',F17.8,' Ry' ) 
 9202 FORMAT( '     cavitation energy         =',F17.8,' Ry' ) 
 9203 FORMAT( '     PV energy                 =',F17.8,' Ry' ) 
-9204 FORMAT( '     slab energy correction    =',F17.8,' Ry' )
+9204 FORMAT( '     periodic energy correct.  =',F17.8,' Ry' )
+9205 FORMAT( '     ionic charge energy       =',F17.8,' Ry' )
 #endif
   !
   CONTAINS
