@@ -35,7 +35,7 @@ SUBROUTINE forces()
   USE force_mod,     ONLY : force, lforce
   USE scf,           ONLY : rho
   USE ions_base,     ONLY : if_pos
-  USE ldaU,          ONLY : lda_plus_u
+  USE ldaU,          ONLY : lda_plus_u, U_projection
   USE extfield,      ONLY : tefield, forcefield
   USE control_flags, ONLY : gamma_only, remove_rigid_rot, textfor, &
                             iverbosity, llondon
@@ -98,8 +98,9 @@ SUBROUTINE forces()
   CALL force_cc( forcecc )
   !
   ! ... The Hubbard contribution
+  !     (included by force_us if using beta as local projectors)
   !
-  IF ( lda_plus_u ) CALL force_hub( forceh )
+  IF ( lda_plus_u .AND. U_projection.NE.'pseudo' ) CALL force_hub( forceh )
   !
   ! ... The ionic contribution is computed here
   !
@@ -125,7 +126,7 @@ SUBROUTINE forces()
     !
     ALLOCATE ( force_mt ( 3 , nat ) )
 #ifdef __ENVIRON
-    IF ( do_environ ) THEN
+    IF ( do_environ .AND. env_static_permittivity .GT. 1.D0 ) THEN
       ALLOCATE( aux( dfftp%nnr ) )
       aux(:) = CMPLX(rhopol( : ),0.D0,kind=dp) 
       CALL fwfft ('Dense', aux, dfftp)
@@ -135,7 +136,7 @@ SUBROUTINE forces()
     CALL wg_corr_force( omega, nat, ntyp, ityp, ngm, g, tau, zv, strf, &
                         nspin, rho%of_g, force_mt )
 #ifdef __ENVIRON
-    IF ( do_environ ) THEN
+    IF ( do_environ .AND. env_static_permittivity .GT. 1.D0  ) THEN
       rho%of_g(:,1) = rho%of_g(:,1) - aux(nl(:))
       DEALLOCATE(aux)
     ENDIF
