@@ -21,24 +21,24 @@ SUBROUTINE iosys()
   !
   !
   USE kinds,         ONLY : DP
+  USE uspp,          ONLY : okvan
   USE funct,         ONLY : dft_has_finite_size_correction, &
                             set_finite_size_volume, get_inlc 
   USE funct,         ONLY: set_exx_fraction, set_screening_parameter
   USE control_flags, ONLY: adapt_thr, tr2_init, tr2_multi
   USE constants,     ONLY : autoev, eV_to_kelvin, pi, rytoev, &
-                            ry_kbar, amconv, bohr_radius_angs, eps8
+                            ry_kbar, amu_ry, bohr_radius_angs, eps8
   USE mp_global,     ONLY : npool, nproc_pool
   !
   USE io_global,     ONLY : stdout, ionode, ionode_id
   !
   USE kernel_table,  ONLY : initialize_kernel_table
   !
-  USE mp,            ONLY : mp_bcast
-  !
   USE bp,            ONLY : nppstr_    => nppstr, &
                             gdir_      => gdir, &
                             lberry_    => lberry, &
                             lelfield_  => lelfield, &
+                            lorbm_     => lorbm, &
                             efield_    => efield, &
                             nberrycyc_ => nberrycyc, &
                             efield_cart_ => efield_cart
@@ -97,8 +97,10 @@ SUBROUTINE iosys()
   USE start_k,       ONLY : init_start_k
   !
   USE ldaU,          ONLY : Hubbard_U_     => hubbard_u, &
+                            Hubbard_J0_ => hubbard_j0, &
                             Hubbard_J_ => hubbard_j, &
                             Hubbard_alpha_ => hubbard_alpha, &
+                            Hubbard_beta_ => hubbard_beta, &
                             lda_plus_u_    => lda_plus_u, &
                             lda_plus_u_kind_    => lda_plus_u_kind, &
                             niter_with_fixed_ns, starting_ns, U_projection
@@ -127,8 +129,8 @@ SUBROUTINE iosys()
                            env_surface_tension_ => env_surface_tension,         &
                            delta_ => delta,                                     &
                            env_pressure_ => env_pressure,                       &
-                           env_slab_geometry, slab_axis,                        &
-                           cion_ => cion,                                       &
+                           env_periodicity, slab_axis,                          &
+                           env_ioncc_concentration_ => env_ioncc_concentration, &
                            zion_ => zion,                                       &
                            rhopb_ => rhopb,                                     &
                            solvent_temperature_ => solvent_temperature
@@ -176,7 +178,9 @@ SUBROUTINE iosys()
                             ldamped, lbands, llang,           &
                             lconstrain, restart, twfcollect, &
                             llondon, do_makov_payne, &
-                            lecrpa_           => lecrpa
+                            lecrpa_           => lecrpa, &
+                            smallmem
+  USE control_flags, ONLY: scf_must_converge_ => scf_must_converge
   !
   USE wvfct,         ONLY : nbnd_ => nbnd, &
                             ecutwfc_ => ecutwfc, &
@@ -227,9 +231,9 @@ SUBROUTINE iosys()
                                nstep, iprint, tstress, tprnfor, dt, outdir,    &
                                wfcdir, prefix, etot_conv_thr, forc_conv_thr,   &
                                pseudo_dir, disk_io, tefield, dipfield, lberry, &
-                               gdir, nppstr, wf_collect,lelfield, efield,      &
+                               gdir, nppstr, wf_collect,lelfield,lorbm,efield, &
                                nberrycyc, lkpoint_dir, efield_cart, lecrpa,    &
-                               vdw_table_name
+                               vdw_table_name, memory
 
 #if defined __MS2
   USE input_parameters, ONLY : MS2_enabled, MS2_handler
@@ -246,6 +250,7 @@ SUBROUTINE iosys()
                                occupations, degauss, smearing, nspin,       &
                                ecfixed, qcutz, q2sigma, lda_plus_U,         &
                                lda_plus_U_kind, Hubbard_U, Hubbard_J,       &
+                               Hubbard_J0, Hubbard_beta,                    &
                                Hubbard_alpha, input_dft, la2F,              &
                                starting_ns_eigenvalue, U_projection_type,   &
                                x_gamma_extrapolation, nqx1, nqx2, nqx3,     &
@@ -273,7 +278,8 @@ SUBROUTINE iosys()
                                mixtype, ndiis, mixrhopol, tolrhopol,    &
                                env_surface_tension, delta,              &
                                env_pressure,                            &
-                               cion, zion, rhopb, solvent_temperature 
+                               env_ioncc_concentration, zion, rhopb,    &
+                               solvent_temperature
 #endif
   !
   ! ... ELECTRONS namelist
@@ -283,7 +289,7 @@ SUBROUTINE iosys()
                                tqr, diago_thr_init, diago_cg_maxiter,      &
                                diago_david_ndim, diagonalization,          &
                                diago_full_acc, startingwfc, startingpot,   &
-                               real_space
+                               real_space, scf_must_converge
   USE input_parameters, ONLY : adaptive_thr, conv_thr_init, conv_thr_multi
   !
   ! ... IONS namelist
@@ -504,6 +510,12 @@ SUBROUTINE iosys()
      CALL errore( 'iosys', 'LSDA not available with electric field' , 1 )
   ENDIF
   !
+  ! ... define memory related internal switches
+  !
+  IF( TRIM( memory ) == 'small' ) THEN
+     smallmem = .TRUE.
+  END IF
+  !
   twfcollect = wf_collect
   !
   ! ... Set Values for electron and bands
@@ -585,8 +597,7 @@ SUBROUTINE iosys()
   CASE( 2 )
      !
      lsda = .true.
-     IF ( noncolin ) &
-        CALL errore( 'iosys', &
+     IF ( noncolin ) CALL errore( 'iosys', &
                      'noncolin .and. nspin==2 are conflicting flags', 1 )
      !
   CASE( 4 )
@@ -600,8 +611,10 @@ SUBROUTINE iosys()
      !
   END SELECT
   !
-  IF (lda_plus_u.and.lda_plus_u_kind.eq.0.and.noncolin) CALL errore('iosys', &
-       'simplified LDA+U not implemented with noncol. magnetism, use lda_plus_u_kind = 1', 1)
+  IF ( lda_plus_u .AND. lda_plus_u_kind == 0 .AND. noncolin ) THEN
+     CALL errore('iosys', 'simplified LDA+U not implemented with &
+                          &noncol. magnetism, use lda_plus_u_kind = 1', 1)
+  END IF
   !
   two_fermi_energies = ( tot_magnetization /= -1._DP)
   IF ( two_fermi_energies .and. tot_magnetization < 0._DP) &
@@ -844,8 +857,10 @@ SUBROUTINE iosys()
   END SELECT
   !
   Hubbard_U(:)    = Hubbard_U(:) / rytoev
+  Hubbard_J0(:)    = Hubbard_J0(:) / rytoev
   Hubbard_J(:,:)  = Hubbard_J(:,:) / rytoev
   Hubbard_alpha(:)= Hubbard_alpha(:) / rytoev
+  Hubbard_beta(:)= Hubbard_beta(:) / rytoev
   !
   ethr = diago_thr_init
   !
@@ -879,6 +894,13 @@ SUBROUTINE iosys()
      startingwfc = 'atomic'
      !
   ENDIF
+  ! 
+  IF (one_atom_occupations .and. startingwfc /= 'atomic' ) THEN
+
+     CALL infomsg( 'iosys', 'one_atom_occupations requires startingwfc atomic' )
+
+     startingwfc = 'atomic'
+  ENDIF
   !
   SELECT CASE( trim( diagonalization ) )
   CASE ( 'cg' )
@@ -1074,12 +1096,11 @@ SUBROUTINE iosys()
   IF ( lberry .OR. lelfield ) THEN
      IF ( npool > 1 ) CALL errore( 'iosys', &
           'Berry Phase/electric fields not implemented with pools', 1 )
-     IF ( noncolin )  CALL errore( 'iosys', &
-          'Noncolinear Berry Phase/electric fields not implemented', 1 )
+     IF ( noncolin .AND. okvan )  CALL errore( 'iosys', &
+         'Noncolinear Berry Phase/electric fields not implemented with USPP', 1 )
      IF ( lgauss .OR. ltetra ) CALL errore( 'iosys', &
           'Berry Phase/electric fields only for insulators!', 1 )
   END IF
-
   !
   ! ... Copy values from input module to PW internals
   !
@@ -1087,6 +1108,7 @@ SUBROUTINE iosys()
   gdir_       = gdir
   lberry_     = lberry
   lelfield_   = lelfield
+  lorbm_      = lorbm
   efield_     = efield
   nberrycyc_  = nberrycyc
   efield_cart_ = efield_cart
@@ -1103,6 +1125,7 @@ SUBROUTINE iosys()
   nstep_      = nstep
   iprint_     = iprint
   lecrpa_     = lecrpa
+  scf_must_converge_ = scf_must_converge
   !
   nat_     = nat
   ntyp_    = ntyp
@@ -1142,7 +1165,9 @@ SUBROUTINE iosys()
   !
   Hubbard_U_(1:ntyp)      = hubbard_u(1:ntyp)
   Hubbard_J_(1:3,1:ntyp)  = hubbard_j(1:3,1:ntyp)
+  Hubbard_J0_(1:ntyp)     = hubbard_j0(1:ntyp)
   Hubbard_alpha_(1:ntyp)  = hubbard_alpha(1:ntyp)
+  Hubbard_beta_(1:ntyp)   = hubbard_beta(1:ntyp)
   lda_plus_u_             = lda_plus_u
   lda_plus_u_kind_        = lda_plus_u_kind
   la2F_                   = la2F
@@ -1218,6 +1243,7 @@ SUBROUTINE iosys()
   solvationrad_( 1:ntyp ) = solvationrad( 1:ntyp )
   ALLOCATE( atomicspread_( ntyp ) )
   atomicspread_( 1:ntyp ) = atomicspread( 1:ntyp )
+  IF ( do_environ ) CALL environ_initions_allocate( nat_, ntyp )
   add_jellium_ = add_jellium
   !
   ifdtype_   = ifdtype
@@ -1230,9 +1256,8 @@ SUBROUTINE iosys()
   !
   delta_     = delta
   !
-  cion_       = cion*bohr_radius_si**3/amu_si
-  zion_       = zion
-  rhopb_      = rhopb
+  zion_      = zion
+  rhopb_     = rhopb
   solvent_temperature_ = solvent_temperature
   !
   SELECT CASE (TRIM(environ_type))
@@ -1242,20 +1267,24 @@ SUBROUTINE iosys()
     env_static_permittivity_ = 1.D0
     env_surface_tension_ = 0.D0
     env_pressure_ = 0.D0
-    env_slab_geometry = .false.
+    env_periodicity = 3
+    env_ioncc_concentration = 0.D0
   CASE ('water')
     ! water, experimental and SCCS tuned parameters
     env_static_permittivity_ = 78.3D0
     env_surface_tension_ = 50.D0*1.D-3*bohr_radius_si**2/rydberg_si
     env_pressure_ = -0.35D0*1.D9/rydberg_si*bohr_radius_si**3
-    env_slab_geometry = .false.
+    env_periodicity = 3
+    env_ioncc_concentration = 0.D0
   CASE ('input')
     ! take values from input, this is the default option
     env_static_permittivity_ = env_static_permittivity
     env_surface_tension_ = &
       env_surface_tension*1.D-3*bohr_radius_si**2/rydberg_si
     env_pressure_ = env_pressure*1.D9/rydberg_si*bohr_radius_si**3
-    env_slab_geometry = .false.
+    env_periodicity = 3
+    env_ioncc_concentration_ = env_ioncc_concentration &
+    * bohr_radius_si**3 / amu_si
   CASE DEFAULT
     call errore ('iosys','unrecognized value for environ_type',1) 
   END SELECT    
@@ -1288,6 +1317,9 @@ SUBROUTINE iosys()
     CASE( 'makov-payne', 'm-p', 'mp' )
       !
       do_makov_payne = .true.
+      IF ( ibrav < 1 .OR. ibrav > 3 ) CALL errore(' iosys', &
+              'Makov-Payne correction defined only for cubic lattices', 1)
+      !
       do_comp_mt     = .false.
       do_comp_esm    = .false.
       !
@@ -1310,30 +1342,38 @@ SUBROUTINE iosys()
 #ifdef __ENVIRON
     CASE( 'slabx' )
       !
-      do_environ_    = .true.
-      env_slab_geometry   = .true.
-      slab_axis      = 1
-      do_makov_payne = .false.
-      do_comp_mt     = .false.
-      do_comp_esm    = .false.
+      do_environ_     = .true.
+      env_periodicity = 2
+      slab_axis       = 1
+      do_makov_payne  = .false.
+      do_comp_mt      = .false.
+      do_comp_esm     = .false.
       !
     CASE( 'slaby' ) 
       !
-      do_environ_    = .true.
-      env_slab_geometry   = .true.
-      slab_axis      = 2
-      do_makov_payne = .false.
-      do_comp_mt     = .false.
-      do_comp_esm    = .false.
+      do_environ_     = .true.
+      env_periodicity = 2
+      slab_axis       = 2
+      do_makov_payne  = .false.
+      do_comp_mt      = .false.
+      do_comp_esm     = .false.
       !
     CASE( 'slabz' ) 
       !
-      do_environ_    = .true.
-      env_slab_geometry   = .true.
-      slab_axis      = 3
-      do_makov_payne = .false.
-      do_comp_mt     = .false.
-      do_comp_esm    = .false.
+      do_environ_     = .true.
+      env_periodicity = 2
+      slab_axis       = 3
+      do_makov_payne  = .false.
+      do_comp_mt      = .false.
+      do_comp_esm     = .false.
+      !
+    CASE( 'pcc' ) 
+      !
+      do_environ_     = .true.
+      env_periodicity = 0
+      do_makov_payne  = .false.
+      do_comp_mt      = .false.
+      do_comp_esm     = .false.
       !
 #endif
     CASE( 'none' )
@@ -1347,7 +1387,8 @@ SUBROUTINE iosys()
       call errore ('iosys','unrecognized value for assume_isolated',1)
   END SELECT
 #ifdef __ENVIRON
-  IF ( do_comp_mt .OR. env_slab_geometry ) add_jellium = .false.
+  IF ( env_ioncc_concentration .GT. 0.D0 .AND. env_periodicity .NE. 2 ) &
+      call errore ('iosys','ioncc requires slab boundary conditions',1)
 #endif
   !
   ! ... read following cards
@@ -1398,7 +1439,7 @@ SUBROUTINE iosys()
      wmass = sum( amass(ityp(:)) )
 #endif
      !
-     wmass = wmass * amconv
+     wmass = wmass * amu_ry
      IF ( calc == 'nd' .or. calc == 'nm' ) THEN
         wmass = 0.75D0 * wmass / pi / pi / omega**( 2.D0 / 3.D0 )
      ELSEIF ( calc == 'cd' .or. calc == 'cm' ) THEN
@@ -1411,7 +1452,7 @@ SUBROUTINE iosys()
      !
      ! ... wmass is given in amu, Renata's dynamics uses masses in atomic units
      !
-     cmass  = wmass * amconv
+     cmass  = wmass * amu_ry
      !
   ENDIF
   !
