@@ -67,9 +67,10 @@ SUBROUTINE setup()
   USE cellmd,             ONLY : calc
   USE uspp_param,         ONLY : upf, n_atom_wfc
   USE uspp,               ONLY : okvan
-  USE ldaU,               ONLY : lda_plus_u, lda_plus_u_kind, Hubbard_U, Hubbard_J, &
-                                 Hubbard_l, Hubbard_alpha, Hubbard_lmax, d_spin_ldau, oatwfc
-  USE bp,                 ONLY : gdir, lberry, nppstr, lelfield, nx_el, nppstr_3d,l3dstring, efield
+  USE ldaU,               ONLY : lda_plus_u, lda_plus_u_kind, U_projection, Hubbard_U, Hubbard_J, &
+                                 Hubbard_l, Hubbard_alpha, Hubbard_lmax, d_spin_ldau, oatwfc,&
+                                 Hubbard_J0, Hubbard_beta
+  USE bp,                 ONLY : gdir, lberry, nppstr, lelfield, lorbm, nx_el, nppstr_3d,l3dstring, efield
   USE fixed_occ,          ONLY : f_inp, tfixed_occ, one_atom_occupations
   USE funct,              ONLY : set_dft_from_name
   USE mp_global,          ONLY : kunit
@@ -404,7 +405,7 @@ SUBROUTINE setup()
   !
   IF ( nks_start == 0 ) THEN
      !
-     IF (lelfield) THEN
+     IF (lelfield .OR. lorbm) THEN
          !
         CALL kpoint_grid_efield (at,bg, npk, &
              k1,k2,k3, nk1,nk2,nk3, nkstot, xk, wk, nspin)
@@ -434,7 +435,12 @@ SUBROUTINE setup()
      !
      IF( lelfield) THEN
         !
-        allocate(nx_el(nkstot*nspin,3))
+        IF(noncolin) THEN
+           allocate(nx_el(nkstot,3))
+        ELSE
+           allocate(nx_el(nkstot*nspin,3))
+        END IF
+
         ! <AF>
         IF ( gdir<0 .OR. gdir>3 ) CALL errore('setup','invalid gdir value',10) 
         IF ( gdir == 0 ) CALL errore('setup','needed gdir probably not set',10) 
@@ -587,7 +593,8 @@ SUBROUTINE setup()
      !
      DO nt = 1, ntyp
         !
-        IF ( Hubbard_U(nt)/=0.d0.OR.Hubbard_alpha(nt)/=0.D0 ) THEN
+        IF ( Hubbard_U(nt)/=0.d0.OR.Hubbard_alpha(nt)/=0.D0 .OR.&
+             Hubbard_J0(nt) /=0.d0 .OR. Hubbard_beta(nt) /=0.d0) THEN
            !
            Hubbard_l(nt) = set_Hubbard_l( upf(nt)%psd )
            !
@@ -598,6 +605,8 @@ SUBROUTINE setup()
      END DO
    elseif ( lda_plus_u_kind.eq.1 ) then
      !
+     IF ( U_projection == 'pseudo' ) CALL errore( 'setup', &
+        & 'full LDA+U not implemented with pseudo projection type', 1 )
 
      if (noncolin) then
       ALLOCATE( d_spin_ldau(2,2,48) )
