@@ -8,16 +8,16 @@
 !--------------------------------------------------------------------------
 !
 MODULE symm_base
-  
+
   USE kinds,      ONLY : DP
   USE cell_base,  ONLY : at, bg
   !
   ! ... The variables needed to describe the symmetry properties
   ! ... and the routines to find crystal symmetries
-  !  
+  !
   ! ... these are acceptance criteria
   !
-  REAL(DP), parameter :: eps1 = 1.0d-6, eps2 = 1.0d-5
+  REAL(DP), PARAMETER :: eps1 = 1.0d-6, eps2 = 1.0d-5
   !
   SAVE
   !
@@ -32,19 +32,21 @@ MODULE symm_base
        s(3,3,48),            &! symmetry matrices, in crystal axis
        invs(48),             &! index of inverse operation: S^{-1}_i=S(invs(i))
        ftau(3,48),           &! fractional translations, in FFT coordinates
-       nrot,                 &! number of bravais lattice symmetries 
+       nrot,                 &! number of bravais lattice symmetries
        nsym = 1,             &! total number of crystal symmetries
        nsym_ns = 0,          &! nonsymmorphic (fractional translation) symms
        nsym_na = 0            ! excluded nonsymmorphic symmetries because
                               ! fract. transl. is noncommensurate with FFT grid
   REAL (DP) :: &
        ft (3,48),            &! fractional translations, in crystal axis
-       sr (3,3,48)            ! symmetry matrices, in cartesian axis
+       sr (3,3,48),          &! symmetry matrices, in cartesian axis
+       accep = 1.0d-5         ! initial value of the acceptance threshold
+                              ! for position comparison by eqvect in checksym
   !
   ! ... note: ftau are used for symmetrization in real space (phonon, exx)
   ! ... in which case they must be commensurated with the FFT grid
   !
-  CHARACTER(LEN=45) ::  sname(48)   ! name of the symmetries
+  CHARACTER(len=45) ::  sname(48)   ! name of the symmetries
   INTEGER :: &
        t_rev(48) = 0          ! time reversal flag, for noncolinear magnetism
   INTEGER, ALLOCATABLE :: &
@@ -52,14 +54,14 @@ MODULE symm_base
   LOGICAL :: &
        time_reversal=.true., &! if .TRUE. the system has time reversal symmetry
        invsym,               &! if .TRUE. the system has inversion symmetry
-       nofrac= .FALSE.,      &! if .TRUE. fract. translations are not allowed
-       allfrac= .FALSE.,     &! if .TRUE. all fractionary transations allowed,
+       nofrac= .false.,      &! if .TRUE. fract. translations are not allowed
+       allfrac= .false.,     &! if .TRUE. all fractionary translations allowed,
                               ! even those not commensurate with FFT grid
-       nosym = .FALSE.,      &! if .TRUE. no symmetry is used
-       nosym_evc = .FALSE.,  &! if .TRUE. symmetry is used only to symmetrize
+       nosym = .false.,      &! if .TRUE. no symmetry is used
+       nosym_evc = .false.,  &! if .TRUE. symmetry is used only to symmetrize
                               ! k points
-       no_t_rev=.FALSE.       ! if .TRUE. remove the symmetries that 
-                              ! require time reversal               
+       no_t_rev=.false.       ! if .TRUE. remove the symmetries that
+                              ! require time reversal
   REAL(DP),TARGET :: &
        d1(3,3,48),           &! matrices for rotating spherical
        d2(5,5,48),           &! harmonics (d1 for l=1, ...)
@@ -83,46 +85,46 @@ CONTAINS
      LOGICAL :: found
      !
      DO isym = 1, nsym
-        found = .FALSE.
+        found = .false.
         DO jsym = 1, nsym
            !
-           ss = MATMUL (s(:,:,jsym),s(:,:,isym))
+           ss = matmul (s(:,:,jsym),s(:,:,isym))
            ! s(:,:,1) is the identity
-           IF ( ALL ( s(:,:,1) == ss(:,:) ) ) THEN
+           IF ( all ( s(:,:,1) == ss(:,:) ) ) THEN
               invs (isym) = jsym
-              found = .TRUE.
-           END IF
-        END DO
-        IF ( .NOT.found) CALL errore ('inverse_s', ' Not a group', 1)
-     END DO
+              found = .true.
+           ENDIF
+        ENDDO
+        IF ( .not.found) CALL errore ('inverse_s', ' Not a group', 1)
+     ENDDO
      !
-   END SUBROUTINE inverse_s 
+   END SUBROUTINE inverse_s
    !
 !-----------------------------------------------------------------------
-subroutine set_sym_bl ( )
+SUBROUTINE set_sym_bl ( )
   !-----------------------------------------------------------------------
   !
   ! Provides symmetry operations for all bravais lattices
-  ! Tests first the 24 proper rotations for the cubic lattice; 
+  ! Tests first the 24 proper rotations for the cubic lattice;
   ! then the 8 rotations specific for the hexagonal axis (special axis c);
   ! then inversion is added
   !
-  implicit none
+  IMPLICIT NONE
   !
   ! sin3 = sin(pi/3), cos3 = cos(pi/3), msin3 = -sin(pi/3), mcos3 = -cos(pi/3)
   !
-  real(DP), parameter :: sin3 = 0.866025403784438597d0, cos3 = 0.5d0, &
-                             msin3 =-0.866025403784438597d0, mcos3 = -0.5d0
+  real(DP), PARAMETER :: sin3 = 0.866025403784438597d0, cos3 = 0.5d0, &
+                        msin3 =-0.866025403784438597d0, mcos3 = -0.5d0
   real(DP) :: s0(3, 3, 32), overlap (3, 3), rat (3), rot (3, 3), value
   ! s0: the s matrices in cartesian axis
   ! overlap: inverse overlap matrix between direct lattice
   ! rat: the rotated of a direct vector ( cartesian )
   ! rot: the rotated of a direct vector ( crystal axis )
   ! value: component of the s matrix in axis basis
-  integer :: jpol, kpol, mpol, irot
+  INTEGER :: jpol, kpol, mpol, irot, imat(24)
   ! counters over the polarizations and the rotations
 
-  character :: s0name (64) * 45
+  CHARACTER (len=45) :: s0name (64)
   ! full name of the rotational part of each symmetry operation
 
   data s0/ 1.d0,  0.d0,  0.d0,  0.d0,  1.d0,  0.d0,  0.d0,  0.d0,  1.d0, &
@@ -213,7 +215,7 @@ subroutine set_sym_bl ( )
                 'inv. 120 deg rotation - cart. axis [1,1,1]   ',&
                 'inv. 120 deg rotation - cart. axis [-1,1,-1] ',&
                 'inv. 120 deg rotation - cart. axis [1,-1,-1] ',&
-                'inv. 120 deg rotation - cart. axis [-1,-1,1] ',& 
+                'inv. 120 deg rotation - cart. axis [-1,-1,1] ',&
                 'inv.  60 deg rotation - cryst. axis [0,0,1]  ',&
                 'inv.  60 deg rotation - cryst. axis [0,0,-1] ',&
                 'inv. 120 deg rotation - cryst. axis [0,0,1]  ',&
@@ -225,83 +227,95 @@ subroutine set_sym_bl ( )
 
   !    compute the overlap matrix for crystal axis
 
-  do jpol = 1,3
-     do kpol = 1,3
+  DO jpol = 1,3
+     DO kpol = 1,3
         rot(kpol,jpol) = at(1,kpol)*at(1,jpol) +&
                          at(2,kpol)*at(2,jpol) +&
                          at(3,kpol)*at(3,jpol)
-     enddo
-  enddo
+     ENDDO
+  ENDDO
   !
   !    then its inverse (rot is used as work space)
   !
-  call invmat (3, rot, overlap, value)
+  CALL invmat (3, rot, overlap, value)
 
   nrot = 1
-  do irot = 1,32
+  DO irot = 1,32
      !
      !   for each possible symmetry
      !
-     do jpol = 1,3
-        do mpol = 1,3
+     DO jpol = 1,3
+        DO mpol = 1,3
            !
            !   compute, in cartesian coordinates the rotated vector
            !
            rat(mpol) = s0(mpol,1,irot)*at(1,jpol) +&
                        s0(mpol,2,irot)*at(2,jpol) +&
                        s0(mpol,3,irot)*at(3,jpol)
-        enddo
+        ENDDO
 
-        do kpol = 1,3
+        DO kpol = 1,3
            !
            !   the rotated vector is projected on the direct lattice
            !
            rot(kpol,jpol) = at(1,kpol)*rat(1) +&
                             at(2,kpol)*rat(2) +&
                             at(3,kpol)*rat(3)
-        enddo
-     enddo
+        ENDDO
+     ENDDO
      !
      !  and the inverse of the overlap matrix is applied
      !
-     do jpol = 1,3
-        do kpol = 1,3
+     DO jpol = 1,3
+        DO kpol = 1,3
            value = overlap(jpol,1)*rot(1,kpol) +&
            &       overlap(jpol,2)*rot(2,kpol) +&
            &       overlap(jpol,3)*rot(3,kpol)
-           if ( abs(DBLE(nint(value))-value) > eps1 ) then
+           IF ( abs(dble(nint(value))-value) > eps1 ) THEN
               !
               ! if a noninteger is obtained, this implies that this operation
               ! is not a symmetry operation for the given lattice
               !
-              go to 10
-           end if
+              GOTO 10
+           ENDIF
            s(kpol,jpol,nrot) = nint(value)
-           sname(nrot)=s0name(irot)
-        enddo
-     enddo
+        ENDDO
+     ENDDO
+     sname(nrot)=s0name(irot)
+     imat(nrot)=irot
      nrot = nrot+1
-10   continue
-  enddo
+     IF (nrot > 25) CALL errore('set_sym_bl','some problem with symmetries',1)
+10   CONTINUE
+  ENDDO
   nrot = nrot-1
+  IF ( nrot /= 1 .AND. nrot /= 2 .AND. nrot /= 4 .AND. nrot /= 6 .AND. &
+       nrot /= 8 .AND. nrot /=12 .AND. nrot /=24 ) CALL errore('set_sym_bl',&
+         'wrong number of symmetries! Use standard orientations for axis',nrot)
   !
   !     set the inversion symmetry ( Bravais lattices have always inversion
   !     symmetry )
   !
-  do irot = 1, nrot
-     do kpol = 1,3
-        do jpol = 1,3
+  DO irot = 1, nrot
+     DO kpol = 1,3
+        DO jpol = 1,3
            s(kpol,jpol,irot+nrot) = -s(kpol,jpol,irot)
-           sname(irot+nrot) = s0name(irot+32)
-        end do
-     end do
-  end do
-
+           sname(irot+nrot) = s0name(imat(irot)+32)
+        ENDDO
+     ENDDO
+  ENDDO
   nrot = 2*nrot
 
-  return
+  !    This happens for instance for an hexagonal lattice with one axis 
+  !    oriented at 15 degrees from the x axis, the opther along (-1,1,0)
+  
+  IF ( .not. is_group ( nrot ) ) THEN
+     CALL errore ('set_sym_bl', &
+         'Symmetry group not a group! Use standard orientations for axis',1)
+  ENDIF
   !
-end subroutine set_sym_bl
+  RETURN
+  !
+END SUBROUTINE set_sym_bl
 !
 !-----------------------------------------------------------------------
 SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
@@ -311,105 +325,117 @@ SUBROUTINE find_sym ( nat, tau, ityp, nr1, nr2, nr3, magnetic_sym, m_loc )
   !     the symmetries of the Bravais lattice which are not allowed
   !     by the atomic positions (or by the magnetization if present)
   !
-  implicit none
+  IMPLICIT NONE
   !
-  integer, intent(in) :: nat, ityp (nat), nr1, nr2, nr3  
-  real(DP), intent(in) :: tau (3,nat), m_loc(3,nat)
-  logical, intent(in) :: magnetic_sym
+  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
+  real(DP), INTENT(in) :: tau (3,nat), m_loc(3,nat)
+  LOGICAL, INTENT(in) :: magnetic_sym
   !
-  logical :: sym (48)
+  INTEGER :: i
+  LOGICAL :: sym (48)
   ! if true the corresponding operation is a symmetry operation
   !
-  IF ( .NOT. ALLOCATED(irt) ) ALLOCATE( irt( 48, nat ) )
+  IF ( .not. allocated(irt) ) ALLOCATE( irt( 48, nat ) )
   irt( :, : ) = 0
   !
   !    Here we find the true symmetries of the crystal
   !
-  CALL sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
-  !
-  !    Here we check for magnetic symmetries
-  !
-  IF ( magnetic_sym ) CALL sgam_at_mag ( nat, m_loc, sym )
-  !
-  !  If nosym_evc is true from now on we do not use the symmetry any more
-  !
-  IF (nosym_evc) THEN
-     sym=.false.
-     sym(1)=.true.
-  ENDIF
-  !
-  !    Here we re-order all rotations in such a way that true sym.ops 
-  !    are the first nsym; rotations that are not sym.ops. follow
-  !
-  nsym = copy_sym ( nrot, sym )
-  !
-  IF ( .not. is_group ( ) ) THEN
-     CALL infomsg ('find_sym', 'Not a group! symmetry disabled')
-     nsym = 1
-  END IF
+  symm: DO i=1,3 !emine: if it is not resolved in 3 steps it is sth else?
+    CALL sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
+    !
+    !    Here we check for magnetic symmetries
+    !
+    IF ( magnetic_sym ) CALL sgam_at_mag ( nat, m_loc, sym )
+    !
+    !  If nosym_evc is true from now on we do not use the symmetry any more
+    !
+    IF (nosym_evc) THEN
+       sym=.false.
+       sym(1)=.true.
+    ENDIF
+    !
+    !    Here we re-order all rotations in such a way that true sym.ops
+    !    are the first nsym; rotations that are not sym.ops. follow
+    !
+    nsym = copy_sym ( nrot, sym )
+    !
+    IF ( .not. is_group ( nsym ) ) THEN
+       IF (i == 1) CALL infomsg ('find_sym', &
+                      'Not a group! Trying with lower acceptance parameter...')
+       accep = accep * 0.5d0
+       IF (i == 3) THEN
+         CALL infomsg ('find_sym', 'Still not a group! symmetry disabled')
+         nsym = 1
+       ENDIF
+       CYCLE symm
+    ELSE
+       IF (i > 1) CALL infomsg ('find_sym', 'Symmetry operations form a group')
+       exit symm
+    ENDIF
+  ENDDO symm
   !
   ! check if inversion (I) is a symmetry.
   ! If so, it should be the (nsym/2+1)-th operation of the group
   !
-  invsym = ALL ( s(:,:,nsym/2+1) == -s(:,:,1) )
+  invsym = all ( s(:,:,nsym/2+1) == -s(:,:,1) )
   !
-  CALL inverse_s ( ) 
+  CALL inverse_s ( )
   !
-  CALL s_axis_to_cart ( ) 
+  CALL s_axis_to_cart ( )
   !
-  return
+  RETURN
   !
 END SUBROUTINE find_sym
 !
 !-----------------------------------------------------------------------
-subroutine sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
+SUBROUTINE sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
   !-----------------------------------------------------------------------
   !
-  !     Given the point group of the Bravais lattice, this routine finds 
+  !     Given the point group of the Bravais lattice, this routine finds
   !     the subgroup which is the point group of the considered crystal.
   !     Non symmorphic groups are allowed, provided that fractional
   !     translations are allowed (nofrac=.false), that the unit cell is
   !     not a supercell, and that they are commensurate with the FFT grid
   !
   !     On output, the array sym is set to .true.. for each operation
-  !     of the original point group that is also a symmetry operation 
+  !     of the original point group that is also a symmetry operation
   !     of the crystal symmetry point group
   !
   USE io_global,  ONLY : stdout
   USE kinds
-  implicit none
+  IMPLICIT NONE
   !
-  integer, intent(in) :: nat, ityp (nat), nr1, nr2, nr3
+  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
   ! nat  : number of atoms in the unit cell
   ! ityp : species of each atom in the unit cell
   ! nr*  : dimensions of the FFT mesh
   !
-  real(DP), intent(in) :: tau (3, nat)
+  real(DP), INTENT(in) :: tau (3, nat)
   !
   ! tau  : cartesian coordinates of the atoms
   !
   !     output variables
   !
-  logical, intent(out) :: sym (48)
+  LOGICAL, INTENT(out) :: sym (48)
   ! sym(isym)    : flag indicating if sym.op. isym in the parent group
   !                is a true symmetry operation of the crystal
   !
-  integer :: na, kpol, nb, irot, i, j
+  INTEGER :: na, kpol, nb, irot, i, j
   ! counters
-  real(DP) , allocatable :: xau (:,:), rau (:,:)
+  real(DP) , ALLOCATABLE :: xau (:,:), rau (:,:)
   ! atomic coordinates in crystal axis
-  logical :: fractional_translations
+  LOGICAL :: fractional_translations
   real(DP) :: ft_(3), ft1, ft2, ft3
   !
-  allocate(xau(3,nat))
-  allocate(rau(3,nat))
+  ALLOCATE(xau(3,nat))
+  ALLOCATE(rau(3,nat))
   !
   !     Compute the coordinates of each atom in the basis of
   !     the direct lattice vectors
   !
-  do na = 1, nat
+  DO na = 1, nat
      xau(:,na) = bg(1,:) * tau(1,na) + bg(2,:) * tau(2,na) + bg(3,:) * tau(3,na)
-  enddo
+  ENDDO
   !
   !      check if the identity has fractional translations
   !      (this means that the cell is actually a supercell).
@@ -421,50 +447,50 @@ subroutine sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
   irot = 1
   !
   fractional_translations = .not. nofrac
-  do na = 2, nat
-     if ( fractional_translations ) then
-        if (ityp (nb) == ityp (na) ) then
+  DO na = 2, nat
+     IF ( fractional_translations ) THEN
+        IF (ityp (nb) == ityp (na) ) THEN
            ft_(:) = xau(:,na) - xau(:,nb) - nint( xau(:,na) - xau(:,nb) )
            !
            sym(irot) = checksym ( irot, nat, ityp, xau, xau, ft_ )
            !
-           if ( sym (irot) .and. &
+           IF ( sym (irot) .and. &
                (abs (ft_(1) **2 + ft_(2) **2 + ft_(3) **2) < 1.d-8) ) &
-               call errore ('sgam_at', 'overlapping atoms', na)
-           if (sym (irot) ) then
+               CALL errore ('sgam_at', 'overlapping atoms', na)
+           IF (sym (irot) ) THEN
               fractional_translations = .false.
               WRITE( stdout, '(5x,"Found symmetry operation: I + (",&
              &   3f8.4, ")",/,5x,"This is a supercell,", &
              &   " fractional translations are disabled")') ft_
-           endif
-        endif
-     end if
-  enddo
+           ENDIF
+        ENDIF
+     ENDIF
+  ENDDO
   !
-  nsym_ns = 0 
-  do irot = 1, nrot
+  nsym_ns = 0
+  DO irot = 1, nrot
      !
      ! check that the grid is compatible with the S rotation
      !
-     if ( mod (s (2, 1, irot) * nr1, nr2) /= 0 .or. &
+     IF ( mod (s (2, 1, irot) * nr1, nr2) /= 0 .or. &
           mod (s (3, 1, irot) * nr1, nr3) /= 0 .or. &
           mod (s (1, 2, irot) * nr2, nr1) /= 0 .or. &
           mod (s (3, 2, irot) * nr2, nr3) /= 0 .or. &
           mod (s (1, 3, irot) * nr3, nr1) /= 0 .or. &
-          mod (s (2, 3, irot) * nr3, nr2) /= 0 ) then
+          mod (s (2, 3, irot) * nr3, nr2) /= 0 ) THEN
         sym (irot) = .false.
         WRITE( stdout, '(5x,"warning: symmetry operation # ",i2, &
              &         " not compatible with FFT grid. ")') irot
         WRITE( stdout, '(3i4)') ( (s (i, j, irot) , j = 1, 3) , i = 1, 3)
-        goto 100
-     endif
+        GOTO 100
+     ENDIF
 
-     do na = 1, nat
+     DO na = 1, nat
         ! rau = rotated atom coordinates
         rau (:, na) = s (1,:, irot) * xau (1, na) + &
                       s (2,:, irot) * xau (2, na) + &
                       s (3,:, irot) * xau (3, na)
-     enddo
+     ENDDO
      !
      !      first attempt: no fractional translation
      !
@@ -474,10 +500,10 @@ subroutine sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
      !
      sym(irot) = checksym ( irot, nat, ityp, xau, rau, ft_ )
      !
-     if (.not.sym (irot) .and. fractional_translations) then
+     IF (.not.sym (irot) .and. fractional_translations) THEN
         nb = 1
-        do na = 1, nat
-           if (ityp (nb) == ityp (na) ) then
+        DO na = 1, nat
+           IF (ityp (nb) == ityp (na) ) THEN
               !
               !      second attempt: check all possible fractional translations
               !
@@ -485,33 +511,33 @@ subroutine sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
               !
               sym(irot) = checksym ( irot, nat, ityp, xau, rau, ft_ )
               !
-              if (sym (irot) ) then
+              IF (sym (irot) ) THEN
                  nsym_ns = nsym_ns + 1
                  ft (:,irot) = ft_(:)
-                 go to 100
-              end if
-           endif
-        enddo
+                 GOTO 100
+              ENDIF
+           ENDIF
+        ENDDO
 
-     endif
-100  continue
-  enddo
+     ENDIF
+100  CONTINUE
+  ENDDO
   !
   ! convert ft to FFT coordinates, check if compatible with FFT grid
   ! for real-space symmetrization (if done: currently, exx, phonon)
-  ! 
+  !
   nsym_na = 0
-  do irot =1, nrot
-     if ( sym(irot) .AND. .NOT. allfrac ) then
+  DO irot =1, nrot
+     IF ( sym(irot) .and. .not. allfrac ) THEN
         ft1 = ft(1,irot) * nr1
         ft2 = ft(2,irot) * nr2
         ft3 = ft(3,irot) * nr3
         ! check if the fractional translations are commensurate
         ! with the FFT grid, discard sym.op. if not
         ! (needed because ph.x symmetrizes in real space)
-        if (abs (ft1 - nint (ft1) ) / nr1 > eps2 .or. &
+        IF (abs (ft1 - nint (ft1) ) / nr1 > eps2 .or. &
             abs (ft2 - nint (ft2) ) / nr2 > eps2 .or. &
-            abs (ft3 - nint (ft3) ) / nr3 > eps2 ) then
+            abs (ft3 - nint (ft3) ) / nr3 > eps2 ) THEN
             !     WRITE( stdout, '(5x,"warning: symmetry operation", &
             !          &     " # ",i2," not allowed.   fractional ", &
             !          &     "translation:"/5x,3f11.7,"  in crystal", &
@@ -519,81 +545,81 @@ subroutine sgam_at ( nat, tau, ityp, nr1, nr2, nr3, sym )
             sym (irot) = .false.
             nsym_na = nsym_na + 1
             nsym_ns = nsym_ns - 1
-         endif
+         ENDIF
          ftau (1, irot) = nint (ft1)
          ftau (2, irot) = nint (ft2)
          ftau (3, irot) = nint (ft3)
-      end if
-  end do
+      ENDIF
+  ENDDO
   !
   !   deallocate work space
   !
-  deallocate (rau)
-  deallocate (xau)
+  DEALLOCATE (rau)
+  DEALLOCATE (xau)
   !
-  return
-end subroutine sgam_at
+  RETURN
+END SUBROUTINE sgam_at
 !
 !-----------------------------------------------------------------------
-subroutine sgam_at_mag ( nat, m_loc, sym )
+SUBROUTINE sgam_at_mag ( nat, m_loc, sym )
   !-----------------------------------------------------------------------
   !
   !   Find magnetic symmetries, i.e. point-group symmetries that are
   !   also symmetries of the local magnetization - including
   !   rotation + time reversal operations
   !
-  implicit none
+  IMPLICIT NONE
   !
-  integer, intent(in) :: nat
-  real(DP), intent(in) :: m_loc(3, nat)
+  INTEGER, INTENT(in) :: nat
+  real(DP), INTENT(in) :: m_loc(3, nat)
   !
   ! m_loc: local magnetization, must be invariant under the sym.op.
   !
-  logical, intent(inout) :: sym (48)
+  LOGICAL, INTENT(inout) :: sym (48)
   !
   ! sym(isym) = .true. if rotation isym is a sym.op. of the crystal
   !                    (i.e. not of the bravais lattice only)
   !
-  integer :: na, nb, irot
-  logical :: t1, t2
-  real(DP) , allocatable ::  mxau(:,:), mrau(:,:)
+  INTEGER :: na, nb, irot
+  LOGICAL :: t1, t2
+  real(DP) , ALLOCATABLE ::  mxau(:,:), mrau(:,:)
   ! magnetization and rotated magnetization in crystal axis
   !
-  allocate ( mxau(3,nat), mrau(3,nat) )
+  ALLOCATE ( mxau(3,nat), mrau(3,nat) )
   !
   !     Compute the local magnetization of each atom in the basis of
   !     the direct lattice vectors
   !
-  do na = 1, nat
+  DO na = 1, nat
      mxau (:, na)= bg (1, :) * m_loc (1, na) + &
                    bg (2, :) * m_loc (2, na) + &
                    bg (3, :) * m_loc (3, na)
-  enddo
+  ENDDO
   !
-  do irot = 1, nrot
+  DO irot = 1, nrot
      !
      t_rev(irot) = 0
      !
-     if ( sym (irot) ) then
+     IF ( sym (irot) ) THEN
         !
         ! mrau = rotated local magnetization
         !
-        do na = 1, nat
+        DO na = 1, nat
             mrau(:,na) = s(1,:,irot) * mxau(1,na) + &
                          s(2,:,irot) * mxau(2,na) + &
-                         s(3,:,irot) * mxau(3,na)  
-        enddo
-        if (sname(irot)(1:3)=='inv') mrau = -mrau
+                         s(3,:,irot) * mxau(3,na)
+        ENDDO
+        IF (sname(irot)(1:3)=='inv') mrau = -mrau
         !
         ! check if this a magnetic symmetry
         !
         t1 = .true.
         t2 = .true.
-        do na = 1, nat
+        DO na = 1, nat
            !
            nb = irt (irot,na)
-           if ( nb < 1 .or. nb > nat ) call errore ('check_mag_sym', &
-               'internal error: out-of-bound atomic index', na) 
+           IF ( nb < 1 .or. nb > nat ) CALL errore ('check_mag_sym', &
+               'internal error: out-of-bound atomic index', na)
            !
            t1 = ( abs(mrau(1,na) - mxau(1,nb)) +       &
                   abs(mrau(2,na) - mxau(2,nb)) +       &
@@ -602,29 +628,29 @@ subroutine sgam_at_mag ( nat, m_loc, sym )
                   abs(mrau(2,na) + mxau(2,nb))+       &
                   abs(mrau(3,na) + mxau(3,nb)) < eps2 ) .and. t2
            !
-        enddo
+        ENDDO
         !
-        if ( .not.t1 .and. .not.t2 ) then
+        IF ( .not.t1 .and. .not.t2 ) THEN
            ! not a magnetic symmetry
            sym(irot) = .false.
-        else if( t2 .and. .not. t1 ) then
+        ELSEIF( t2 .and. .not. t1 ) THEN
            ! magnetic symmetry with time reversal, if allowed
            IF (no_t_rev) THEN
               sym(irot) = .false.
            ELSE
               t_rev(irot) = 1
            ENDIF
-        end if
+        ENDIF
         !
-     end if
+     ENDIF
      !
-  enddo
+  ENDDO
   !
   !   deallocate work space
   !
-  deallocate ( mrau, mxau )
+  DEALLOCATE ( mrau, mxau )
   !
-  return
+  RETURN
 END SUBROUTINE sgam_at_mag
 !
 SUBROUTINE set_sym(nat, tau, ityp, nspin_mag, m_loc, nr1, nr2, nr3)
@@ -634,14 +660,14 @@ SUBROUTINE set_sym(nat, tau, ityp, nspin_mag, m_loc, nr1, nr2, nr3)
   ! dimensions nr1, nr2, nr3; it sets the symmetry elements of this module.
   ! Note that at and bg are those in cell_base. It sets nrot, nsym, s,
   ! sname, sr, invs, ftau, irt, t_rev,  time_reversal, and invsym
-  ! 
+  !
   !-----------------------------------------------------------------------
   !
   IMPLICIT NONE
-  ! input 
-  INTEGER, INTENT(IN)  :: nat, ityp(nat), nspin_mag, nr1, nr2, nr3
-  REAL(DP), INTENT(IN) :: tau(3,nat)
-  REAL(DP), INTENT(IN) :: m_loc(3,nat) 
+  ! input
+  INTEGER, INTENT(in)  :: nat, ityp(nat), nspin_mag, nr1, nr2, nr3
+  REAL(DP), INTENT(in) :: tau(3,nat)
+  REAL(DP), INTENT(in) :: m_loc(3,nat)
   !
   time_reversal = (nspin_mag /= 4)
   t_rev(:) = 0
@@ -652,29 +678,29 @@ SUBROUTINE set_sym(nat, tau, ityp, nspin_mag, m_loc, nr1, nr2, nr3)
   END SUBROUTINE set_sym
 !
 !-----------------------------------------------------------------------
-INTEGER FUNCTION copy_sym ( nrot_, sym ) 
+INTEGER FUNCTION copy_sym ( nrot_, sym )
 !-----------------------------------------------------------------------
   !
-  implicit none
-  integer, intent(in) :: nrot_
-  logical, intent(inout) :: sym(48)
+  IMPLICIT NONE
+  INTEGER, INTENT(in) :: nrot_
+  LOGICAL, INTENT(inout) :: sym(48)
   !
-  integer :: stemp(3,3), ftemp(3), ttemp, irot, jrot
+  INTEGER :: stemp(3,3), ftemp(3), ttemp, irot, jrot
   REAL(dp) :: ft_(3)
-  integer, allocatable :: irtemp(:)
-  character(len=45) :: nametemp
+  INTEGER, ALLOCATABLE :: irtemp(:)
+  CHARACTER(len=45) :: nametemp
   !
   ! copy symm. operations in sequential order so that
   ! s(i,j,irot) , irot <= nsym          are the sym.ops. of the crystal
   !               nsym+1 < irot <= nrot are the sym.ops. of the lattice
   ! on exit copy_sym returns nsym
   !
-  allocate ( irtemp( size(irt,2) ) )
+  ALLOCATE ( irtemp( size(irt,2) ) )
   jrot = 0
-  do irot = 1, nrot_
-     if (sym (irot) ) then
+  DO irot = 1, nrot_
+     IF (sym (irot) ) THEN
         jrot = jrot + 1
-        if ( irot > jrot ) then 
+        IF ( irot > jrot ) THEN
            stemp = s(:,:,jrot)
            s (:,:, jrot) = s (:,:, irot)
            s (:,:, irot) = stemp
@@ -693,70 +719,71 @@ INTEGER FUNCTION copy_sym ( nrot_, sym )
            ttemp = t_rev(jrot)
            t_rev(jrot) = t_rev(irot)
            t_rev(irot) = ttemp
-        endif
-     endif
-  enddo
+        ENDIF
+     ENDIF
+  ENDDO
   sym (1:jrot) = .true.
   sym (jrot+1:nrot_) = .false.
-  deallocate ( irtemp )
+  DEALLOCATE ( irtemp )
   !
   copy_sym = jrot
-  return
+  RETURN
   !
 END FUNCTION copy_sym
 
 !
 !-----------------------------------------------------------------------
-LOGICAL FUNCTION is_group ( )
+LOGICAL FUNCTION is_group ( nsym_ )
   !-----------------------------------------------------------------------
   !
-  !  Checks that {S} is a group 
+  !  Checks that {S} is a group
   !
   IMPLICIT NONE
   !
+  INTEGER, INTENT(in) :: nsym_
   INTEGER :: isym, jsym, ksym, ss (3, 3)
   REAL (DP) :: st(3), dt(3)
   LOGICAL :: found
   !
-  DO isym = 1, nsym
-     DO jsym = 1, nsym
-        ! 
-        ss = MATMUL (s(:,:,isym),s(:,:,jsym))
+  DO isym = 1, nsym_
+     DO jsym = 1, nsym_
+        !
+        ss = matmul (s(:,:,isym),s(:,:,jsym))
         st(:)= ft(:,jsym) + s(1,:,jsym)*ft(1,isym) + &
                             s(2,:,jsym)*ft(2,isym) + &
-                            s(3,:,jsym)*ft(3,isym) 
+                            s(3,:,jsym)*ft(3,isym)
         !
         !     here we check that the input matrices really form a group:
         !        S(k)   = S(i)*S(j)
         !        ftau_k = S(j)*ftau_i+ftau_j (modulo a lattice vector)
         !
         found = .false.
-        DO ksym = 1, nsym
-           dt(:) = ft(:,ksym) - st(:) - NINT( ft(:,ksym) - st(:) ) 
-           IF ( ALL( s(:,:,ksym) == ss(:,:) ) .AND. &
-                ( ABS ( dt(1) ) < eps2 ) .AND. &
-                ( ABS ( dt(2) ) < eps2 ) .AND. &
-                ( ABS ( dt(3) ) < eps2 ) ) THEN
+        DO ksym = 1, nsym_
+           dt(:) = ft(:,ksym) - st(:) - nint( ft(:,ksym) - st(:) )
+           IF ( all( s(:,:,ksym) == ss(:,:) ) .and. &
+                ( abs ( dt(1) ) < eps2 ) .and. &
+                ( abs ( dt(2) ) < eps2 ) .and. &
+                ( abs ( dt(3) ) < eps2 ) ) THEN
               IF (found) THEN
                  is_group = .false.
                  RETURN
-              END IF
+              ENDIF
               found = .true.
-           END IF
-        END DO
-        IF ( .NOT.found) then
+           ENDIF
+        ENDDO
+        IF ( .not.found) THEN
            is_group = .false.
            RETURN
-        END IF
-     END DO
-  END DO
+        ENDIF
+     ENDDO
+  ENDDO
   is_group=.true.
   RETURN
   !
 END FUNCTION is_group
 !
 !-----------------------------------------------------------------------
-logical function checksym ( irot, nat, ityp, xau, rau, ft_ )
+LOGICAL FUNCTION checksym ( irot, nat, ityp, xau, rau, ft_ )
   !-----------------------------------------------------------------------
   !
   !   This function receives as input all the atomic positions xau,
@@ -765,146 +792,146 @@ logical function checksym ( irot, nat, ityp, xau, rau, ft_ )
   !   which is of the same type of na, and coincide with it after the
   !   symmetry operation. Fractional translations are allowed.
   !
-  implicit none
+  IMPLICIT NONE
   !
-  integer, intent(in) :: nat, ityp (nat), irot
+  INTEGER, INTENT(in) :: nat, ityp (nat), irot
   ! nat : number of atoms
   ! ityp: the type of each atom
-  real(DP), intent(in) :: xau (3, nat), rau (3, nat), ft_(3)
+  real(DP), INTENT(in) :: xau (3, nat), rau (3, nat), ft_(3)
   ! xau: the initial vectors (in crystal coordinates)
   ! rau: the rotated vectors (as above)
   ! ft_: fractionary translation (as above)
   !
-  integer :: na, nb
-  logical, external :: eqvect
+  INTEGER :: na, nb
+  LOGICAL, EXTERNAL :: eqvect
   ! the testing function
   !
-  do na = 1, nat
-     do nb = 1, nat
+  DO na = 1, nat
+     DO nb = 1, nat
         checksym = ( ityp (na) == ityp (nb) .and. &
-                     eqvect (rau (1, na), xau (1, nb), ft_) )
-        if ( checksym ) then
+                     eqvect (rau (1, na), xau (1, nb), ft_ , accep) )
+        IF ( checksym ) THEN
            !
            ! the rotated atom does coincide with one of the like atoms
            ! keep track of which atom the rotated atom coincides with
            !
            irt (irot, na) = nb
-           goto 10
-        endif
-     enddo
+           GOTO 10
+        ENDIF
+     ENDDO
      !
      ! the rotated atom does not coincide with any of the like atoms
      ! s(ir) + ft is not a symmetry operation
      !
-     return 
-10   continue
-  enddo
+     RETURN
+10   CONTINUE
+  ENDDO
   !
   ! s(ir) + ft is a symmetry operation
   !
-  return 
-end function checksym
+  RETURN
+END FUNCTION checksym
 !
 !-----------------------------------------------------------------------
-subroutine checkallsym ( nat, tau, ityp, nr1, nr2, nr3 ) 
+SUBROUTINE checkallsym ( nat, tau, ityp, nr1, nr2, nr3 )
   !-----------------------------------------------------------------------
   !     given a crystal group this routine checks that the actual
   !     atomic positions and bravais lattice vectors are compatible with
   !     it. Used in relaxation/MD runs to check that atomic motion is
   !     consistent with assumed symmetry.
   !
-  implicit none
+  IMPLICIT NONE
   !
-  integer, intent(in) :: nat, ityp (nat), nr1, nr2, nr3
-  real(DP), intent(in) :: tau (3, nat)
+  INTEGER, INTENT(in) :: nat, ityp (nat), nr1, nr2, nr3
+  real(DP), INTENT(in) :: tau (3, nat)
   !
-  integer :: na, kpol, isym, i, j, k, l
-  logical :: loksym (48)
+  INTEGER :: na, kpol, isym, i, j, k, l
+  LOGICAL :: loksym (48)
   real(DP) :: sx (3, 3), sy(3,3)
-  real(DP) , allocatable :: xau(:,:), rau(:,:)
+  real(DP) , ALLOCATABLE :: xau(:,:), rau(:,:)
   !
-  allocate (xau( 3 , nat))    
-  allocate (rau( 3 , nat))    
+  ALLOCATE (xau( 3 , nat))
+  ALLOCATE (rau( 3 , nat))
   !
   !     check that s(i,j, isym) is an orthogonal operation
   !
-  do isym = 1, nsym
-     sx = DBLE( s(:,:,isym) )
+  DO isym = 1, nsym
+     sx = dble( s(:,:,isym) )
      sy = matmul ( bg, sx )
      sx = matmul ( sy, transpose(at) )
      ! sx is s in cartesian axis
      sy = matmul ( transpose ( sx ), sx )
      ! sy = s*transpose(s) = I
-     do i = 1, 3
+     DO i = 1, 3
         sy (i,i) = sy (i,i) - 1.0_dp
-     end do
-     if (any (abs (sy) > eps1 ) ) &
-         call errore ('checkallsym', 'not orthogonal operation', isym)
-  enddo
+     ENDDO
+     IF (any (abs (sy) > eps1 ) ) &
+         CALL errore ('checkallsym', 'not orthogonal operation', isym)
+  ENDDO
   !
   !     Compute the coordinates of each atom in the basis of the lattice
   !
-  do na = 1, nat
-     do kpol = 1, 3
+  DO na = 1, nat
+     DO kpol = 1, 3
         xau (kpol, na) = bg (1, kpol) * tau (1, na) + &
                          bg (2, kpol) * tau (2, na) + &
                          bg (3, kpol) * tau (3, na)
-     enddo
-  enddo
+     ENDDO
+  ENDDO
   !
   !     generate the coordinates of the rotated atoms
   !
-  do isym = 1, nsym
-     do na = 1, nat
-        do kpol = 1, 3
+  DO isym = 1, nsym
+     DO na = 1, nat
+        DO kpol = 1, 3
            rau (kpol, na) = s (1, kpol, isym) * xau (1, na) + &
                             s (2, kpol, isym) * xau (2, na) + &
                             s (3, kpol, isym) * xau (3, na)
-        enddo
-     enddo
+        ENDDO
+     ENDDO
      !
      loksym(isym) =  checksym ( isym, nat, ityp, xau, rau, ft(1,isym) )
      !
-  enddo
+  ENDDO
   !
   !   deallocate work space
   !
-  deallocate(rau)
-  deallocate(xau)
+  DEALLOCATE(rau)
+  DEALLOCATE(xau)
   !
-  do isym = 1,nsym
-     if (.not.loksym (isym) ) call errore ('checkallsym', &
+  DO isym = 1,nsym
+     IF (.not.loksym (isym) ) CALL errore ('checkallsym', &
           'the following symmetry operation is not satisfied  ', -isym)
-  end do
-  if (ANY (.not.loksym (1:nsym) ) ) then
+  ENDDO
+  IF (any (.not.loksym (1:nsym) ) ) THEN
       !call symmetrize_at (nsym, s, invs, ft, irt, nat, tau, at, bg, &
       !                    alat, omega)
-      call errore ('checkallsym', &
+      CALL errore ('checkallsym', &
            'some of the original symmetry operations not satisfied ',1)
-  end if
+  ENDIF
   !
-  return
-end subroutine checkallsym
+  RETURN
+END SUBROUTINE checkallsym
 
 !----------------------------------------------------------------------
-subroutine s_axis_to_cart ( )
+SUBROUTINE s_axis_to_cart ( )
   !----------------------------------------------------------------------
   !
   !     This routine transforms symmetry matrices expressed in the
   !     basis of the crystal axis into rotations in cartesian axis
   !
   USE kinds
-  implicit none
+  IMPLICIT NONE
   !
-  integer :: isym
+  INTEGER :: isym
   real(dp):: sa(3,3), sb(3,3)
   !
-  do isym = 1,nsym
-     sa (:,:) = DBLE ( s(:,:,isym) )
-     sb = MATMUL ( bg, sa )
-     sr (:,:, isym) = MATMUL ( at, TRANSPOSE (sb) )
-  enddo
+  DO isym = 1,nsym
+     sa (:,:) = dble ( s(:,:,isym) )
+     sb = matmul ( bg, sa )
+     sr (:,:, isym) = matmul ( at, transpose (sb) )
+  ENDDO
   !
- end subroutine s_axis_to_cart
+ END SUBROUTINE s_axis_to_cart
 
 END MODULE symm_base
