@@ -78,7 +78,13 @@ for dir in $dirs; do
         # handle special cases
         sed '/@\/cineca\/prod\/hpm\/include\/f_hpm.h@/d' \
             make.depend > make.depend.tmp
-        sed '/@iso_c_binding@/d' make.depend.tmp > make.depend
+        sed '/@iso_c_binding@/d;/@ifcore@/d' make.depend.tmp > make.depend
+
+        if test "$DIR" = "Modules"
+        then
+            sed '/@mpi@/d' make.depend > make.depend.tmp
+            sed '/@elpa1@/d' make.depend.tmp > make.depend
+        fi
 
         if test "$DIR" = "clib"
         then
@@ -90,7 +96,6 @@ for dir in $dirs; do
         then
             mv make.depend make.depend.tmp
             sed '/@environ_base@/d' make.depend.tmp > make.depend
-            mv make.depend.tmp make.depend
         fi
 
         rm -f make.depend.tmp
