@@ -24,123 +24,12 @@
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_REAL1
-#if 0 <= __IOTK_MAXRANK
+#ifdef __IOTK_REAL2
+#if 3 <= __IOTK_MAXRANK
 
-! This is needed as a workaround for bugged pack 
-subroutine iotk_private_pack_REAL1(out,in,n,l)
-    use iotk_base
-    implicit none
-    integer,                                    intent(in)  :: n,l
-    REAL (kind=iotk_REAL1), intent(out) :: out(n)
-    REAL (kind=iotk_REAL1), intent(in)  :: in(n)
-    out = in
-end subroutine iotk_private_pack_REAL1
 
-subroutine iotk_write_REAL1(val,string,fmt,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_xtox_interf
-  use iotk_fmt_interf
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  REAL(kind=iotk_REAL1), intent(in) :: val(:)
-#ifdef __IOTK_WORKAROUND6
-  character(len=*)              :: string
-#else
-  character(len=*), intent(out) :: string
-#endif
-  character(len=*), intent(in)  :: fmt
-  integer, intent(out) :: ierr
-  character(len=100) :: tmpval
-  integer :: index,iostat
-  ierr = 0
-  iostat = 0 
-  string(1:1) = iotk_eos
-  if(size(val)==0) return
-  if(len(string)==0) then
-    call iotk_error_issue(ierr,"iotk_write",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.21 ")
-    return
-  end if
-  do index=1,size(val)
-    if (trim(fmt)=="!") then
-       write(tmpval,trim(iotk_wfmt("REAL",kind(val),1,-1," ")),iostat=iostat) val(index)
-    else
-       write(tmpval,fmt=fmt,iostat=iostat) val(index)
-    end if
-    if(iostat/=0) then
-      call iotk_error_issue(ierr,"iotk_write",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierr,' ')
-call iotk_error_write(ierr,"iostat",iostat)
-      return
-    end if
-    call iotk_strcat(string,trim(adjustl(tmpval))//" ",ierr)
-    if(ierr/=0) then
-      call iotk_error_issue(ierr,"iotk_write",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.21 ")
-      return
-    end if
-  end do
-! the last blank is deleted
-  string(iotk_strlen(string):iotk_strlen(string)) = iotk_eos
-end subroutine iotk_write_REAL1
 
-subroutine iotk_read_REAL1(val,string,index,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_xtox_interf
-  use iotk_misc_interf
-  implicit none
-  REAL(kind=iotk_REAL1), intent(inout) :: val(:)
-  character(len=*), intent(in) :: string
-  integer, intent(inout) :: index
-  integer, intent(out) :: ierr
-  integer :: pos,pos1,iostat
-  integer :: maxindex
-#ifdef __IOTK_WORKAROUND9
-  character(len=100) :: tmpstr ! debug
-#endif
-  pos = 0
-  pos1= 0
-  ierr = 0
-  iostat = 0
-    maxindex = size(val)
-! for the moment, commas are considered as blanks
-  do
-    pos = verify(string(pos1+1:)," ,")+pos1
-    if(pos==pos1) exit
-    pos = pos - 1
-    pos1 = scan(string(pos+1:)," ,")+pos
-    if(pos1==pos) pos1 = len(string) + 1
-!READ string(pos+1:pos1-1)
-    index = index+1
-    if(index>maxindex) then
-      call iotk_error_issue(ierr,"iotk_read",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierr,'Too many data')
-    end if
-#ifdef __IOTK_WORKAROUND9
-    tmpstr = TRIM( string(pos+1:pos1-1) )
-    read( tmpstr, "(G100.95)",iostat=iostat) val(index)
-#else
-    read( string(pos+1:pos1-1), "(G100.95)",iostat=iostat) val(index)
-#endif
-    if(iostat/=0) then
-      call iotk_error_issue(ierr,"iotk_read",__FILE__,__LINE__)
-call iotk_error_msg(ierr,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierr,'Error reading a REAL number from string')
-call iotk_error_write(ierr,"string",string(pos+1:pos1-1))
-call iotk_error_write(ierr,"iostat",iostat)
-      return
-    end if
-    if(pos1>=len(string)) exit
-  end do
-end subroutine iotk_read_REAL1
-
-subroutine iotk_write_attr_REAL1_0(attr,name,val,dummy,first,newline,fmt,ierr)
+subroutine iotk_write_attr_REAL2_3(attr,name,val,dummy,first,newline,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf
@@ -149,222 +38,7 @@ subroutine iotk_write_attr_REAL1_0(attr,name,val,dummy,first,newline,fmt,ierr)
   implicit none
   character(*), intent(inout) :: attr
   character(*), intent(in)    :: name
-  REAL(kind=iotk_REAL1), intent(in)  :: val 
-  type(iotk_dummytype), optional :: dummy
-  logical, optional, intent(in)  :: first
-  logical, optional, intent(in)  :: newline
-  character(*), optional, intent(in) :: fmt
-  integer, optional, intent(out) :: ierr
-  integer :: ierrl
-  integer :: attlen
-  integer :: vallen
-  integer :: namlen
-  character :: delim
-  character(len=300) :: usefmt
-  character(iotk_vallenx) :: tmpval
-  logical :: nl
-  if(present(newline)) then
-    nl = newline
-  else
-    nl = .false.
-  endif
-!-<
-  if (present(fmt)) then
-    usefmt = fmt
-  else
-    usefmt = "!"
-  end if
-!->
-  ierrl = 0
-  if(present(first)) then
-    if(first) attr(1:1) = iotk_eos
-  end if
-  attlen = iotk_strlen_trim(attr)
-  namlen = iotk_strlen_trim(name)
-  if(.not.iotk_check_name(name)) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Wrong tag name')
-call iotk_error_write(ierrl,"name",name(1:namlen))
-    goto 1
-  end if
-  delim = '"'
-  call iotk_write((/val/),tmpval,usefmt,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-    goto 1
-  end if
-  vallen = iotk_strlen(tmpval)
-  if(attlen+vallen+namlen+5>len(attr)) then
-    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
-    goto 1
-  end if
-  if(.not. nl) then
-    attr(attlen+1:attlen+vallen+namlen+5) = " "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
-  else
-    attr(attlen+1:attlen+vallen+namlen+len(iotk_newline)+5) &
-       = iotk_newline//" "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
-  endif
-1 continue
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl/=0) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_write_attr_REAL1_0
-
-subroutine iotk_scan_attr_REAL1_0(attr,name,val,dummy,found,default,eos,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_attr_interf, only : iotk_read
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  character(*),             intent(in)  :: attr
-  character(*),             intent(in)  :: name
-#ifdef __IOTK_WORKAROUND6
-  REAL(kind=iotk_REAL1)                        :: val 
-#else
-  REAL(kind=iotk_REAL1), intent(out)           :: val 
-#endif
-  type(iotk_dummytype), optional :: dummy
-  logical,        optional, intent(out) :: found
-  REAL(kind=iotk_REAL1), optional, intent(in)  :: default 
-  logical,        optional, intent(in)  :: eos
-  integer,        optional, intent(out) :: ierr
-  integer :: ierrl
-  integer :: attlen,pos,equal,namlen
-  character :: delim
-  logical :: foundl
-  character(iotk_vallenx) :: valc
-  integer :: index
-  REAL(kind=iotk_REAL1), allocatable :: tmpval (:)
-  ierrl = 0
-  attlen=iotk_strlen(attr)
-  namlen=iotk_strlen_trim(name)
-  foundl = .false.
-  equal = 0
-  do
-    pos = verify(attr(equal+1:attlen)," ")
-    if(pos<=0) exit
-    equal = equal + pos
-    pos = scan(attr(equal+1:attlen),"=")
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'')
-call iotk_error_write(ierrl,"attr",attr(equal+1:attlen))
-      goto 1
-    end if
-    equal = equal + pos
-    if(trim(attr(equal-pos:equal-1))==name(1:namlen)) foundl = .true.
-    pos = verify(attr(equal+1:attlen)," ")
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    equal = equal + pos
-    delim = attr(equal:equal)
-    if(delim/="'" .and. delim/='"') then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    pos = scan(attr(equal+1:attlen),delim)
-    if(pos<=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-    if(foundl) exit
-    equal = equal + pos
-  end do
-  if(foundl) then
-    call iotk_strcpy(valc,attr(equal+1:equal+pos-1),ierrl)
-    if(ierrl/=0) then
-      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-      goto 1
-    end if
-  else
-    goto 1
-  end if
-  allocate(tmpval(1))
-  index = 0
-  call iotk_str_clean(valc)
-  call iotk_read(tmpval,valc(1:iotk_strlen(valc)),index,ierrl)
-  if(ierrl/=0) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-    goto 1
-  end if
-  if(index/=1) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute size does not match')
-call iotk_error_write(ierrl,"attr",valc)
-call iotk_error_write(ierrl,"size",size(tmpval))
-    goto 1
-  end if
-  val = tmpval(1)
-  deallocate(tmpval)
-1 continue
-  if(ierrl/=0) foundl=.false.
-  if(present(found)) found = foundl
-  if(ierrl==0 .and. .not. present(found) .and. .not. present(default) .and. .not. foundl) then
-    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
-call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
-call iotk_error_msg(ierrl,'Attribute not found')
-call iotk_error_write(ierrl,"name",name)
-    ierrl = - ierrl
-  end if
-  if(present(default) .and. .not. foundl) then
-    val = default
-  end if
-  if(present(ierr)) then
-    ierr = ierrl
-  else
-    if(ierrl/=0) call iotk_error_handler(ierrl)
-  end if
-end subroutine iotk_scan_attr_REAL1_0
-
-#endif
-#endif
-
-subroutine iotk_attr_dummy_REAL1_0
-  write(0,*)
-end subroutine iotk_attr_dummy_REAL1_0
-
-
-
-
-!------------------------------------------------------------------------------!
-! Inclusion of configuration file
-#include "iotk_config.h"
-!------------------------------------------------------------------------------!
-
-#include "iotk_auxmacros.h"
-
-
-#ifdef __IOTK_REAL1
-#if 1 <= __IOTK_MAXRANK
-
-
-
-subroutine iotk_write_attr_REAL1_1(attr,name,val,dummy,first,newline,fmt,ierr)
-  use iotk_base
-  use iotk_error_interf
-  use iotk_attr_interf
-  use iotk_str_interf
-  use iotk_misc_interf
-  implicit none
-  character(*), intent(inout) :: attr
-  character(*), intent(in)    :: name
-  REAL(kind=iotk_REAL1), intent(in)  :: val (:)
+  REAL(kind=iotk_REAL2), intent(in)  :: val (:,:,:)
   type(iotk_dummytype), optional :: dummy
   logical, optional, intent(in)  :: first
   logical, optional, intent(in)  :: newline
@@ -429,9 +103,9 @@ call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_attr_REAL1_1
+end subroutine iotk_write_attr_REAL2_3
 
-subroutine iotk_scan_attr_REAL1_1(attr,name,val,dummy,found,default,eos,ierr)
+subroutine iotk_scan_attr_REAL2_3(attr,name,val,dummy,found,default,eos,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_read
@@ -441,13 +115,13 @@ subroutine iotk_scan_attr_REAL1_1(attr,name,val,dummy,found,default,eos,ierr)
   character(*),             intent(in)  :: attr
   character(*),             intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  REAL(kind=iotk_REAL1)                        :: val (:)
+  REAL(kind=iotk_REAL2)                        :: val (:,:,:)
 #else
-  REAL(kind=iotk_REAL1), intent(out)           :: val (:)
+  REAL(kind=iotk_REAL2), intent(out)           :: val (:,:,:)
 #endif
   type(iotk_dummytype), optional :: dummy
   logical,        optional, intent(out) :: found
-  REAL(kind=iotk_REAL1), optional, intent(in)  :: default (:)
+  REAL(kind=iotk_REAL2), optional, intent(in)  :: default (:,:,:)
   logical,        optional, intent(in)  :: eos
   integer,        optional, intent(out) :: ierr
   integer :: ierrl
@@ -456,7 +130,7 @@ subroutine iotk_scan_attr_REAL1_1(attr,name,val,dummy,found,default,eos,ierr)
   logical :: foundl
   character(iotk_vallenx) :: valc
   integer :: index
-  REAL(kind=iotk_REAL1), allocatable :: tmpval (:)
+  REAL(kind=iotk_REAL2), allocatable :: tmpval (:)
   ierrl = 0
   attlen=iotk_strlen(attr)
   namlen=iotk_strlen_trim(name)
@@ -545,14 +219,14 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_attr_REAL1_1
+end subroutine iotk_scan_attr_REAL2_3
 
 #endif
 #endif
 
-subroutine iotk_attr_dummy_REAL1_1
+subroutine iotk_attr_dummy_REAL2_3
   write(0,*)
-end subroutine iotk_attr_dummy_REAL1_1
+end subroutine iotk_attr_dummy_REAL2_3
 
 
 
@@ -565,12 +239,12 @@ end subroutine iotk_attr_dummy_REAL1_1
 #include "iotk_auxmacros.h"
 
 
-#ifdef __IOTK_REAL1
-#if 2 <= __IOTK_MAXRANK
+#ifdef __IOTK_REAL2
+#if 4 <= __IOTK_MAXRANK
 
 
 
-subroutine iotk_write_attr_REAL1_2(attr,name,val,dummy,first,newline,fmt,ierr)
+subroutine iotk_write_attr_REAL2_4(attr,name,val,dummy,first,newline,fmt,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf
@@ -579,7 +253,7 @@ subroutine iotk_write_attr_REAL1_2(attr,name,val,dummy,first,newline,fmt,ierr)
   implicit none
   character(*), intent(inout) :: attr
   character(*), intent(in)    :: name
-  REAL(kind=iotk_REAL1), intent(in)  :: val (:,:)
+  REAL(kind=iotk_REAL2), intent(in)  :: val (:,:,:,:)
   type(iotk_dummytype), optional :: dummy
   logical, optional, intent(in)  :: first
   logical, optional, intent(in)  :: newline
@@ -644,9 +318,9 @@ call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_write_attr_REAL1_2
+end subroutine iotk_write_attr_REAL2_4
 
-subroutine iotk_scan_attr_REAL1_2(attr,name,val,dummy,found,default,eos,ierr)
+subroutine iotk_scan_attr_REAL2_4(attr,name,val,dummy,found,default,eos,ierr)
   use iotk_base
   use iotk_error_interf
   use iotk_attr_interf, only : iotk_read
@@ -656,13 +330,13 @@ subroutine iotk_scan_attr_REAL1_2(attr,name,val,dummy,found,default,eos,ierr)
   character(*),             intent(in)  :: attr
   character(*),             intent(in)  :: name
 #ifdef __IOTK_WORKAROUND6
-  REAL(kind=iotk_REAL1)                        :: val (:,:)
+  REAL(kind=iotk_REAL2)                        :: val (:,:,:,:)
 #else
-  REAL(kind=iotk_REAL1), intent(out)           :: val (:,:)
+  REAL(kind=iotk_REAL2), intent(out)           :: val (:,:,:,:)
 #endif
   type(iotk_dummytype), optional :: dummy
   logical,        optional, intent(out) :: found
-  REAL(kind=iotk_REAL1), optional, intent(in)  :: default (:,:)
+  REAL(kind=iotk_REAL2), optional, intent(in)  :: default (:,:,:,:)
   logical,        optional, intent(in)  :: eos
   integer,        optional, intent(out) :: ierr
   integer :: ierrl
@@ -671,7 +345,7 @@ subroutine iotk_scan_attr_REAL1_2(attr,name,val,dummy,found,default,eos,ierr)
   logical :: foundl
   character(iotk_vallenx) :: valc
   integer :: index
-  REAL(kind=iotk_REAL1), allocatable :: tmpval (:)
+  REAL(kind=iotk_REAL2), allocatable :: tmpval (:)
   ierrl = 0
   attlen=iotk_strlen(attr)
   namlen=iotk_strlen_trim(name)
@@ -760,13 +434,228 @@ call iotk_error_write(ierrl,"name",name)
   else
     if(ierrl/=0) call iotk_error_handler(ierrl)
   end if
-end subroutine iotk_scan_attr_REAL1_2
+end subroutine iotk_scan_attr_REAL2_4
 
 #endif
 #endif
 
-subroutine iotk_attr_dummy_REAL1_2
+subroutine iotk_attr_dummy_REAL2_4
   write(0,*)
-end subroutine iotk_attr_dummy_REAL1_2
+end subroutine iotk_attr_dummy_REAL2_4
+
+
+
+
+!------------------------------------------------------------------------------!
+! Inclusion of configuration file
+#include "iotk_config.h"
+!------------------------------------------------------------------------------!
+
+#include "iotk_auxmacros.h"
+
+
+#ifdef __IOTK_REAL2
+#if 5 <= __IOTK_MAXRANK
+
+
+
+subroutine iotk_write_attr_REAL2_5(attr,name,val,dummy,first,newline,fmt,ierr)
+  use iotk_base
+  use iotk_error_interf
+  use iotk_attr_interf
+  use iotk_str_interf
+  use iotk_misc_interf
+  implicit none
+  character(*), intent(inout) :: attr
+  character(*), intent(in)    :: name
+  REAL(kind=iotk_REAL2), intent(in)  :: val (:,:,:,:,:)
+  type(iotk_dummytype), optional :: dummy
+  logical, optional, intent(in)  :: first
+  logical, optional, intent(in)  :: newline
+  character(*), optional, intent(in) :: fmt
+  integer, optional, intent(out) :: ierr
+  integer :: ierrl
+  integer :: attlen
+  integer :: vallen
+  integer :: namlen
+  character :: delim
+  character(len=300) :: usefmt
+  character(iotk_vallenx) :: tmpval
+  logical :: nl
+  if(present(newline)) then
+    nl = newline
+  else
+    nl = .false.
+  endif
+!-<
+  if (present(fmt)) then
+    usefmt = fmt
+  else
+    usefmt = "!"
+  end if
+!->
+  ierrl = 0
+  if(present(first)) then
+    if(first) attr(1:1) = iotk_eos
+  end if
+  attlen = iotk_strlen_trim(attr)
+  namlen = iotk_strlen_trim(name)
+  if(.not.iotk_check_name(name)) then
+    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+call iotk_error_msg(ierrl,'Wrong tag name')
+call iotk_error_write(ierrl,"name",name(1:namlen))
+    goto 1
+  end if
+  delim = '"'
+  call iotk_write(pack(val,mask=.true.),tmpval,usefmt,ierrl)
+  if(ierrl/=0) then
+    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+    goto 1
+  end if
+  vallen = iotk_strlen(tmpval)
+  if(attlen+vallen+namlen+5>len(attr)) then
+    call iotk_error_issue(ierrl,"iotk_write_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+call iotk_error_msg(ierrl,'Attribute dummy argument is too short')
+    goto 1
+  end if
+  if(.not. nl) then
+    attr(attlen+1:attlen+vallen+namlen+5) = " "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
+  else
+    attr(attlen+1:attlen+vallen+namlen+len(iotk_newline)+5) &
+       = iotk_newline//" "//name(1:namlen)//"="//delim//tmpval(1:vallen)//delim//iotk_eos
+  endif
+1 continue
+  if(present(ierr)) then
+    ierr = ierrl
+  else
+    if(ierrl/=0) call iotk_error_handler(ierrl)
+  end if
+end subroutine iotk_write_attr_REAL2_5
+
+subroutine iotk_scan_attr_REAL2_5(attr,name,val,dummy,found,default,eos,ierr)
+  use iotk_base
+  use iotk_error_interf
+  use iotk_attr_interf, only : iotk_read
+  use iotk_str_interf
+  use iotk_misc_interf
+  implicit none
+  character(*),             intent(in)  :: attr
+  character(*),             intent(in)  :: name
+#ifdef __IOTK_WORKAROUND6
+  REAL(kind=iotk_REAL2)                        :: val (:,:,:,:,:)
+#else
+  REAL(kind=iotk_REAL2), intent(out)           :: val (:,:,:,:,:)
+#endif
+  type(iotk_dummytype), optional :: dummy
+  logical,        optional, intent(out) :: found
+  REAL(kind=iotk_REAL2), optional, intent(in)  :: default (:,:,:,:,:)
+  logical,        optional, intent(in)  :: eos
+  integer,        optional, intent(out) :: ierr
+  integer :: ierrl
+  integer :: attlen,pos,equal,namlen
+  character :: delim
+  logical :: foundl
+  character(iotk_vallenx) :: valc
+  integer :: index
+  REAL(kind=iotk_REAL2), allocatable :: tmpval (:)
+  ierrl = 0
+  attlen=iotk_strlen(attr)
+  namlen=iotk_strlen_trim(name)
+  foundl = .false.
+  equal = 0
+  do
+    pos = verify(attr(equal+1:attlen)," ")
+    if(pos<=0) exit
+    equal = equal + pos
+    pos = scan(attr(equal+1:attlen),"=")
+    if(pos<=0) then
+      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+call iotk_error_msg(ierrl,'')
+call iotk_error_write(ierrl,"attr",attr(equal+1:attlen))
+      goto 1
+    end if
+    equal = equal + pos
+    if(trim(attr(equal-pos:equal-1))==name(1:namlen)) foundl = .true.
+    pos = verify(attr(equal+1:attlen)," ")
+    if(pos<=0) then
+      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+      goto 1
+    end if
+    equal = equal + pos
+    delim = attr(equal:equal)
+    if(delim/="'" .and. delim/='"') then
+      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+      goto 1
+    end if
+    pos = scan(attr(equal+1:attlen),delim)
+    if(pos<=0) then
+      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+      goto 1
+    end if
+    if(foundl) exit
+    equal = equal + pos
+  end do
+  if(foundl) then
+    call iotk_strcpy(valc,attr(equal+1:equal+pos-1),ierrl)
+    if(ierrl/=0) then
+      call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+      goto 1
+    end if
+  else
+    goto 1
+  end if
+  allocate(tmpval(size(val)))
+  index = 0
+  call iotk_str_clean(valc)
+  call iotk_read(tmpval,valc(1:iotk_strlen(valc)),index,ierrl)
+  if(ierrl/=0) then
+    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+    goto 1
+  end if
+  if(index/=size(val)) then
+    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+call iotk_error_msg(ierrl,'Attribute size does not match')
+call iotk_error_write(ierrl,"attr",valc)
+call iotk_error_write(ierrl,"size",size(tmpval))
+    goto 1
+  end if
+  val = reshape (source=tmpval,shape=shape(val))
+  deallocate(tmpval)
+1 continue
+  if(ierrl/=0) foundl=.false.
+  if(present(found)) found = foundl
+  if(ierrl==0 .and. .not. present(found) .and. .not. present(default) .and. .not. foundl) then
+    call iotk_error_issue(ierrl,"iotk_scan_attr",__FILE__,__LINE__)
+call iotk_error_msg(ierrl,"CVS Revision: 1.21 ")
+call iotk_error_msg(ierrl,'Attribute not found')
+call iotk_error_write(ierrl,"name",name)
+    ierrl = - ierrl
+  end if
+  if(present(default) .and. .not. foundl) then
+    val = default
+  end if
+  if(present(ierr)) then
+    ierr = ierrl
+  else
+    if(ierrl/=0) call iotk_error_handler(ierrl)
+  end if
+end subroutine iotk_scan_attr_REAL2_5
+
+#endif
+#endif
+
+subroutine iotk_attr_dummy_REAL2_5
+  write(0,*)
+end subroutine iotk_attr_dummy_REAL2_5
 
 
